/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                               DEMO PROGRAMS SET
                          xml_test3.cpp  -  description
                             -------------------
    begin                : August 20, 2007
    copyright            : (C) 2003-2007 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef __BORLANDC__
#include <vcl.h>
#pragma hdrstop
#endif

#include <sptk3/sptk.h>
#include <sptk3/cxml>
#include <iostream>
#include <sstream>
#include <stdexcept>
#include <iomanip>

using namespace std;
using namespace sptk;

void testXPath(string fileName,string xpath,int expectedNodeCount=-1) {
    CXmlDoc doc;
    CBuffer buf;

    cout << "Test file: " << fileName << endl;
    cout << "XPath is : " << xpath << endl;
    cout << "Selected Nodes are:" << endl;

    buf.loadFromFile(fileName);
    doc.load(buf);

    CXmlNodeVector selectedNodes;
    doc.select(selectedNodes,xpath);
    for (CXmlNode::iterator itor = selectedNodes.begin(); itor != selectedNodes.end(); itor++) {
        CXmlNode* node = *itor;
        cout << node->name();
        if (node->hasAttribute("N"))
            cout << ", N=" << node->getAttribute("N");
        if (!node->value().empty())
            cout << ", value=" << node->value();
        cout << endl;
    }
    cout << endl;
    if (expectedNodeCount != -1 && expectedNodeCount != (int) selectedNodes.size()) {
        stringstream st;
        st << "ERROR: expected " << expectedNodeCount << " node(s) in selection, got " << selectedNodes.size() << " node(s)" << endl;
        throw runtime_error(st.str());
    }
}

int main(int argc,char *argv[]) {
    try {
        cout << "The XPath selection test started." << endl << endl;

/*
        CBuffer buf;
        buf.loadFromFile("test2.xml");
        doc.load(buf);

        CDateTime start = CDateTime::Now();
        CXmlNodeVector  nodes;
        doc.select(nodes,"//SPEAKER");
        cout << "Selected " << nodes.size() << " nodes" << endl;
        CDateTime end = CDateTime::Now();

        double duration = (end-start) * 3600 * 24;

        out << "The XPath selection test completed, duration " << setprecision(3) << fixed << duration << " seconds"  << endl;
*/
        // http://www.zvon.org/xxl/XPathTutorial/Output/example1.html
        testXPath("xml/xpath_test1.xml","/AAA",1);          // Select the root element AAA
        testXPath("xml/xpath_test1.xml","/AAA/CCC",2);      // Select all elements CCC which are children of the root element AAA
        testXPath("xml/xpath_test1.xml","/AAA/DDD/BBB",1);  // Select all elements BBB which are children of DDD which are children of the root element AAA

        // http://www.zvon.org/xxl/XPathTutorial/Output/example2.html
        testXPath("xml/xpath_test2.xml","//BBB",5);
        testXPath("xml/xpath_test2.xml","//DDD/BBB",3);

        // http://www.zvon.org/xxl/XPathTutorial/Output/example3.html
        testXPath("xml/xpath_test3.xml","/AAA/CCC/DDD/*",4);
        testXPath("xml/xpath_test3.xml","/*/*/*/BBB",5);
        testXPath("xml/xpath_test3.xml","//*",17);

        // http://www.zvon.org/xxl/XPathTutorial/Output/example4.html
        testXPath("xml/xpath_test4.xml","/AAA/BBB[1]",1);
        testXPath("xml/xpath_test4.xml","/AAA/BBB[last()]",1);

        // http://www.zvon.org/xxl/XPathTutorial/Output/example5.html
        testXPath("xml/xpath_test5.xml","//@id",2);
        testXPath("xml/xpath_test5.xml","//BBB[@id]",2);
        testXPath("xml/xpath_test5.xml","//BBB[@name]",1);
        testXPath("xml/xpath_test5.xml","//BBB[@*]",3);

        // http://www.zvon.org/xxl/XPathTutorial/Output/example6.html
        testXPath("xml/xpath_test6.xml","//BBB[@id='b1']",1);
        testXPath("xml/xpath_test6.xml","//BBB[@name='bbb']",1);

        cout << "The XPath selection test completed" << endl;

    } catch (std::exception& e) {
        cout << e.what() << endl;
        return 1;
    }
    return 0;
}
