/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBase64.cpp  -  description
                             -------------------
    begin                : July 19 2003
    copyright            : (C) 2003 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef __CBASE64_H__
#define __CBASE64_H__

#include <sptk3/CException.h>
#include <sptk3/sptk.h>
#include <sptk3/CBuffer.h>
#include <string>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief Base64 encoding/decoding.
///
/// This class is used for encoding and decoding the parts of mail messages.
/// All the methods of this class are static.
class CBase64 {
public:
    /// @brief Data encoding.
    /// Encodes data (base64) in given buffer bufSource to destination buffer bufDest.
    /// @param bufDest CBuffer Destination buffer
    /// @param bufSource CBuffer Source buffer
    static void encode(CBuffer& bufDest, const CBuffer& bufSource);

    /// @brief Data encoding.
    /// Encodes data (base64) in given buffer bufSource and returns result in std::string.
    /// @param strDest std::string Destination string
    /// @param bufSource CBuffer& Source buffer
    static void encode(std::string& strDest, const CBuffer& bufSource);

    /// @brief Data decoding.
    /// Decodes base64 encoded buffer bufSource into buffer bufDest.
    /// Throws CException in case of error
    /// @param bufDest CBuffer destination buffer
    /// @param bufSource CBuffer source buffer that holds base64 decoded data
    /// @returns length of returned buffer
    static int decode(CBuffer &bufDest, const CBuffer &bufSource) throw(CException);

    /// @brief Data decoding.
    /// Decodes base64 encoded string strSource into buffer bufDest.
    /// Throws CException in case of error
    /// @param bufDest CBuffer, destination buffer
    /// @param strSource const std::string &, source string that holds base64 decoded data
    /// @returns length of the returned string
    static int decode(CBuffer &bufDest, const std::string &strSource) throw(CException);
};
/// @}
}

#endif
