/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDirectoryDS.h  -  description
                             -------------------
    begin                : Tue Mar 26 2002
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef __CDIRECTORYDS_H__
#define __CDIRECTORYDS_H__

#include <sys/stat.h>
#include <sptk3/CMemoryDS.h>
#include <sptk3/CStrings.h>
#include <sptk3/CSmallPixmapIDs.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// Directory Show Policies
enum CDirectoryDSpolicies {
    DDS_SHOW_ALL = 0,         ///< Show everything
    DDS_HIDE_FILES = 1,       ///< Hide files
    DDS_HIDE_DOT_FILES = 2,   ///< Hide files with the name started with '.' (Unix hidden files,mostly)
    DDS_HIDE_DIRECTORIES = 4, ///< Hide directories
    DDS_NO_SORT = 8           ///< Do not sort
};

/// @brief Directory datasource
///
/// A datasource with the list of files
/// and directories along with their attributes. It works just
/// as any other datasource. You set up the parameters, call open()
/// and may use the list. Method close() should be called aftewards
/// to release any allocated resourses.
class CDirectoryDS : public CMemoryDS  {
protected:
    /// Sets up an appropriate image and a name for the file type
    /// @param st const struct stat &, the file type information
    /// @param image CSmallPixmapType&, the image type
    /// @param fname const char *, file name
    /// @returns the file type name
    std::string getFileType(const struct stat& st,CSmallPixmapType& image,const char *fname) const;

private:
    std::string    m_directory;   ///< Current directory
    CStrings       m_pattern;     ///< Current file pattern
    int            m_showPolicy;  ///< Show policy, see CDirectoryDSpolicies for more information

public:
    /// Default Constructor
    CDirectoryDS() : CMemoryDS(), m_showPolicy(0) {}

    /// Destructor
    virtual ~CDirectoryDS() {
        close();
    }

    /// Returns current show policy, @see CDirectoryDSpolicies for more information
    /// @returns current show policy
    int showPolicy() const {
        return m_showPolicy;
    }

    /// Sets current show policy, see CDirectoryDSpolicies for more information
    void showPolicy(int type) {
        m_showPolicy = type;
    }

    /// Returns current directory
    void directory(std::string d);

    /// Sets current directory
    const std::string &directory() const {
        return m_directory;
    }

    /// Sets pattern in format like: "*.txt;*.csv;*.xls"
    void pattern(std::string pattern) {
        m_pattern.fromString(pattern,";");
    }

    /// Returns pattern in format like: "*.txt;*.csv;*.xls"
    const std::string pattern() const {
        return m_pattern.asString(";");
    }

    /// Opens the directory and fills in the dataset
    virtual bool open();
};
/// @}
}
#endif
