/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cexception.h  -  description
                             -------------------
    begin                : Thu Apr 27 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __EXCEPTION_H__
#define __EXCEPTION_H__

#include <sptk3/sptk.h>
#include <string>
#include <stdexcept>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief SPTK exception class.
///
/// Contains information about what
/// happened and where. It's based on std::exception, so if you
/// just want to catch STL and SPTK exceptions - you can use
/// try {} catch (std::exception& e) {} block.
class SP_EXPORT CException : public std::exception {
    std::string m_file;         ///< The file where exception occurs
    int         m_line;         ///< The line number in the file where exception occurs
    std::string m_text;         ///< The exception text
    std::string m_description;  ///< The extended error information
    std::string m_fullMessage;  ///< The complete error information combining everything together
public:
    /// Constructor
    /// @param text std::string, the exception text
    /// @param file std::string, the file where exception occurs
    /// @param line int, the line number in the file where exception occurs
    /// @param description std::string, the optional description information
    CException(std::string text,std::string file="",int line=0,std::string description="");

    /// Destructor
    virtual ~CException() throw() {}

    /// Returns complete text of exception
    virtual const char * what() const throw() {
        return m_fullMessage.c_str();
    }

    /// Returns exception message without file name, line number, or description
    std::string message() const {
        return m_text;
    }

    /// Returns exception file name
    std::string file() const {
        return m_file;
    }

    /// Returns exception line number
    int line() const {
        return m_line;
    }

    /// Returns exception description
    std::string description() const {
        return m_description;
    }
};

/// Defines a handy macros that automatically registers filename and line number
/// for the place an exception is thrown from
#define throwException(msg) throw CException(msg,__FILE__,__LINE__)

/// @}
}
#endif
