/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CField.h  -  description
                             -------------------
    begin                : Wed Dec 15 1999
    copyright            : (C) 1999-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CFIELDS_H__
#define __CFIELDS_H__

#include <sptk3/CBuffer.h>
#include <sptk3/CDateTime.h>
#include <sptk3/CVariant.h>
#include <sptk3/cxml>

#include <string>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

class CQuery;

#include <sptk3/CVariant.h>

class CFieldList;

/// @brief Data field for CDataSource.
///
/// Contains field name, field type, field data and field format information.
class SP_EXPORT CField : public CVariant {
    friend class CFieldList;
protected:
    std::string       m_name;     ///< Field name

public:
    std::string       displayName;///< Optional display field name
public:
    /// @brief Constructor
    /// @param name const char *, field name
    CField(const char *name);

    int32_t           width;      ///< Field width
    int32_t           precision;  ///< Field precision
    int32_t           flags;      ///< Field flags like alignment, etc
    bool              visible;    ///< Is field visible?

    /// Returns field name
    const std::string& fieldName() const {
        return m_name;
    }

    /// Assignment operation
    CField& operator = (const CVariant &C)    {
        if (this == &C)
            return *this;
        setData(C);
        return *this;
    };

    /// Assignment operation
    CField& operator = (int32_t value)            {
        setInteger(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (uint32_t value)       {
        setInteger(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (int16_t value)          {
        setInteger(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (uint16_t value) {
        setInteger(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (float value)          {
        setFloat(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (double value)         {
        setFloat(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (const char * value)   {
        setString(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (const std::string& value) {
        setString(value.c_str(),(uint32_t)value.length());
        return *this;
    }

    /// Assignment operation
    CField& operator = (CDateTime value)      {
        setDateTime(value);
        return *this;
    }

    /// Assignment operation
    CField& operator = (const void *value)    {
        setImagePtr(value);
        return *this;
    }

    /// Better (than in base class) conversion method
    virtual std::string asString() const throw(CException);

    /// @brief Exports the field data into XML node
    ///
    /// If the compactXmlMode flag is true, the field is exported as an attribute.
    /// Otherwise, the field is exported as subnodes.
    /// For the fields of the VAR_TEXT type, the subnode is created containing CDATA section.
    /// @param node CXmlNode&, a node to export field data into
    /// @param compactXmlMode bool, compact XML mode flag
    void toXML(CXmlNode& node,bool compactXmlMode) const;
};
/// @}
}
#endif
