/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFileLog.h  -  description
                             -------------------
    begin                : Tue Jan 31 2006
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 
    This module creation was sponsored by Total Knowledge (http://www.total-knowledge.com).
    Author thanks the developers of CPPSERV project (http://www.total-knowledge.com/progs/cppserv)
    for defining the requirements for this class.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CFILELOG_H__
#define __CFILELOG_H__

#include <sptk3/CBaseLog.h>
#include <fstream>

namespace sptk {

/// @addtogroup log Log Classes
/// @{

/// @brief A log stored in the regular file.
///
/// A simplest possible way to implement logging.
/// The log file is created automatically if it doesn't exist.
/// @see CBaseLog for more information about basic log abilities.
class CFileLog : public CBaseLog {
    std::ofstream   m_fileStream;   ///< Log file stream
    std::string     m_fileName;      ///< Log file name
protected:

public:
    /// @brief Stores or sends log message to actual destination
    /// @param date CDateTime, message timestamp
    /// @param message const char *, message text
    /// @param sz uint32_t, message size
    /// @param priority CLogPriority, message priority. @see CLogPriority for more information.
    virtual void saveMessage(CDateTime date,const char *message,uint32_t sz,const CLogPriority *priority) throw(CException);

public:
    /// @brief Constructor
    ///
    /// Creates a new log object based on the file name.
    /// If this file doesn't exist - it will be created.
    /// @param fileName string, log file name
    CFileLog(std::string fileName) : m_fileName(fileName) {}

    /// @brief Destructor
    ///
    /// Destructs the log object, closes the log file, releases all the allocated resources
    virtual ~CFileLog();

    /// @brief Restarts the log
    ///
    /// The current log content is cleared. The file is recreated.
    virtual void reset() throw(CException);

    /// @brief Returns log file name
    std::string fileName() const {
        return m_fileName;
    }
};
/// @}
}

#endif
