/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CGuard.h  -  description
                             -------------------
    begin                : Tue Apr 10 2007
    copyright            : (C) 2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef __CGUARD_H__
#define __CGUARD_H__

#include <sptk3/sptk.h>
#include <sptk3/CWaiter.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief Guardian object
///
/// Automatically locks the lock on the declaration and unlocks that lock when goes out of scope.
/// That is needed for thread safety.
class SP_EXPORT CGuard {

    CWaiter& m_object;    ///< Controlled lock

public:
    /// @brief Constructor
    ///
    /// Automatically locks the lock object. That is needed for thread safety.
    /// @param object CSQLite3Database&, SQLite3 object to lock.
    CGuard(CWaiter& object) : m_object(object) {
        m_object.lock();
    }

    /// @brief Destructor
    ///
    /// Unlocks the lock object defined in constructor.
    ~CGuard() {
        m_object.unlock();
    }
};

/// @}
}

#endif
