/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CHttpConnect.h  -  description
                             -------------------
    begin                : July 22 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CHTTPCONNECT_H__
#define __CHTTPCONNECT_H__

#include <sptk3/CSocket.h>
#include <sptk3/CStrings.h>
#include <sptk3/CHttpParams.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief A map of HTTP headers and their values (string to string)
typedef std::map<std::string,std::string> CHttpHeaders;

/// @brief HTTP socket
///
/// Implements the GET and POST methods of HTTP protocol.
/// Allows to define the host, port, submit information, and then GET or POST the HTML data to the server.
class CHttpConnect : public CSocket {
    CBuffer         m_readBuffer;           ///< Internal read buffer

protected:
    CHttpHeaders    m_requestHeaders;       ///< HTTP request headers
    CHttpHeaders    m_responseHeaders;      ///< HTTP response headers

protected:
    /// @brief Sends a single command to HTTP server
    ///
    /// CRLF characters are automatically appended to the command.
    /// @param cmd std::string, HTTP command
    void sendCommand(std::string cmd);

public:

    /// @brief Default constructor
    CHttpConnect();

    /// @brief Destructor
    ~CHttpConnect();

    /// @brief Retrieves the server response on the command
    ///
    /// Stops when HTTP server closes the connection. The server response can then be
    /// accessed through the htmlData() method.
    void getResponse();

    /// @brief Returns the internal read buffer
    ///
    /// The buffer makes sense only after sending a command to the server (if that command calls
    /// getResponse() method internally). The buffer doesn't contain HTTP headers that are parsed
    /// int m_headers map.
    /// @returns internal read buffer reference
    const CBuffer& htmlData() const {
        return m_readBuffer;
    }

    /// @brief Returns the HTTP request headers
    ///
    /// The HTTP headers request should be set before sending a command to the server
    /// @returns internal http request headers reference
    CHttpHeaders& requestHeaders() {
        return m_requestHeaders;
    }

    /// @brief Returns the HTTP headers
    ///
    /// The HTTP response headers make sense only after sending a command to the server (if that command calls
    /// getResponse() method internally).
    /// @returns internal http headers reference
    const CHttpHeaders& responseHeaders() const {
        return m_responseHeaders;
    }

    /// @brief Sends the GET command to the server
    ///
    /// Retrieves the server response into internal read buffer.
    /// @param pageName std::string, the name of the page without the server name.
    /// @param postData CHttpParams&, the list of HTTP data to pass to the server
    void cmd_get(std::string pageName,CHttpParams& postData);

    /// @brief Sends the POST command to the server
    ///
    /// Retrieves the server response into internal read buffer.
    /// @param pageName std::string, the name of the page without the server name.
    /// @param postData CHttpParams&, the list of HTTP data to pass to the server
    void cmd_post(std::string pageName,CHttpParams& postData);
};
/// @}
}
#endif
