/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CImapDS.h - description
                             -------------------
    begin                : Mar 19 2003
    copyright            : (C) 2003 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CIMAPDS_H__
#define __CIMAPDS_H__

#include <sptk3/CMemoryDS.h>
#include <sptk3/CImapConnect.h>
#include <string>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// Progression callback function prototype
typedef void (*CProgressCallback)(int total,int progress);

/// @brief IMAP datasource
///
/// Allows to browse the list of messages and folders on IMAP server.
/// It returns a dataset with message headers.
class CImapDS : public CMemoryDS  {
protected:
    CImapConnect         m_imap;        ///< IMAP socket connector
    std::string          m_folder;      ///< IMAP folder name
    std::string          m_user;        ///< IMAP user name
    std::string          m_password;    ///< IMAP user password
    bool                 m_fetchbody;   ///< True, if we want to fetch the message headers AND message body
    CProgressCallback    m_callback;    ///< Internal prograssion callback for open()
    int                  m_msgid;       ///< Internal message ID
public:

    /// Default constructor
    CImapDS() : CMemoryDS(), m_fetchbody(false), m_callback(NULL) {
        m_msgid = 0;
    }

    /// Destructor
    virtual ~CImapDS() {
        close();
    }

    /// IMAP host name
    void host(std::string host_name)  {
        m_imap.host(host_name);
    }

    /// IMAP host name
    std::string host() const          {
        return m_imap.host();
    }

    /// IMAP user name
    void user(std::string usr)        {
        m_user = usr;
    }

    /// IMAP user name
    std::string user() const          {
        return m_user;
    }

    /// IMAP user password
    void password(std::string pwd)    {
        m_password = pwd;
    }

    /// IMAP user password
    std::string password() const      {
        return m_password;
    }

    /// IMAP folder name
    void folder(std::string d)        {
        m_folder = d;
    }

    /// IMAP folder name
    const std::string &folder() const {
        return m_folder;
    }

    /// IMAP message ID (message number in the folder). If defined,
    /// the open() will retrieve only the message with the selected ID (if any)
    void messageID(int msgid)       {
        m_msgid = msgid;
    }

    /// Returns the ID of the message when defined to retrieve one message only
    int messageID()   const         {
        return m_msgid;
    }

    /// Sets the fetch body flag. Should be called prior to open().
    /// If the fetch body flag is not set, only the message headers will be retrieved and that is much faster.
    void fetchBody(bool fb)       {
        m_fetchbody = fb;
    }

    /// Returns the current value of the fetch body flag
    /// @returns the fetch body flag
    bool fetchBody() const        {
        return m_fetchbody;
    }

    /// Opens the IMAP server connection with user name and password defined with user() and password().
    /// Scans the IMAP folder defined with folder(), than closes the IMAP server connection.
    virtual bool open();

    /// Optional callback for the open() method progression.
    /// @param cb CProgressCallback, a callback function
    void callback(CProgressCallback cb) {
        m_callback = cb;
    }
};
/// @}
}
#endif
