/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cintlist.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CINTLIST_H__
#define __CINTLIST_H__

#include <sptk3/sptk.h>
#include <vector>
#include <algorithm>

namespace sptk {

class CControl;

/// @addtogroup utility Utility Classes
/// @{

/// @brief The list of integers.
///
/// Extends the std::vector<int>
/// with CSV string encoders and decoders.
class SP_EXPORT CIntList : public std::vector<uint32_t> {
public:
    /// Constructor
    CIntList() {}

    /// Assigns another CIntList
    CIntList& operator = (const CIntList& list) {
        resize(list.size());
        std::copy(list.begin(),list.end(),begin());
        return *this;
    }

    /// Converts to string as a list of values, separated with user-defined character
    /// (comma, by default)
    std::string toString(const char * separator=",") const;

    /// Converts from string of a list of values, separated with user-defined character
    /// (comma, by default)
    void fromString(const char * s,const char * separator=",");
};
/// @}
}
#endif
