/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMailMessageBody.h  -  description
                             -------------------
    begin                : 25 Oct 2004
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CMAILMESSAGEBODY_H__
#define __CMAILMESSAGEBODY_H__

#include <sptk3/sptk.h>
#include <string>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// Defines the type of the mail message
enum CMailMessageType {
    MMT_PLAIN_TEXT_MESSAGE,        ///< Message has plain text only
    MMT_HTML_MESSAGE               ///< Message has plain text and HTML parts
};

/// @brief Mail message body text
///
/// Contains the message text as plain text, or as an HTML text and stripped HTML text (where HTML tags removed)
class CMailMessageBody {
    CMailMessageType m_type;       ///< Message type
    std::string      m_plainText;  ///< Plain text part of the message
    std::string      m_htmlText;   ///< Optional HTML part of the message

    /// Builds a plain text string from HTML text
    std::string stripHtml(const std::string& origHtml);

    /// Internal tag processing
    std::string processTag(std::string tag,std::string params);

public:
    /// @brief Default constructor
    CMailMessageBody() {
        m_type = MMT_PLAIN_TEXT_MESSAGE;
    }

    /// @brief Sets the message text.
    ///
    /// Tries to detect the HTML messages by searching HTML tag in the first 100 bytes of the message
    /// @param messageText const std::string& messageText, the text of the message
    /// @param smtp bool, special processing for smtp
    void text(const std::string& messageText,bool smtp);

    /// @brief Returns the message body type
    CMailMessageType type() {
        return m_type;
    }

    /// @brief Returns the plain text version of the message
    const std::string& text() const {
        return m_plainText;
    }

    /// @brief Returns the html version of the message (if presented)
    const std::string& html() const {
        return m_htmlText;
    }
};
/// @}
}
#endif
