/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CPackedStrings.h  -  description
                             -------------------
    begin                : Tue Mar 21 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CPACKEDSTRINGS_H__
#define __CPACKEDSTRINGS_H__

#include <sptk3/CStrings.h>
#include <sptk3/CFieldList.h>

namespace sptk {

    /// @addtogroup gui GUI Classes
    /// @{

    typedef char *          pchar;  ///< Type definition for pchar
    typedef const char *    cpchar; ///< Type definition for cpchar

    /// @brief Packed strings list
    ///
    /// Special data structure to contain several strings packed into same memory block.
    /// The idea was to minimize the memory allocation, and decrease the total required memory.
    /// Also, contains special attributes for CListView row support.
    class SP_EXPORT CPackedStrings {
            uint16_t   m_size;   ///< Number of strings
            void      *m_buffer; ///< Strings buffer
            void      *m_data;   ///< User data

        public:

            /// Constructor
            /// @param cnt int, source strings count
            /// @param strings const char *, source strings
            CPackedStrings(int cnt,const char *strings[]);

            /// Constructor
            /// @param fields CFieldList, the fields data
            /// @param keyField int, the key field number
            CPackedStrings(CFieldList& fields,int keyField);

            /// Constructor
            /// @param strings const CStrings&, source strings
            CPackedStrings(const CStrings& strings);

            /// Destructor
            ~CPackedStrings();

            /// Strings count
            uint16_t size() const { return *(uint16_t *)m_buffer; }

            /// String access by index
            const char * operator[](uint16_t index) const;

            /// Assignment operator
            CPackedStrings& operator=(const CPackedStrings&);

            /// Assignment operator
            CPackedStrings& operator=(const CStrings&);

            /// Sets user_data as void *
            void user_data(void *d) { m_data = d;           }

            /// Returns user_data as void *
            void *user_data() const { return m_data;        }

            /// Sets user_data as integer
            void argument(int32_t arg)  { m_data = (void *) arg; }

            /// Returns user_data as integer
            int32_t argument() const   { return (int32_t)(long) m_data;   }

        public:

            /// Row height for CListView
            unsigned char    height;

            /// Row flags for CListView
            unsigned char    flags;
    };
    /// @}
}
#endif
