/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CProxyLog.h  -  description
                             -------------------
    begin                : Tue Jan 11 2008
    copyright            : (C) 2008 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CProxyLog_H__
#define __CProxyLog_H__

#include <sptk3/CBaseLog.h>
#include <fstream>

namespace sptk {

/// @addtogroup log Log Classes
/// @{

/// @brief A log that sends all the log messages into another log.
///
/// The destination log is locked for a message adding period.
/// Multiple CProxyLog objects may send messages from different threads
/// into the same destination log. 
/// The log options defining message format and min priority are used 
/// from destination log.
/// @see CBaseLog for more information about basic log abilities.
class CProxyLog : public CBaseLog {
   CBaseLog&  m_destination;  /// The actual log to store messages to (destination log)

protected:

    /// @brief Sends log message to actual destination
    /// @param date CDateTime, message timestamp
    /// @param message const char *, message text
    /// @param sz uint32_t, message size
    /// @param priority CLogPriority, message priority. @see CLogPriority for more information.
    virtual void saveMessage(CDateTime date,const char *message,uint32_t sz,const CLogPriority *priority) throw(CException);

public:
    /// @brief Constructor
    ///
    /// Creates a new log object based on the file name.
    /// If this file doesn't exist - it will be created.
    /// @param destination CBaseLog&, destination log object
    CProxyLog(CBaseLog& destination) : m_destination(destination) {}

    /// @brief Restarts the log
    ///
    /// The current log content is cleared. The file is recreated.
    virtual void reset() throw(CException);

    /// @brief Returns the default priority
    ///
    /// The default priority is used for the new message,
    /// if you are not defining priority.
    virtual const CLogPriority& defaultPriority() const {
        return m_destination.defaultPriority();
    }

    /// @brief Returns the min priority
    ///
    /// Messages with priority less than requested are ignored
    virtual const CLogPriority& minPriority() const {
        return m_destination.minPriority();
    }
};
/// @}
}

#endif
