/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRWLock.h  -  description
                             -------------------
    begin                : Thu Apr 28 2005
    copyright            : (C) 2005 by Ilya A. Volynets-Evenbakh
    email                : ilya@total-knowledge.com
    based on             : CWaiter.h by Alexey Parshin. All rights reserved.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CRWLOCK_H__
#define __CRWLOCK_H__

#ifndef _WIN32

#include <unistd.h>
#include <pthread.h>

#else

#include <windows.h>
#include <process.h>

#endif

#include <sptk3/sptk.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// Read-write synchronization object
class SP_EXPORT CRWLock {
protected:

#ifndef _WIN32

    pthread_rwlock_t    m_rwlock;   ///< pthreads rwlock object
#else

    HANDLE               m_readLock;
    HANDLE               m_writeLock;
    CRITICAL_SECTION    m_criticalSection;
    int                  m_readerCount;
    bool                 m_writerMode;
#endif

public:

    /// Constructor
    CRWLock();

    /// Destructor
    ~CRWLock();

    /// Try to lock the object for reading. Blocks if object is locked for writing, or there are pending write locks.
    /// @param timeout int, timeout in milliseconds
    int lockR(int timeout=-1);

    /// Try to lock the object for writing. Blocks if object is locked for reading or writing.
    /// @param timeout int, timeout in milliseconds
    int lockRW(int timeout=-1);

    /// Releases lock on the object.
    void unlock();
};
/// @}
}

#endif
