/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CSmtpConnect.h  -  description
                             -------------------
    begin                : Jun 25 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CSMTPCONNECT_H__
#define __CSMTPCONNECT_H__

#include <sptk3/CSocket.h>
#include <sptk3/CStrings.h>
#include <sptk3/CBaseMailConnect.h>

#include <string>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief SMTP socket
///
/// Sends an e-mail message using SMTP protocol.
/// It uses CSocket class to establish the connection, and CBaseMailConnect
/// to make the complete RFC 822 message.
class CSmtpConnect : public CBaseMailConnect, public CSocket {
    CStrings     m_response;

    /// Processes tag for strippedHtml.
    /// Extracts the text information for the tag
    /// @param tag std::string, the tag name
    /// @param params std::string, the tag parameters
    /// @returns the extracted text like URL
    static std::string processTag(std::string tag,std::string params);
protected:

    /// @brief Sends command using SMTP protocol
    ///
    /// The CRLF characters after the command are added automatically.
    /// @param cmd std::string, SMTP protocol command
    /// @param encode bool, encode the arguments to Base64 or not
    void sendCommand(std::string cmd,bool encode=false);

    /// @brief Retrieves the server response after the command into internal CStrings buffer
    ///
    /// The response can be read then with response() method.
    /// @param decode bool, decode the response from Base64 or not
    int  getResponse(bool decode=false);

    /// Mime-encodes the string
    static std::string mime(std::string s);

    /// Mime-decodes the string
    static std::string unmime(std::string s);

public:

    /// Default constructor
    CSmtpConnect();

    /// Destructor
    ~CSmtpConnect();

    /// Sends command using SMTP protocol and retrieve the server response.
    /// The response can be read then with response() method.
    /// The CRLF characters after the command are added automatically.
    /// @param cmd std::string, SMTP protocol command
    /// @param encodeCommand bool, encode the comand argument to Base64 or not
    /// @param decodeResponse bool, decode the response from Base64 or not
    int command(std::string cmd,bool encodeCommand=false,bool decodeResponse=false);

    /// @brief The response from the server - makes sence after calling any command
    CStrings& response() {
        return m_response;
    }

    /// @brief Logs in to the server host()
    /// @param user std::string, user name
    /// @param password std::string, user password
    void cmd_login(std::string user,std::string password);

    /// @brief Sends the message
    ///
    /// Message is based on the information defined by the methods from
    /// CBaseMailConnect, and retrieves the server output. An alias for sendMessage().
    void cmd_send();


    /// Ends the SMTP session
    void cmd_quit();

    /// @brief Sends the message
    ///
    /// The message based on the information defined by the methods from
    /// CBaseMailConnect, and retrieves the server output.
    virtual void sendMessage();

    /// @brief Strips HTML tags off the message, prepare the alternative text for an HTML message
    /// @param html const std::string&, the HTML text
    /// @returns plain text with stripped HTML messages
    static std::string stripHtml(const std::string& html);
};
/// @}
}
#endif
