/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CStringStack.h  -  description
                             -------------------
    begin                : Mon Aug 4 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CSTRINGSTACK_H__
#define __CSTRINGSTACK_H__

#include <sptk/CStringList.h>

namespace sptk {

/// Simple String Stack
class CStringStack : protected std::vector<std::string> {
public:
   /// Default constructor
   CStringStack() {}

   /// Pushes a string into the stack
   /// @param str const char *, string to push
   void push(const char *str) { push_back(str); };

   /// Pushes a string into the stack
   /// @param str const CString&, string to push
   void push(const std::string& str) { push_back(str); };

   /// Pops a string from the stack
   /// @returns string from the stack
   std::string pop();

   /// Returns a stack's top string reference
   /// @returns string from the stack
   std::string& top();

   /// Returns true if the stack is empty
   bool empty() const { return size() == 0; }
};

}

#endif
