/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTar.h  -  description
                             -------------------
    begin                : Fri Sep 1 2006
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.

    This module creation was sponsored by Total Knowledge (http://www.total-knowledge.com).
    Author thanks the developers of CPPSERV project (http://www.total-knowledge.com/progs/cppserv)
    for defining the requirements for this class.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTAR_H__
#define __CTAR_H__

#include <sptk3/sptk.h>
#include <sptk3/CException.h>
#include <sptk3/CStrings.h>
#include <sptk3/CBuffer.h>

namespace sptk {

/// @brief Tar memory handle
class CMemoryTarHandle {
public:
    uint32_t    position;         ///< Memory buffer position
    char*       sourceBuffer;     ///< Memory buffer
    uint32_t    sourceBufferLen;  ///< Memory buffer len
public:
    /// @brief Constructor
    /// @param buffer CBuffer*, source data
    CMemoryTarHandle(CBuffer* buffer=0) {
        position = 0;
        if (buffer) {
            sourceBuffer = buffer->data();
            sourceBufferLen = buffer->size();
        } else {
            sourceBuffer = 0;
            sourceBufferLen = 0;
        }
    }
};

typedef std::map<int,CMemoryTarHandle*> CTarHandleMap;

/// @brief A wrapper for libtar functions
///
/// Allows reading tar archive files into memory buffers.
/// The main usage currently is to read an SPTK theme from tar-archive.
class CTar {
    typedef std::map<std::string,CBuffer*>  CFileCollection;
    void*                 m_tar;         ///< Tar file header
    CFileCollection       m_files;       ///< File name to the file data map
    CStrings              m_fileNames;   ///< List of files in archive
    bool                  m_memoryRead;  ///< Flag to indicate if tar data is red from the memory buffer
    std::string           m_fileName;    ///< Tar file name

    /// @brief Loads tar file into memory
    bool loadFile() throw (CException);

    /// @brief Throws an error
    void throwError(std::string fileName) throw (CException);
public:
    static int            lastTarHandle; ///< The last generated tar handle
    static CTarHandleMap *tarHandleMap;  ///< The map of tar handles

        /// @brief Returns memory handle
    /// @param handle int, tar handle
    static CMemoryTarHandle* tarMemoryHandle(int handle);

    /// @brief Overwrites standard tar open
    static int mem_open(const char *name, int x, ...);

    /// @brief Overwrites standard tar close
    /// @param handle int, tar handle
    static int mem_close(int handle);

    /// @brief Overwrites standard tar read
    /// @param handle int, tar handle
    /// @param buf void*, data buffer
    /// @param len size_t, read size
    static int mem_read(int handle, void *buf, size_t len);

    /// @brief Overwrites standard tar write
    ///@param handle int, tar handle
    /// @param buf void*, data buffer
    /// @param len size_t, write size
    static int mem_write(int handle, const void *buf, size_t len);

public:
    /// @brief Constructor
    CTar();

    /// @brief Destructor
    ~CTar() { clear(); }

    /// @brief Reads tar archive from file
    ///
    /// The archive content is red into the internal set of buffers
    /// @param fileName std::string, file name to open
    void read(const std::string& fileName) throw (CException) {
        read(fileName.c_str());
    }

    /// @brief Reads tar archive from file
    ///
    /// The archive content is red into the internal set of buffers
    /// @param fileName std::string, file name to open
    void read(const char* fileName) throw (CException);

    /// @brief Reads tar archive from buffer
    ///
    /// The archive content is red into the internal set of buffers
    /// @param tarData const CBuffer&, tar file buffer
    void read(const CBuffer& tarData) throw (CException);

    /// @brief returns a list of files in tar archive
    const CStrings& fileList() const { return m_fileNames; }

    /// @brief Returns file data by file name
    /// @param fileName std::string, file name
    const CBuffer& file(std::string fileName) const throw (CException);

    /// @brief Clears the allocated memory
    void clear();
};

}

#endif
