/***************************************************************************
                           SIMPLY POWERFUL TOOLKIT (SPTK)
                             CThread.h  -  description
                                -------------------
        begin                : Thu Jul 12 2001
        copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
        email                : alexeyp@gmail.com
  ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTHREAD_H__
#define __CTHREAD_H__

#include <sptk3/CWaiter.h>
#include <sptk3/CFileLog.h>
#include <sptk3/CStrings.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief Base thread object.
///
/// Should be used for deriving a user thread
/// by overwriting threadFunction().
class CThread {
protected:
    bool              m_terminated; ///< Flag: is the thread terminated?
    bool              m_running;    ///< Flag: is the thread running?
    bool              m_politeMode; ///< Flag: does the thread uses the 'polite' mode for shutdown?
    std::string       m_name;       ///< Thread name
    CFileLog*         m_log;        ///< Optional thread log
#ifndef _WIN32
    pthread_t         m_thread;     ///< Thread handle
#else

    HANDLE            m_thread;     ///< Thread handle
    HANDLE            m_timer;      ///< Thread timer handle
#endif

#ifndef _WIN32
    /// Internally starts a new thread
    static void *threadStart(void *p);
#else
    /// Internally starts a new thread
    static unsigned __stdcall threadStart(void *p);
#endif
    /// Internal cleanup handler - will call onThreadExit
    static void threadCleanup(void *p);

protected:

    /// Creates a thread
    void createThread();

    /// Destroys the thread
    void destroyThread();

    /// Executes the thread function
    void runThread();

    /// Exit handler, called whenever thread terminates. Does nothing by default
    virtual void onThreadExit() {}

public:

    /// Constructor
    /// @param name CString, name of the thread for future references.
    /// @param politeMode bool, true if we want to shutdown thread waitng for the crrect exit.
    /// @param fileLog CFileLog*, optional logger
    CThread(std::string name,bool politeMode=true,CFileLog* fileLog=0);

    /// Destructor
    virtual ~CThread();

    /// Starts the already created thread
    void run();

    /// The thread function. Should be overwritten by the derived class.
    virtual void threadFunction() = 0;

    /// Requests to terminate the thread
    void terminate();

    /// Returns true if the thread is terminated
    bool terminated() {
        return m_terminated;
    }

    /// Returns true if the thread is running
    bool running() {
        return m_running;
    }

    /// Returns the name of the thread
    const std::string& name() const {
        return m_name;
    }

    /// Pauses the thread
    /// @param msec int, pause time in milliseconds
    static void msleep(int msec);
};
/// @}
}

#endif
