/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CUniqueInstance.h  -  description
                             -------------------
    begin                : June 6 2002
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CUNIQUE_INSTANCE_H__
#define __CUNIQUE_INSTANCE_H__

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#endif

#include <sptk3/sptk.h>
#include <sptk3/CStrings.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief Unique instance object
///
/// Tries to create a mutex object and check if it's unique for the
/// program with the instance name. If one instance per user is desired
/// simply incorporate the user name into the instance.
class SP_EXPORT CUniqueInstance {
    std::string  m_instanceName;    ///< Instance name
    bool     m_lockCreated;       ///< Lock is created

#ifdef _WIN32

    HANDLE   m_mutex;             ///< The named mutex object
#else

    std::string  m_fileName;      ///< The lock file name

    int      read_pid();          ///< Gets the process ID
    int      write_pid();         ///< Writes the process ID into the lock file
#endif

public:
    /// Constructor
    /// @param instanceName CString, instance name
    CUniqueInstance(std::string instanceName);

    /// Destructor
    ~CUniqueInstance();

    /// Reports true if the instance is unique
    bool isUnique();
}
;
/// @}
}

#endif
