/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWaiter.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef __CWAITER_H__
#define __CWAITER_H__

#ifndef _WIN32
   #include <unistd.h>
   #include <pthread.h>
#else
   #include <winsock2.h>
   #include <windows.h>
   #include <process.h>
#endif

#include <sptk3/sptk.h>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// @brief Synchronization object
class SP_EXPORT CWaiter {

protected:

#ifndef _WIN32
    pthread_mutex_t   m_waiter;             ///< Mutex object
    pthread_cond_t    m_condition;          ///< Condition object
#else
    HANDLE            m_waiter;             ///< Event object
#endif

public:

    /// @brief Constructor
    CWaiter();

    /// @brief Destructor
    ~CWaiter();

    /// @brief Tries to lock waiter. Blocks until the lock.
    void lock();

    /// @brief Tries to lock waiter. Returns false if it fails. Doesn't block.
    bool try_lock();

    /// @brief Unlocks the waiter.
    void unlock();

    /// @brief Waits for a signal, timeout in msec
    /// @param timeout int, timeout in milliseconds
    int  waitForSignal(int timeout=-1);

    /// @brief Sends signal to stop waiting.
    ///
    /// Caller needs to aquire lock explicitly before calling this.
    void sendSignalNoLock();

    /// @brief Sends signal to stop waiting
    void sendSignal();

    /// @brief Sends signal to all waiting threads.
    ///
    /// Caller needs to aquire lock explicitly before calling this.
    void broadcastSignalNoLock();

    /// @brief Sends signal to all waiting threads.
    void broadcastSignal();
}
;
/// @}
}

#endif
