/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDatabaseField.h  -  description
                             -------------------
    begin                : Wednesday November 2 2005
    copyright            : (C) 2005 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CDATABASEFIELD_H__
#define __CDATABASEFIELD_H__

#include <sptk3/sptk.h>

#include <sptk3/CVariant.h>
#include <sptk3/CStrings.h>
#include <sptk3/CField.h>

namespace sptk {

/// @addtogroup Database Database Support
/// @{

class CQuery;
class CDatabase;

/// @brief database field
///
/// A special variation of CField to support database field essentials
class SP_EXPORT CDatabaseField : public CField {
    friend class CQuery;
    friend class CDatabase;
protected:
    int          m_fldType;     ///< Native database data type
    int          m_fldColumn;   ///< Field column number in recordset
    int          m_fldSize;     ///< Field size
    int          m_fldScale;    ///< Field scale, optional, for floating point fields

public:

    /// Constructor
    /// @param fieldName std::string, field name
    /// @param fieldColumn int, field column number
    /// @param fieldType int, database field type
    /// @param dataType CVariantType, variant data type
    /// @param fieldLength int, database field length
    /// @param fieldScale int, database field scale
    CDatabaseField(const std::string fieldName,int fieldColumn,int fieldType,CVariantType dataType,int fieldLength,int fieldScale=0);

    std::string      displayName;       ///< Column display name
    std::string      displayFormat;     ///< Column display format
    int              alignment;         ///< Column alignment
    bool             visible;           ///< Is column visible?

    /// @brief Checks the internal buffer size
    ///
    /// The internal buffer is automatically extended to fit the required size of data
    /// @param sz uint32_t, data size (in bytes)
    /// @returns true if success
    bool checkSize(uint32_t sz);

    /// Reports field column number
    int          fieldColumn() const {
        return m_fldColumn;
    }

    /// Reports database field type
    int          fieldType()   const {
        return m_fldType;
    }

    /// Reports field size
    uint32_t     fieldSize()   const {
        return m_fldSize;
    }
};
/// @}
}
#endif
