/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTransaction.h  -  description
                             -------------------
    begin                : Mon Apr 17 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTRANSACTION_H__
#define __CTRANSACTION_H__

#include <sptk3/db/CDatabase.h>
#include <sptk3/CException.h>

namespace sptk {

/// @addtogroup Database Database Support
/// @{

/// @brief Database Transaction.
///
/// Allows to begin, commit, and rollback the transaction automatically.
/// If the transaction object is deleted w/o commiting or rolling back
/// the transaction, it rolls back the transaction (if active)
class CTransaction {
    bool         m_active;   ///< Transaction activity
    CDatabase   *m_db;       ///< Database to work with
public:
    /// Constructor
    /// @param db CDatabase&, the database to work with
    CTransaction(CDatabase& db);

    /// Destructor
    ~CTransaction();

    /// Begins the transaction
    void begin();

    /// Commits the transaction
    void commit();

    /// Rolls back the transaction
    void rollback();

    /// Is transaction active?
    bool active() const {
        return m_active;
    }
};
/// @}
}
#endif
