/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelBAT.h  -  description
                             -------------------
    begin                : 19 March 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELBAT_H__
#define __CEXCELBAT_H__

#include <sptk3/CBuffer.h>
#include <stdio.h>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

class CExcelHeader;
class CExcelProperties;

/// The collection of the BAT blocks for the Excel file header
class CExcelBAT {
    uint32_t  m_batStartBlock;        ///< The location of the first BAT block in the file
    uint32_t  m_batBlockCount;        ///< Total number of BAT blocks in the file
    uint32_t  m_xbatStartBlock;       ///< The location of the first extended BAT block in the file
    uint32_t  m_xbatBlockCount;       ///< Total number of extended BAT blocks in the file
    uint32_t  m_streamStartBlock;     ///< The first stream block in the file
    uint32_t  m_streamBlockCount;     ///< Total number of blocks in stream
    CBuffer m_batIndex;             ///< The binary representation of BAT index (for the file header part)
    CBuffer m_batBuffer;            ///< The binary representation of BAT itself including XBAT if presented
    CBuffer m_xbatIndex;            ///< The binary representation of XBAT index
public:

    /// Constructor
    CExcelBAT();

    /// Reads BAT from the Excel file
    /// @param fileHeader CExcelHeader *, the Excel file header
    /// @param f FILE *, the input file
    void extract(CExcelHeader *fileHeader,FILE *f);

    /// Builds BAT table and index from m_streamStartBlock, m_streamBlockCount etc
    /// @param properties CExcelProperties *, the list of Excel properties
    /// @param batStartBlock uint32_t, defines the start block of the XBAT and BAT
    void build(CExcelProperties *properties,uint32_t batStartBlock);

    /// Returns binary representation of the BAT index
    const CBuffer& batIndex() const;

    /// Returns binary representation of the XBAT index
    const CBuffer& xbatIndex() const;

    /// Returns binary representation of the BAT table
    const CBuffer& batData() const;

    /// Resets the data buffers and sets the pointers to default values
    void clear();

    /// Defines the start block of the stream
    void streamStartBlock(uint32_t ssb);

    /// Returns the start block of the stream
    uint32_t streamStartBlock() const {
        return m_streamStartBlock;
    }

    /// Defines the block count of the stream
    void streamBlockCount(uint32_t sbc);

    /// Returns the block count of the stream
    uint32_t streamBlockCount() const {
        return m_streamBlockCount;
    }

    /// Returns the XBAT start block number
    uint32_t xbatStartBlock() const {
        return m_xbatStartBlock;
    }

    /// Returns the required XBAT block count
    uint32_t xbatBlockCount() const {
        return m_xbatBlockCount;
    }

    /// Returns the start block of the BAT
    uint32_t batStartBlock() const {
        return m_batStartBlock;
    }

    /// Returns total number of blocks in BAT
    uint32_t blockCount() const {
        return m_batBlockCount;
    }

    /// Returns total size occupied by BAT
    uint32_t bytes() const {
        return m_batBuffer.bytes();
    }
};
/// @}
}
#endif
