/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelColumn.h  -  description
                             -------------------
    begin                : 19 March 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELCOLUMN_H__
#define __CEXCELCOLUMN_H__

#include <sptk3/excel/CExcelRecord.h>
#include <sptk3/excel/CExcelRecordsBuffer.h>
#include <vector>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// @brief Excel COLINFO record (a column information record)
///
/// The current approach is to have one record per column
class CExcelColumn : public CExcelRecord {
    friend class CExcelColumns;
protected:
public:
    /// Default constructor
    CExcelColumn(uint32_t columnIndex=0);

    /// Returns column index
    uint32_t index() const;

    /// Returns column width in characters of the default font
    uint16_t width() const;

    /// Sets the column width
    /// @param w uint32_t, the column width in characters of the default font
    void width(uint16_t w);
};

/// A set of CExcelColumn in memory
class CExcelColumns : public std::vector<CExcelColumn *> {
public:
    /// Default constructor
    CExcelColumns() {}

    /// Destructor
    ~CExcelColumns() {
        clear();
    }

    /// Cleaner
    void clear();

    /// Resizes the column set
    /// @param newSize uint32_t, new number of columns
    void resize(uint32_t newSize);

    /// Saves columns information into Excel records
    /// @param records CExcelRecordsBuffer&, Excel records
    void saveRecords(CExcelRecordsBuffer& records);
};
/// @}
}
#endif
