/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelHeader.h  -  description
                             -------------------
    begin                : Mon Jan 19 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELHEADER_H__
#define __CEXCELHEADER_H__

#include <sptk3/excel/CExcelBAT.h>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

#define BIG_BLOCK_SIZE    0x0200         ///< The size of the big block in Excel header
#define SMALL_BLOCK_SIZE  0x040          ///< The size of the small block in Excel header
#define END_OF_CHAIN      (uint32_t)-2   ///< End of chain mark
#define UNUSED_BLOCK      (uint32_t)-1   ///< Unused block mark

#define _SIGNATURE               0xE11AB1A1E011CFD0L  ///< Excel file signature
#define _BAT_ARRAY_OFFSET        0x4c                 ///< BAT array data offset in the header
#define _MAX_BATS_IN_HEADER      (BIG_BLOCK_SIZE - _BAT_ARRAY_OFFSET) / 4  ///< Max BAT blocks in the header

// useful offsets
#define _SIGNATURE_OFFSET        0     ///< Signature offset in the file
#define _BAT_COUNT_OFFSET        0x2C  ///< BAT count offset in the file
#define _PROPERTY_START_OFFSET   0x30  ///< Property list offset in the file
#define _SBAT_START_OFFSET       0x3C  ///< Small BAT data offset in the file
#define _SBAT_COUNT_OFFSET       0x40  ///< Small BAT count offset in the file
#define _XBAT_START_OFFSET       0x44  ///< Large BAT data offset in the file
#define _XBAT_COUNT_OFFSET       0x48  ///< Large BAT count offset in the file

/// Represents the Excel file header
class CExcelHeader {
    char m_data[BIG_BLOCK_SIZE];  ///< Binary copy of the header
public:

    /// Constructor
    /// @param data const char *, external header data to copy into header
    CExcelHeader(const char *data=0L);

    /// Defines the BAT table - for creating an Excel file
    /// @param batTable CExcelBAT&, the BAT table
    void setBatTable(CExcelBAT& batTable);

    /// Defines the position and data of SBAT table - for creating an Excel file
    /// @param locationBlock uint32_t, the block where SBAT table is located
    /// @param batTable CExcelBAT&, the SBAT table
    void setSBatTable(uint32_t locationBlock,CExcelBAT& batTable);

    /// Defines the Property table start - for creating an Excel file
    /// @param locationBlock uint32_t, the block where property table is located
    void setPropertyTableStart(uint32_t locationBlock);

    /// Returns the pointer to header data
    const char *data() const {
        return m_data;
    }

    /// Sets the external header data
    /// @param data const char *, external header data
    void data(const char *data);

    /// Reads header from file
    /// @param f FILE *, the input file
    void read(FILE *f);

    /// Saves header to file
    /// @param f FILE *, the output file
    void save(FILE *f);

    /// Returns the total BAT block count
    uint32_t batBlockCount() const;

    /// Sets the start block of XBAT
    /// @param startBlock uint32_t, XBAT start block
    void xbatStartBlock(uint32_t startBlock);

    /// Sets the start block of XBAT
    /// @param blockCount uint32_t, XBAT block count
    void xbatBlockCount(uint32_t blockCount);

    /// Returns the start block of XBAT
    uint32_t xbatStartBlock() const;

    /// Returns the XBAT block count
    uint32_t xbatBlockCount() const;

    /// Resets header to default state
    void clear();
};
/// @}
}
#endif
