/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelIndex.h  -  description
                             -------------------
    begin                : Mon Jan 19 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELINDEX_H__
#define __CEXCELINDEX_H__

#include <sptk3/excel/CExcelRows.h>
#include <sptk3/excel/CExcelDBCellRecord.h>
#include <sptk3/excel/CExcelRecordsBuffer.h>
#include <sptk3/excel/CExcelSST.h>

#include <vector>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// Represents Excel INDEX record
class CExcelIndex : public CExcelRecord {
    CExcelRows                        *m_rows;          ///< Rows in the index
    std::vector<CExcelDBCellRecord *>  m_dbcellRecords; ///< DBCELL records in the index
public:
    /// Constructor
    /// @param rows std::vector<CExcelRow *> *, the list of CExcelRow - memory presentation of Excel rows
    CExcelIndex(CExcelRows *rows);

    /// Destructor
    ~CExcelIndex();

    /// Saves INDEX, DBCELL, and cell records into records' buffer
    /// @param records CExcelRecordsBuffer&, the output records buffer
    /// @param sst const CExcelSST&, the existing Shared String Table (SST)
    /// @param sheetStreamPosition uint32_t, the current sheet's beginning position in the Excel stream
    /// @param defColWidthPosition uint32_t, the current sheet's DEFCOLWIDTH record position in the Excel stream
    void saveRecords(CExcelRecordsBuffer& records,const CExcelSST& sst,uint32_t sheetStreamPosition,uint32_t defColWidthPosition);
};
/// @}
}
#endif
