/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelNumericFormat.h  -  description
                             -------------------
    begin                : Fri May 6 2004
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELNUMERICFORMAT_H__
#define __CEXCELNUMERICFORMAT_H__

#include <sptk3/CVariant.h>

#include <sptk3/excel/CExcelRecordsBuffer.h>

#include <map>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// The numeric format descriptor
class CExcelNumericFormat {
    CVariantType   m_kind;           ///< Format data type
    uint32_t         m_index;          ///< Format index
    std::string    m_format;         ///< Format string
public:
    /// Constructor
    /// @param kind CVariantType, format data type
    /// @param index uint32_t, format index
    /// @param format CString, format string
    CExcelNumericFormat(CVariantType kind,uint32_t index,std::string format);

    /// Default constructor.
    /// The missing 'kind' parameter is guessed from the format string
    /// @param index uint32_t, format index
    /// @param format CString, format string
    CExcelNumericFormat(uint32_t index=0,std::string format="");

    /// Copy constructor
    /// @param nf const CExcelNumericFormat&, numeric format to copy from
    CExcelNumericFormat(const CExcelNumericFormat& nf);

    /// Returns format data type
    CVariantType kind() const {
        return m_kind;
    }  
    
    /// Returns format index
    uint32_t index() const {
        return m_index;
    }    

    /// Returns format string
    std::string format() const {
        return m_format;
    }  
     
    /// Sets format string
    void format(const char *f) {
        m_format = f;
    }  
};

/// The list of an Excel numeric formats
class CExcelNumericFormats : public std::map<uint32_t,CExcelNumericFormat *> {
public:
    /// Constructor
    CExcelNumericFormats()  {}

    /// Destructor
    ~CExcelNumericFormats() {
        clear();
    }

    /// Cleaner
    void clear();

    /// Encodes FORMAT records into internal Excel format
    void saveRecords(CExcelRecordsBuffer& records);
};
/// @}
}
#endif
