/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelPalette.h  -  description
                             -------------------
    begin                : Wed May 12 2004
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELPALETTE_H__
#define __CEXCELPALETTE_H__

#include <sptk3/excel/CExcelRecord.h>
#include <sptk3/excel/CExcelRecordsBuffer.h>

#include <map>

namespace sptk {

#define EPC_BLACK   0
#define EPC_WHITE   1
#define EPC_RED     2
#define EPC_GREEN   3
#define EPC_BLUE    4
#define EPC_YELLOW  5
#define EPC_MAGENTA 6
#define EPC_CYAN    7

#define EPC_SYSBORDER       0x40
#define EPC_SYSBACKGROUND   0x41
#define EPC_SYSTEXT         0x7FFF

/// @addtogroup excel Excel Classes
/// @{

/// @brief Excel palette record.
///
/// Represents the colours available for this Excel file.
/// First eight colours are standard and should not be redefined.
class CExcelPalette : public CExcelRecord {
    /// The standard colors palette
    static const uint32_t  standardPalette[8];

protected:
    std::map<uint32_t,uint32_t> m_colorMap;        ///< The map of defined colors
    uint32_t                    m_colors[64];      ///< The list of defined colors

public:

    /// Default constructor
    CExcelPalette() : CExcelRecord(PALETTE,258,new char[258],true) {
        clear();
    }

    /// Initializes the default colours int the palette
    void clear();

    /// Returns the palette index for the RGB color.
    /// If the color isn't part of the palette yet it's added to palette.
    /// The maximum number of colors in the palette is 56
    /// @param rgbColor uint32_t, the RGB color value
    /// @returns the index of color in the palette
    uint32_t colorIndex(uint32_t rgbColor);

    /// Returns the palette RGB color for the index.
    /// Index may be in 8..63
    uint32_t colorForIndex(uint32_t colorIndex) const;

    /// Saves PALETTE record into the external Excel format.
    /// @param records CExcelRecordsBuffer&, the output buffer of Excel records
    void saveRecords(CExcelRecordsBuffer& records);
};
/// @}
}
#endif
