/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelRecordsBuffer.h  -  description
                             -------------------
    begin                : Mon Jan 19 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELRECORDSBUFFER_H__
#define __CEXCELRECORDSBUFFER_H__

#include <sptk3/CBuffer.h>
#include <sptk3/excel/CExcelRecord.h>
#include <vector>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// The buffer that contains the Excel commands
class CExcelRecordsBuffer : public CBuffer {
    uint32_t               m_currentPosition;         /// Current position for next()
	std::vector<uint32_t>  m_sheetPosition;           /// The list of BOUNDSHEET record positions - for creating a new records buffer
public:
    /// Constructor
    /// creates an empty buffer of pre-allocated size
    /// @param sz uint32_t, the required size of the buffer
    CExcelRecordsBuffer(uint32_t sz) : CBuffer(sz) {
        m_currentPosition = 0;
    }

    /// Constructor
    /// creates a buffer of the appropriate size and stores data into it
    /// @param data void *, the pointer to the data
    /// @param sz uint32_t, the size of the data
    CExcelRecordsBuffer(void *data,uint32_t sz) : CBuffer(data,sz) {
        m_currentPosition = 0;
    }

    /// Reset the internal data
    void reset();

    /// Adds a new BOUNDSHEET record position in the buffer - for creating a new records buffer
    /// @param pos uint32_t, the position of the new BOUNDSHEET record in the stream
    void addSheetPosition(uint32_t pos) {
        m_sheetPosition.push_back(pos);
    }

    /// Returns a position of the sheet in the stream
    /// @param sheetIndex uint32_t, the sheet index
    uint32_t sheetPosition(uint32_t sheetIndex) const {
        return m_sheetPosition[sheetIndex];
    }

    /// Positions the current record to the first available record, or NULL
    /// @returns pointer to the new CExcelRecord object with the data of the first available record
    /// Application should delete this object eventually
    CExcelRecord *first();

    /// Positions the current record to the next available record, or NULL
    /// @returns pointer to the new CExcelRecord object with the data of the first available record
    /// Application should delete this object eventually
    CExcelRecord *next();

    /// Appends a copy of CExcelRecord to the record buffer
    /// @param rec const CExcelRecord *, the Excel record buffer
    void append(const CExcelRecord *rec);

    /// Appends the raw data to the record buffer
    /// @param recordType uint32_t, record type
    /// @param dataSize uint32_t, record data size
    /// @param data const char *, record data
    void append(uint32_t recordType,uint32_t dataSize,const char *data);

    /// Appends the raw data to the record buffer
    /// @param data const char *, raw data
    /// @param dataSize uint32_t, raw data size
    void append(const char *data,uint32_t dataSize) {
        CBuffer::append(data,dataSize);
    }

    /// Appends the raw data buffer to the record buffer
    /// @param buffer const CBuffer&, raw data
    void append(const CBuffer& buffer) {
        CBuffer::append(buffer.data(),buffer.bytes());
    }
};
/// @}
}
#endif
