/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelRow.h  -  description
                             -------------------
    begin                : Apr 23 2004
    copyright            : (C) 2004-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELROW_H__
#define __CEXCELROW_H__

#include <sptk3/excel/CExcelCell.h>
#include <vector>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// Represents a row of Excel data in memory
class CExcelRow {
    typedef std::vector<CExcelCell *> CCellsVector; ///< Internal class to store cells

    CCellsVector    m_cells;    ///< Cells in the row

public:

    uint32_t m_height;      ///< The row height
    uint32_t   m_position;    ///< The row position
public:
    /// Default constructor
    CExcelRow() {
        m_height = 0xFF;
        m_position = 0;
    }

    /// Destructor
    ~CExcelRow() {
        clear();
    }

    /// Cleaner
    void clear();

    /// Provides access to the cells in the row
    /// @param i uint32_t, cell index
    CExcelCell& operator [] (uint32_t i) const {
        return *m_cells[i];
    }

    /// @brief Returns number of cells in the row
    uint32_t size() const {
        return (uint32_t) m_cells.size();
    }

    /// @brief Resize the row
	/// @param newSize uint32_t, new number of cells in the row
    void resize(uint32_t newSize);
};
/// @}
}
#endif
