/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelRowRecord.h  -  description
                             -------------------
    begin                : Apr 23 2004
    copyright            : (C) 2004-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/


#ifndef __CEXCELROWRECORD_H__
#define __CEXCELROWRECORD_H__

#include <sptk3/excel/CExcelRecordsBuffer.h>
#include <sptk3/excel/CExcelRow.h>
#include <sptk3/excel/CExcelSST.h>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

/// Represents Excel row
class CExcelRowRecord : public CExcelRecord {
    uint32_t       m_rowNumber;           ///< Row number, 0-based
    uint32_t       m_streamPosition;      ///< Stream position of the row record
    uint32_t       m_firstCellPosition;   ///< Stream position of the first cell record for the row
    uint32_t       m_firstDefinedColumn;  ///< The column of the first defined cell in the row
    uint32_t       m_lastDefinedColumn;   ///< The column of the last defined cell in the row
    CExcelRow*   m_cells;               ///< The actual Excel row

    void saveMultipleRK(CExcelRecordsBuffer& records,char *mulRKBuffer,uint32_t firstColumn,uint32_t& count);

    /// Updates row record with the actual cells columns and positions
    /// @param recordStreamPtr char *, ROW record pointer (in the output stream)
    void updateRowRecord(char *recordStreamPtr);
public:

    /// Constructor
    /// @param rowNumber uint32_t, number of row in the spreadseet, 0-based
    /// @param row CExcelRow *, the actual data of the row, kept as a pointer to an external data
    CExcelRowRecord(uint32_t rowNumber,CExcelRow *row);

    /// Saves ROW record into the external Excel format
    /// @param records CExcelRecordsBuffer&, the output buffer of Excel records
    void saveRowRecord(CExcelRecordsBuffer& records);

    /// Saves cell records (for this) into the external Excel format.
    /// Also updates the ROW record with the actual stream position, first and last cells of the row.
    /// @param records CExcelRecordsBuffer&, the output buffer of Excel records
    /// @param sst const CExcelSST&, the existing SST
    /// @param secondRowStart uint32_t, the position of the second row of this set in the stream
    void saveCellRecords(CExcelRecordsBuffer& records,const CExcelSST& sst,uint32_t secondRowStart);

    /// Returns the count of cells in the row, including empty cells
    uint32_t size() const {
        return m_cells->size();
    }

    /// Provides access to the cells in the row
    /// @param i uint32_t, cell index
    CVariant& operator [] (uint32_t i) const {
        return (*m_cells)[i];
    }

    /// Returns stream position of the row record
    uint32_t streamPosition() const {
        return m_streamPosition;
    }

    /// Returns stream position of the first cell record for the row
    uint32_t firstCellPosition() const {
        return m_firstCellPosition;
    }
};
/// @}
}
#endif
