/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelSST.h  -  description
                             -------------------
    begin                : Tue Apr 6 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELSST_H__
#define __CEXCELSST_H__

#include <sptk3/CStrings.h>
#include <sptk3/excel/CExcelRecordsBuffer.h>

#include <map>

namespace sptk {

/// @addtogroup excel Excel Classes
/// @{

class CExcelBook;

/// Excel Shared String Table (SST)
class CExcelSST {
    typedef std::map<std::string,uint32_t> CSstStringMap;
    uint32_t                m_totalBookStrings; ///< Total number of strings in the workbook
    uint32_t                m_sstSize;          ///< Total number of unique strings in the workbook
    CStrings              m_stringList;       ///< List of the strings in the SST
    CSstStringMap         m_stringMap;        ///< Map of the strings in the SST. The (uint32_t) in the map is the index of the string in SST
    const CExcelBook     *m_parentBook;       ///< Parent workbook

    /// Appends a long string (8221+ bytes) to Excel records buffer
    /// @param records CExcelRecordsBuffer&, the output records buffer
    /// @param recordType int16_t&, current record type (SST or CONTINUE)
    /// @param sstData char *, raw SST data buffer
    /// @param recordStart uint32_t&, the position of the current record in the record buffer
    /// @param pos uint32_t&, position of the string in the raw SST data buffer
    /// @param len uint32_t, the length of the string
    void saveLongString(CExcelRecordsBuffer& records,int16_t& recordType,char *sstData,uint32_t& recordStart,uint32_t& pos,uint32_t len);
public:
    /// Default constructor
    CExcelSST() {
        m_parentBook = 0L;
        clear();
    }

    /// Fast insert of the string into SST. Used when we know that the string isn't in SST yet.
    uint32_t forceInsert(const char *str);

    /// Inserts string into the SST. The string is auto-checked for the uniqueness.
    uint32_t insert(const char *str);

    /// Returns index of the given string in SST, or -1 if not found
    uint32_t index(const char *str) const;

    /// Clears the content of the SST
    void clear();

    /// Encodes SST into internal Excel format
    void saveRecords(CExcelRecordsBuffer& records);

    /// Returns string by it's index in SST
    const std::string& operator [] (uint32_t i) const {
        return m_stringList[i];
    }

    /// Returns count of the unique strings in SST
    uint32_t size() const {
        return (uint32_t) m_stringList.size();
    }

    /// Returns parent workbook
    const CExcelBook *parentBook() const {
        return m_parentBook;
    }

    /// Sets parent workbook
    /// @param book const CExcelBook *, the pointer to the parent workbook
    void parentBook(const CExcelBook *book) {
        m_parentBook = book;
    }
};
/// @}
}
#endif
