/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CExcelStyle.h  -  description
                             -------------------
    begin                : Mon Jan 19 2004
    copyright            : (C) 2004 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEXCELSTYLE_H__
#define __CEXCELSTYLE_H__

namespace sptk {
/// @addtogroup excel Excel Classes
/// @{

/// Represents Excel Style record
class CExcelStyle {
    char   m_buffer[4];
public:
    /// Constructor
    /// @param buffer const char *, Excel STYLE record data (4 bytes)
    CExcelStyle(const char *buffer=0L);

    /// Copy constructor
    /// @param buffer const CExcelStyle&, Excel STYLE record
    CExcelStyle(const CExcelStyle& buffer);

    /// Set the style directly, from the buffer
    /// @param buffer const char *, buffer with the Excel style
    void set(const char *buffer);

    /// Set index to style XF record
    /// @param index uint32_t, index to style XF record
    void ixfe(uint32_t index);

    /// Get index to style XF record
    uint32_t ixfe() const;

    /// Set built-in style number.
    /// @param biStyle uint32_t, built-in style number
    void builtInStyle(uint32_t biStyle) {
        m_buffer[2] = char(biStyle);
    }

    /// Get built-in style number.
    uint32_t builtInStyle() const {
        return m_buffer[2];
    }

    /// Set the level of outline style (Row or Column)
    /// @see builtInStyle
    void outlineStyleLevel(uint32_t osLevel) {
        m_buffer[3] = char(osLevel);
    }

    /// Get the level of outline style (Row or Column)
    /// @see builtInStyle
    uint32_t outlineStyleLevel() const {
        return m_buffer[3];
    }

    /// Returns pointer to the internal data buffer
    const char *data() const {
        return m_buffer;
    }
};

/// The simple list of the STYLE records
class CExcelStyles : public std::vector<CExcelStyle> {
public:
    /// Default constructor
    CExcelStyles() {}

    /// Creates a list of the minimal set of styles
    void init();
};
/// @}
}
#endif
