/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          ccheckbuttons.h  -  description
                             -------------------
    begin                : Thi Dec 12 2002
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CCheckButtons_H__
#define __CCheckButtons_H__

#include <FL/Fl_Check_Button.H>
#include <sptk3/gui/CLayoutClient.h>
#include <sptk3/gui/CRadioButtons.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief The check button.
///
/// Implements the SPTK check button that understands and reports
/// its size requirements, and can be a layout client.
class SP_EXPORT CCheckButton : public Fl_Check_Button, public CLayoutClient {

protected:

    /// Draws the check button
    virtual void draw();

public:

    /// SPTK-style constructor
    /// @param label const char *, the widget label
    /// @param layoutSize int, the size of widget in layout
    /// @param layoutAlignment CLayoutAlign, widget align in the layout
    CCheckButton(const char *label,int layoutSize,CLayoutAlign layoutAlignment)
            : Fl_Check_Button(0,0,10,10,label), CLayoutClient(this,layoutSize,layoutAlignment) {}

    /// A special handle method. See FLTK Fl_Widget::handle() for more information.
    /// @param event int, FLTK event type
    /// @return true if the event was processed
    int handle(int event);

    /// Reports the preferred size of the widget based on offered size.
    /// @param w int, input/output desirable widget width
    /// @param h int, input/output desirable widget heigth
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);
};

/// @brief A set of check buttons.
///
/// Implements a group of check buttons with an optional CInput for the 'Other' choice.
/// Data connector returns the current state as a string with selected buttons names separated with '|' character.
class SP_EXPORT CCheckButtons : public CRadioButtons {
    static void checkButtonsCallback(Fl_Widget *,void *);
protected:
    virtual Fl_Button *createButton(const char *label,int sz=10,CLayoutAlign layoutAlignment=SP_ALIGN_TOP);
public:

    /// SPTK-style constructor
    /// @param label const char *, the widget label
    /// @param layoutSize int, the size of widget in layout
    /// @param layoutAlignment CLayoutAlign, widget align in the layout
    CCheckButtons(const char * label=0,int layoutSize=20,CLayoutAlign layoutAlignment=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// FLTK-style constructor
    /// @param x int, widget x-coordinate
    /// @param y int, widget y-coordinate
    /// @param w int, widget width
    /// @param h int, widget height
    /// @param label int, optional widget label
    CCheckButtons(int,int,int,int,const char * = 0);
#endif

    /// Gets the currently selected buttons argument()s
    void getSelections(CIntList& selection) const;

    /// Sets the currently selected buttons matching with argument()s
    void setSelections(const CIntList& selection);

    /// Universal data connection reader.
    /// @returns the list of selected button labels as pipe ('|') separated strings
    virtual CVariant data() const;

    /// Universal data connection writer.
    /// @param v CVariant, sets the selection from the list of labels as pipe ('|') separated strings
    virtual void     data(const CVariant v);

    /// Returns an internal SPTK control type information
    virtual CControlKind kind() const {
        return DCV_CHECKBUTTONS;
    }

    /// Returns an internal SPTK control class name information
    virtual std::string className() const {
        return "check_buttons";
    }
    
    /// @brief Creates chack buttons group based on the XML node information
    static CLayoutClient* creator(CXmlNode *node);
};
/// @}
}
#endif
