/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          ccontrollistcslists.h  -  description
                             -------------------
    begin                : 12 June 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CCONTROLLIST_H__
#define __CCONTROLLIST_H__

#include <sptk3/istring.h>
#include <sptk3/gui/CControl.h>
#include <map>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// String to Control map.
/// Uses case-insensitive strings and pointers to CControl.
/// Strings are control field names.
typedef std::map<istring,CControl *> CStringControlMap;

/// List of CControl object pointers in Fl_Group
class CControlList : public CStringControlMap {
protected:
    /// Scan group to find all CControl objects inside,
    /// including children groups
    void scanControls(const Fl_Group *group);
public:
    /// Constructor
    CControlList() {}

    /// Adds a CControl pointer into the list
    void add
        (CControl *control);

    /// Adds a list of CControl pointers into the list
    void add
        (const CControlList& l);

    /// Adds a list of CControl pointers from the group into the list
    void add
        (const Fl_Group& g);

    /// Removes CControl pointer from the list
    void remove
        (CControl *control) {
        erase(control->fieldName());
    }

    /// Removes a list of CControl pointers from the list
    void remove
        (const CControlList& l);

    /// Returns true if the control for the same field name exists
    bool contains(CControl *control) const {
        if (!control)
            return false;
        return find(control->fieldName()) != end();
    }

    /// Assignment operation
    CControlList& operator =  (const Fl_Group& g)     {
        clear();
        add
            (g);
        return *this;
    }
    /// Assignment operation
    CControlList& operator =  (const CControlList& l) {
        clear();
        add
            (l);
        return *this;
    }

    /// Addition operation
    CControlList& operator << (CControl *c)           {
        add
            (c);
        return *this;
    }

    /// Addition operation
    CControlList& operator << (const Fl_Group& g)     {
        add
            (g);
        return *this;
    }

    /// Addition operation
    CControlList& operator << (const CControlList& l) {
        add
            (l);
        return *this;
    }

    /// Sends reset() signal to all the widgets in the list
    void reset();
};
/// @}
}
#endif
