/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDateControl.h - description
                             -------------------
    begin                : Mar 19 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CDATECONTROL_H__
#define __CDATECONTROL_H__

#include <FL/Fl_Group.H>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

class CSmallButton;
class CInput_;

/// @brief Date input widget.
///
/// A masked date input with a small drop-down button inside.
/// Clicking on the button displays the calendar window.
class CDateControl : public Fl_Group {

    /// Drop-down button
    CSmallButton   *m_button;

    /// Input box
    CInput_        *m_input;

protected:

    /// Internal callback
    static void date_cb(Fl_Widget *w,void *data);
public:

    /// Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label const char *, label
    CDateControl(int x,int y,int w,int h,const char *label=0L);

    /// Returns internal small button pointer
    CSmallButton *button() {
        return m_button;
    }

    /// Returns internal input box
    CInput_ *input() {
        return m_input;
    }

    /// Resizes the control.
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    void resize(int x,int y,int w,int h);

    /// Computes the best size for the control
    /// @param w int&, control width
    /// @param h int&, control height
    /// @param editorWidth int& the width of date input
    /// @param buttonWidth int& the width of button
    void preferredSize(int& w,int& h,int& editorWidth,int& buttonWidth);

    /// Computes the best size for the control
    /// @param h int&, control height
    void preferredHeight(int& h);
};
/// @}
}
#endif
