/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDateIntervalInput.h  -  description
                             -------------------
    begin                : Fri Oct 27 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CDATEINTERVALINPUT_H__
#define __CDATEINTERVALINPUT_H__

#include <sptk3/sptk.h>

#include <sptk3/CDateTime.h>
#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

class CPopupCalendar;
class CDateControl;

/// @brief Date interval input widget
///
/// Masked date interval input with the drop-down calendar
class SP_EXPORT CDateIntervalInput : public CInput {
    typedef class CInput inherited;

    CDateControl   *m_firstDateInput;   ///< First date input widget
    CDateControl   *m_secondDateInput;  ///< Second date input widget
    std::string     m_separator;            ///< Separator to display
    CPopupCalendar *m_calendarWindow;   ///< Pop-up calendar window

    /// @brief Constructor initializer
    void ctor_init();

    /// @brief Internal callback function
    static void intervalCalendarButtonPressed(Fl_Widget *btn,void *data);

protected:

    /// @brief Shows drop-down calendar for the particular date input that owns the button btn
    /// @param btn Fl_Widget *, button of the date input widget
    void showCalendar(Fl_Widget *btn);

public:

    /// @brief Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CDateIntervalInput(const char *label=0,int layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CDateIntervalInput(int x,int y,int w,int h,const char *label=0);
#endif

    /// @brief Sets the separator to display between date inputs
    /// @param s std::string, separator value
    void separator(std::string s);

    /// @brief Returns the separator between date inputs
    std::string separator() {
        return m_separator;
    }

    /// @brief Returns the date from the first date input - the beginning of interval
    CDateTime beginOfInterval() const;

    /// @brief Sets the date in the first date input - the beginning of interval
    /// @param dt const CDateTime, date and time value for the first control
    void      beginOfInterval(const CDateTime dt);

    /// @brief Returns the date from the second date input - the end of interval
    CDateTime endOfInterval() const;

    /// Sets the date from the second date input - the end of interval
    /// @param dt const CDateTime, date and time value for the second control
    void      endOfInterval(const CDateTime dt);

    /// @brief Returns the control kind, SPTK-style RTTI
    /// @see CControlKind for more information
    virtual CControlKind kind() const {
        return DCV_DATEINTERVAL;
    }

    /// @brief Returns the control class name, SPTK-style RTTI
    virtual std::string className() const {
        return "date_interval_input";
    }

    /// @brief Universal data connection, returns data from control
    virtual CVariant data() const;

    /// @brief Universal data connection, sets data from control
    virtual void     data(const CVariant v);

    /// @brief Resizes the control and inside widgets.
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    virtual void     resize(int x,int y,int w,int h);

    /// @brief Returns true if the control state is valid, and the data is inside the limits
    /// if applicable
    virtual bool     valid() const;

    /// @brief Creates a widget based on the XML node information
    static CLayoutClient* creator(CXmlNode *node);
};
/// @}
}
#endif
