/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cdropdownbox.h  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CDROPDOWNBOX_H__
#define __CDROPDOWNBOX_H__
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Box.H>
#include <sptk3/gui/CPopupWindow.h>
#include <sptk3/gui/CDBListView.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

class CDatabase;
class CParam;

/// @brief Drop-down list view.
///
/// A special derivation of CDBListView to support CDBDropDownList class.
class SP_EXPORT CDBDropDownListView : public CDBListView {

protected:

    /// Internal mouse click callback
    void item_clicked(int mouse_btn);

    /// Internal focus exits callback
    void onExit();

public:

    /// Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CDBDropDownListView(const char *label,int layoutSize,CLayoutAlign layoutAlign) : CDBListView(label,layoutSize,layoutAlign) {}

#ifdef __COMPATIBILITY_MODE__
    /// Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    CDBDropDownListView(int x,int y,int w,int h) : CDBListView(x,y,w,h) {}
#endif

}
;

/// @brief Drop-down window with CListView widget inside.
///
/// Designed specifically for CComboBox.
class SP_EXPORT CDBDropDownList : public CPopupWindow {
    friend class CBaseListBox;
    friend class CDBDropDownListView;
protected:
    /// Special version of handle() function
    int  handle(int);
public:
    /// Modified list view widget
    CDBDropDownListView *listView;

    /// Constructor
    /// @param w int, window width
    /// @param h int, window height
    /// @param label const char *, window caption
    CDBDropDownList(int w, int h, const char *label = 0);

    /// Resizes window and internal widgets
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    virtual void resize(int x,int y,int w,int h);

    /// Computes the preferred size of the popup window
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);
};
/// @}
}
#endif
