/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CEditor.h  -  description
                             -------------------
    begin                : Tue Jun 17 2003
    copyright            : (C) 2003 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEDITOR_H__
#define __CEDITOR_H__

#include <FL/Fl_Text_Editor.H>

#include <sptk3/gui/CLayoutClient.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Text editor widget
///
/// Class CEditor is the Fl_Text_Editor combined with the layout client.
/// It has a modified handle() function to track the current cursor position.
/// It also allocates the text buffer in constructor and destroys it in destructor.
class CEditor : public Fl_Text_Editor, public CLayoutClient {

    /// Constructor initializer
    void ctor_init();

    int  m_lastCursorPosition;   ///< Last cursor position in bytes, from the beginning of the text buffer
    bool m_wrapMode;             ///< Current wrap mode
public:

    /// Constructor in SPTK style
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CEditor(int layoutSize=100,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CEditor(int x, int y, int w, int h, const char *label);
#endif

    /// Destructor
    ~CEditor();

    /// Retruns internal text buffer
    Fl_Text_Buffer *textBuffer() {
        return mBuffer;
    }

    /// Retruns internal style buffer
    Fl_Text_Buffer *styleBuffer() {
        return mStyleBuffer;
    }

    /// Computes the optimal widgets size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);

    /// Returns current cursor position
    /// @param row int&, row number
    /// @param col int&, column number
    void cursorRowCol(int& row,int& col);

    /// Returns current wrap mode
    bool wrapMode() {
        return m_wrapMode;
    }

    /// Sets current wrap mode
    void wrapMode(bool wm);

    /// Special version of handle() function
    int handle(int event);
};
/// @}
}
#endif
