/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cevent.h  -  description
                             -------------------
    begin                : Thu Apr 27 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CEVENT_H__
#define __CEVENT_H__

#include <sptk3/sptk.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// Event type constants
enum CEvent {
    CE_NONE,             ///< Empty event (no event)
    CE_DATA_CHANGED,     ///< Control data changed
    CE_FOCUS,            ///< Control received focus
    CE_UNFOCUS,          ///< Control lost focus
    CE_SHOW,             ///< Control visibility changed to show
    CE_HIDE,             ///< Control visibility changed to hide
    CE_KEYBOARD,         ///< Keyboard event in control
    CE_MOUSE_CLICK,      ///< Mouse clicked on control
    CE_MOUSE_DOUBLE_CLICK, ///< Mouse double-clicked on control
    CE_MOUSE_DRAG,       ///< Mouse dragged on control
    CE_MOUSE_MOVE,       ///< Mouse moved over control
    CE_MOUSE_RELEASE,    ///< Mouse released over control
    CE_MOUSE_WHEEL,      ///< Mouse wheel over control
    CE_PROGRESS,         ///< Progress event (if control supports it)
    CE_BUTTON_PRESSED,   ///< Keyboard button pressed
    UC_ADD_ITEM,         ///< List event - item added (if control supports it)
    UC_EDIT_ITEM,        ///< List event - item edited or changed (if control supports it)
    UC_DELETE_ITEM,      ///< List event - item deleted (if control supports it)
    UC_REFRESH,          ///< List event - list refreshed (if control supports it)
    CE_LAST_EVENT        ///< Last event number (no event)
};

/// Event type information
class CEventInfo {
    CEvent        m_event;     ///< Event type
    int32_t       m_argument;  ///< Event argument

public:
    /// Constructor
    /// @param eventType CEvent, event type
    /// @param eventArg CEvent, event argument
    CEventInfo(CEvent eventType=CE_NONE,int32_t eventArg=0) {
        m_event = eventType;
        m_argument = eventArg;
    }

    /// Reports event type
    CEvent  type() const {
        return m_event;
    }

    /// Reports event argument
    int32_t    argument() const {
        return m_argument;
    }
};
/// @}
}

#endif
