/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFloatInput.h  -  description
                             -------------------
    begin                : Fri Oct 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CFLOATINPUT_H__
#define __CFLOATINPUT_H__

#include <sptk3/sptk.h>

#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Floating point value input widget.
///
/// Allows to input only Floating point numbers, and you can also define the input value range for the valid().
class SP_EXPORT CFloatInput : public CInput {
    typedef class CInput inherited;

    /// @brief Minimum value (optional)
    double   m_minValue;

    /// @brief Maximum value (optional)
    double   m_maxValue;

    /// @brief Number of decimal places to show
    int      m_decimalPlaces;

protected:

    /// @brief Saves data to query
    virtual void save(CQuery *);

    /// @brief Returns true if the input data is valid
    virtual bool valid() const;

public:

    /// @brief Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CFloatInput(const char * label=0,int layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CFloatInput(int,int,int,int,const char * = 0);
#endif

    /// @brief Sets limits for the value inside
    /// @param limited bool, true if use the limits
    /// @param min double, minimum value
    /// @param max double, maximum value
    void setLimits(bool limited,double min=0,double max=0);

    /// @brief Returns the number of decimal places
    int     decimalPlaces() const {
        return m_decimalPlaces;
    }

    /// @brief Sets the number of decimal places to show
    void    decimalPlaces(int dp) {
        m_decimalPlaces = dp;
    }

    /// @brief Returns the control kind, SPTK-style RTTI
    /// @see CControlKind for more information
    virtual CControlKind kind() const {
        return DCV_FLOAT;
    }

    /// @brief Returns the control class name, SPTK-style RTTI
    virtual std::string className() const {
        return "float_input";
    }

    /// @brief Creates a widget based on the XML node information
    static CLayoutClient* creator(CXmlNode *node);
};
/// @}
}
#endif
