/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CHtmlBox.h  -  description
                             -------------------
    begin                : Oct 17 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __HTMLBOX_H__
#define __HTMLBOX_H__

#include <sptk3/sptk.h>

#include <FL/Fl_Group.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Input_.H>
#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Simple HTML viewer
///
/// Multiple line HTML text box (read-only)
class SP_EXPORT CHtmlBox : public CInput {
    typedef class CInput inherited;

    /// @brief Constructor initializer
    void ctor_init(const char *label);

public:

    /// @brief Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CHtmlBox(const char * label=0,int layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CHtmlBox(int,int,int,int,const char * = 0);
#endif

    /// @brief Universal data connection, returns data from control
    virtual CVariant data() const;

    /// @brief Universal data connection, sets data from control
    virtual void     data(const CVariant v);

    /// @brief Returns the control kind, SPTK-style RTTI
    /// @see CControlKind for more information
    virtual CControlKind kind() const {
        return DCV_HTMLBOX;
    }

    /// @brief Returns the control class name, SPTK-style RTTI
    virtual std::string className() const {
        return "html";
    }

    /// @brief Returns the input text font type
    virtual Fl_Font  textFont() const;

    /// @brief Sets the input text font type
    virtual void     textFont(Fl_Font f);

    /// @brief Returns the input text font size
    virtual uchar    textSize() const;

    /// @brief Sets the input text font size
    virtual void     textSize(uchar s);

    /// @brief Computes the total height of HTML text inside if the widgets height isn't limited
    int totalHeight() const;

    /// @brief Computes the optimal widget size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);

    /// @brief Creates a widget based on the XML node information
    static CLayoutClient* creator(CXmlNode *node);
};

/// @}
}

#endif
