/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CIcon.h  -  description
                             -------------------
    begin                : Sun Aug 20 2006
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CICON_H__
#define __CICON_H__

#include <sptk3/gui/CPngImage.h>
#include <sptk3/CTar.h>
#include <sptk3/cxml>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Icon size variations
enum CIconSize {
    IS_COMBO_ICON,  ///< Combo box icon, 12x12
    IS_SMALL_ICON,  ///< Small button icon, 16x16
    IS_LARGE_ICON,  ///< Large button icon, 22x22
    IS_DIALOG_ICON ///< Dialog icon, 48x48
};

/// @brief Class CIcon provides a storage of PNG image
class CIcon {
    CPngImage*  m_image;  ///< Icon image
    std::string m_name;   ///< Icon image symbolic name
public:
    bool        m_shared; ///< Is this icon shared (stock) icon
public:
    /// @brief Constructor
    CIcon(std::string name="",CPngImage* image=0L,bool shared=true) {
        m_name = name;
        m_image = image;
        m_shared = shared;
    }

    /// @brief Destructor
    ~CIcon() {
        if (m_image && !m_shared)
            delete m_image;
    }

    /// @brief Loads a PNG image from buffer
    /// @param imageData const CBuffer&, an image data to load PNG image from
    void load(const CBuffer& imageData);

    /// @brief Returns an icon image
    CPngImage* image() {
        return m_image;
    }

    /// @brief Returns an icon symbolic name
    const std::string name() const {
        return m_name;
    }

    /// @brief Sets the image pointer
    /// @param img CPngImage*, an image pointer
    /// @param shared bool, if false then memory should be maintained by the class
    void image(CPngImage* img,bool shared) {
        if (m_image && !m_shared)
            delete m_image;
        m_image = img;
        m_shared = shared;
    }
};

/// @brief A map of theme icon names to theme icons
class CIconMap : public std::map<std::string,CIcon*> {
    bool    m_shared;
public:
    /// @brief Default constructor
    /// @param shared bool, if true then memory, allocated for the icons, isn't managed
    CIconMap(bool shared=false) { m_shared = shared; }

    /// @brief Destructor
    ~CIconMap() {
        clear();
    }

    /// @brief Removes all the allocated images
    void clear();

    /// @brief Inserts a new icon into icon map
    /// @param icon CIcon*, a pointer to an icon
    void insert(CIcon* icon);

    /// @brief Loads a collection of images using information from XML node
    ///
    /// Any incorrect image information is ignored.
    /// @param tar CTar&, the loaded tar archive where images are located
    /// @param node CXmlNode*, an XML node describing the theme icon images
    void load(CTar& tar,CXmlNode* node);
};

/// @}
}
#endif
