/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDataControl.h  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CINTEGERINPUT_H__
#define __CINTEGERINPUT_H__

#include <sptk3/sptk.h>

#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Integer value input widget.
///
/// Allows to input only integers, and you can also define the input value range for the valid().
class SP_EXPORT CIntegerInput : public CInput {
    typedef class CInput inherited;

    /// @brief Minimum value (optional)
    int        m_minValue;

    /// @brief Maximum value (optional)
    int        m_maxValue;

protected:

    /// @brief Saves data to query
    virtual void save(CQuery *);

    /// @brief Returns true if the input data is valid
    virtual bool valid() const;

public:

    /// @brief Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CIntegerInput(const char * label=0,int layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CIntegerInput(int x,int y,int w,int h,const char *label=0);
#endif

    /// @brief Sets limits for the value inside
    /// @param limited bool, true if use the limits
    /// @param min int, minimum value
    /// @param max int, maximum value
    void             setLimits(bool limited,int min=0,int max=0);

    /// @brief Returns the control kind, SPTK-style RTTI
    /// @see CControlKind for more information
    virtual CControlKind kind() const {
        return DCV_INTEGER;
    }

    /// @brief Returns the control class name, SPTK-style RTTI
    virtual std::string className() const {
        return "integer_input";
    }

    /// @brief Creates a widget based on the XML node information
    static CLayoutClient* creator(CXmlNode *node);
};
/// @}
}
#endif
