/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          clistviewrows.h  -  description
                             -------------------
    begin                : Tue Jul 23 2002
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CLISTVIEWROWS_H__
#define __CLISTVIEWROWS_H__

#include <sptk3/CVariant.h>
#include <sptk3/CPackedStrings.h>
#include <sptk3/CIntList.h>
#include <sptk3/CVariant.h>

#include <vector>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

class CListView;

/// The list of CPackedStrings * used inside CListView as the list of rows.
class SP_EXPORT CListViewRows {
    typedef CPackedStrings * PPackedStrings;
    typedef std::vector<PPackedStrings> CPSVector;
    friend class CListView;
private:
    int32_t      m_sortColumn;      ///< The sort column number
    CVariantType m_sortColumnType;  ///< The type of the sort column - determines the sort algorithm
    bool         m_sortAscending;   ///< Sort direction - ascending/descending
protected:
    uint32_t     m_fullHeight;      ///< The summary height of the rows
    CPSVector    m_rows;            ///< The list of the rows

    /// The sort column number for the sort functions
    static int currentSortColumn;

    /// The string data compare function
    static bool compare_strings(const PPackedStrings&,const PPackedStrings&);

    /// The integer data compare function
    static bool compare_integers(const PPackedStrings&,const PPackedStrings&);

    /// The float data compare function
    static bool compare_floats(const PPackedStrings&,const PPackedStrings&);

    /// The date data compare function
    static bool compare_dates(const PPackedStrings&,const PPackedStrings&);

    /// The date and time data compare function
    static bool compare_datetimes(const PPackedStrings&,const PPackedStrings&);
public:

    /// Default constructor
    CListViewRows();

    /// Destructor
    ~CListViewRows();

    /// destroys all the rows and removes them from the list
    void     clear();

    /// Truncates a row list
    /// @param count uint32_t, the maximum list size
    void     truncate(uint32_t count);

    /// Removes a row
    /// @param index uint32_t, row index
    void     remove(uint32_t index);

    /// Adds the new row.
    /// @param row CPackedStrings *, the new row pointer
    /// @returns row index
    uint32_t add
        (CPackedStrings *row);

    /// Inserts the new row.
    /// @param index uint32_t, insert position
    /// @param row CPackedStrings *, the new row pointer
    /// @returns row index
    uint32_t insert(uint32_t index,CPackedStrings *row);

    /// Updates the row by replacing it with a new one.
    /// @param index uint32_t, row index
    /// @param row CPackedStrings *, the new row pointer
    /// @returns row index
    uint32_t update(uint32_t index,CPackedStrings *row);

    /// Row access by row index
    /// @param index uint32_t, row index
    /// @returns row pointer, or 0L if index is out of range
    CPackedStrings* operator [] (uint32_t index) const {
        if (index < m_rows.size())
            return (CPackedStrings *)m_rows[index];
        return 0L;
    }

    /// Finds the row
    /// @param row CPackedStrings *, the row pointer
    /// @returns the row index, or -1 if not found
    int32_t indexOf(CPackedStrings *row) const;

    /// Returns the row count
    uint32_t size() const {
        return m_rows.size();
    }

    /// Sorts rows
    void     sort();

    /// Returns the sort column number
    int32_t sortColumn() const {
        return m_sortColumn;
    }

    /// Sets the sort column
    /// @param column int, sort column number (-1 means 'unsorted')
    /// @param ctype CVariantType, the sort column data type
    /// @param sortNow bool, true if you want to sort immediatedly
    void sortColumn(int column,CVariantType ctype,bool sortNow);

    /// Returns the sort direction - ascending/descending
    bool sortAscending() const {
        return m_sortAscending;
    }

    /// Sets the sort direction - ascending/descending
    /// @param ascending bool, sort direction - ascending/descending
    /// @param sortNow bool, true if you want to sort immediatedly
    void sortAscending(bool ascending,bool sortNow);

    /// Returns summary height of all the rows
    int32_t fullHeight() const {
        return m_fullHeight;
    }
};
/// @}
}
#endif
