/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CListViewSelection.h  -  description
                             -------------------
    begin                : Jul 14 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CLISTVIEWSELECTION_H__
#define __CLISTVIEWSELECTION_H__

#include <sptk3/CPackedStrings.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

#define CLV_SELECTED       1    ///< List view item is selected
#define CLV_CHECKED        2    ///< List view item is checked
#define CLV_NOTDISPLAYED   4    ///< List view item is hidden

/// @brief List view & friend items selection
///
/// Allows CListView to work with one or few selected rows
/// without scanning through all the rows in list view for selected rows.
class SP_EXPORT CSelection {
    typedef CPackedStrings * PPackedStrings;
    typedef std::vector<PPackedStrings> CPSVector;

    CPSVector          m_selectedRows;  ///< The list of the rows

public:

    /// Default constructor
    CSelection()  {}

    /// Destructor
    ~CSelection() {
        clear();
    }

    /// Selects a row and adds it to the selection list
    void select(CPackedStrings *row);

    /// Deselects a row and removes it from the selection list
    void deselect(CPackedStrings *row);

    /// Removes the row from the selection list
    void remove
        (CPackedStrings *row);

    /// Deselects all the rows in the selection list and empties the list
    void deselectAll();

    /// Removes all rows from the selection list
    void clear();

    /// Returns the number of selected rows
    uint32_t size() const {
        return m_selectedRows.size();
    }

    /// Element access to the selection, const version only
    /// @param index int, index in the selection list
    CPackedStrings& operator[] (int index) const {
        return *m_selectedRows[index];
    }

    /// Finds an item with a particular key value. Returns NULL if not found.
    /// @param keyValue int, key value
    CPackedStrings *findKey(int keyValue) const;

    /// Finds an item with a particular caption. Returns NULL if not found.
    /// @param caption CString, item caption (a string in the first column)
    CPackedStrings *findCaption(std::string caption) const;
};
/// @}
}
#endif
