/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMenuBar.h  -  description
                             -------------------
    begin                : Wed Jun 18 2003
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CMENUBAR_H__
#define __CMENUBAR_H__

#include <FL/Fl_Menu_Bar.H>
#include <sptk3/gui/CLayoutClient.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Menu bar
///
/// Extended version of FLTK's standard Fl_Menu_Bar.
/// Added the CLayoutClient interface and preferred size computations.
class CMenuBar : public Fl_Menu_Bar, public CLayoutClient {
public:

    /// Default constructor
    CMenuBar() : Fl_Menu_Bar(0,0,10,10), CLayoutClient(this,10,SP_ALIGN_TOP) {
        box(FL_THIN_UP_BOX);
    }

    /// Computes the optimal menu size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    virtual bool preferredSize(int& w,int& h) {
        h = textsize() + 10;
        return false;
    }

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node) {
       CMenuBar* widget = new CMenuBar;
       widget->load(node,LXM_LAYOUTDATA);
       return widget;
    }

    /// @brief Returns widget class name (internal SPTK RTTI).
    virtual std::string className() const {
        return "menu";
    }
};
/// @}
}

#endif
