/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMessageDialog.h  -  description
                             -------------------
    begin                : Thu Oct 28 2004
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CMESSAGEDIALOG_H__
#define __CMESSAGEDIALOG_H__

#include <sptk3/gui/CDialog.h>
#include <sptk3/gui/CDataControl.h>

#include <string>

#include <FL/Fl_Pixmap.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief A Yes/No dialog box
///
/// Creates a dialog with the text or HTML text of question (message) and two buttons (Ok/Cancel)
class CAskDialog : public CDialog {
    Fl_Box    *m_imageBox; ///< The icon box
    CHtmlBox  *m_textBox;  ///< The HTML box widget to show the message text
protected:
    CInput   *m_inputBox; ///< Input box, optionally shown
public:
    /// Constructor
    /// @param label const char *, window caption
    /// @param w int, window width
    CAskDialog(const char *label,int w=400);

    /// Shows message and waits for the user
    /// @param msg const char *, message to show
    bool execute(std::string msg);

    /// Sets icon for the window
    /// @param image Fl_Pixmap *, icon to use, suggested size is 48x48 XPM
    void icon(Fl_Image *image) {
        m_imageBox->image(image);
    }
};

/// @brief An information dialog box
///
/// Creates a dialog with the text or HTML text of message and one button (Ok)
class CMessageDialog : public CAskDialog {
public:
    /// Constructor
    /// @param label const char *, window caption
    /// @param w int, window width
    CMessageDialog(const char *label,int w=400) : CAskDialog(label,w) {
        m_cancelButton->hide();
    }
};

/// @brief A single input Yes/No dialog box
///
/// Creates a dialog with the text or HTML text of question (message) and two buttons (Ok/Cancel)
class CInputDialog : public CAskDialog {
public:
    /// Constructor
    /// @param label const char *, window caption
    /// @param w int, window width
    CInputDialog(const char *label,int w=400) : CAskDialog(label,w) {
        m_inputBox->show();
    }

    /// Shows message and waits for the user input
    /// @param msg std::string, message to show
    /// @param inputText std::string&, string to input
    bool execute(std::string msg,std::string& inputText);
};

/// @brief A Yes/No dialog box function
///
/// Creates a dialog with the text or HTML text of question (message) and two buttons (Ok/Cancel)
int spAsk(std::string message);

/// @brief A warning information dialog box warning function
///
/// Creates a dialog with the text or HTML text of message and one button (Ok)
int spWarning(std::string message);

/// @brief An error information dialog box function
///
/// Creates a dialog with the text or HTML text of message and one button (Ok)
int spError(std::string message);

/// @brief An information dialog box function
///
/// Creates a dialog with the text or HTML text of message and one button (Ok)
int spInformation(std::string message);

/// @brief A single input Yes/No dialog box
///
/// Creates a dialog with the text or HTML text of question (message),
/// two buttons (Ok/Cancel), and input box
int spInput(std::string message,std::string& inputText);

/// @}
}
#endif
