/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CProgressBar.h  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CPROGRESSBAR_H__
#define __CPROGRESSBAR_H__

#include <sptk3/gui/CDataControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Progress bar widget
///
/// The extended version of FLTK's standard Fl_Progress.
/// Added the CLayoutClient interface and preferred size computations.
class CProgressBar : public CInput {

    /// Constructor initializer
    /// @param label const char *, widget label
    void ctor_init(const char *label);

public:
    /// Constructor in SPTK style
    /// @param lbl const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CProgressBar(const char *lbl,int layoutSize=20,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char * label
    CProgressBar(int x,int y,int w,int h,const char * label=0);
#endif

    /// Computes the optimal progress bar size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);

    /// Defines the progress bar minimum value
    /// The default is 0
    void minimum(float min);

    /// Returnes the progress bar minimum value
    float minimum() const;

    /// Defines the progress bar maximum value
    /// The default is 100
    void maximum(float max);

    /// Returnes the progress bar maximum value
    float maximum() const;

    /// Sets progress bar units, default is "%"
    void units(const char *un);

    /// Returns progress bar units
    std::string units() const;

    /// Show the progress text or not?
    /// The default is true
    void showText(bool flag=true);

    /// Universal data connection, returns data from date control.
    /// Data is returned as VAR_FLOAT.
    virtual CVariant data() const;

    /// Universal data connection, sets data to date control
    virtual void     data(const CVariant v);

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node);
};
/// @}
}
#endif
