/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRadioButtons.h  -  description
                             -------------------
    begin                : Thi Dec 12 2002
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CRadioButtons_H__
#define __CRadioButtons_H__

#include <sptk3/sptk.h>
#include <sptk3/gui/CButtonGroup.h>
#include <string>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Radio buttons widget.
///
/// A group of radio buttons with an optional
/// edit entry for the 'Other' choice.
class SP_EXPORT CRadioButtons : public CButtonGroup {
    /// Internal callback function
    static void radioButtonsCallback(Fl_Widget *,void *);
protected:
    /// Creates a radio button and adds it to the group
    virtual Fl_Button *createButton(const char *label,int sz=10,CLayoutAlign layoutAlignment=SP_ALIGN_TOP);
public:
    /// SPTK-style constructor
    /// @param label const char *, the widget label
    /// @param layoutSize int, the size of widget in layout
    /// @param layoutAlignment CLayoutAlign, widget align in the layout
    CRadioButtons(const char * label=0,int layoutSize=20,CLayoutAlign layoutAlignment=SP_ALIGN_TOP);
#ifdef __COMPATIBILITY_MODE__
    /// FLTK-style constructor
    /// @param x int, widget x-coordinate
    /// @param y int, widget y-coordinate
    /// @param w int, widget width
    /// @param h int, widget height
    /// @param label int, optional widget label
    CRadioButtons(int,int,int,int,const char * = 0);
#endif
    /// SPTK internal control type
    virtual CControlKind kind() const {
        return DCV_RADIOBUTTONS;
    }

    /// SPTK internal class name
    virtual std::string className() const {
        return "radio_buttons";
    }

    /// Gets the selected button's ID
    int32_t intValue() const;

    /// Sets the selected button by ID
    void intValue(int32_t ID);

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node);
};
/// @}
}
#endif
