/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRect.h  -  description
                             -------------------
    begin                : Fri Feb 25 2005
    copyright            : (C) 2005 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CRECT_H__
#define __CRECT_H__

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// General class to work with rectangles
class CRect {
    int m_x;    ///< rectangle X-coordinate
    int   m_y;  ///< rectangle Y-coordinate
    int   m_w;  ///< rectangle width
    int   m_h;  ///< rectangle height
public:
    /// Default constructor
    /// @param x int, rectangle X-coordinate
    /// @param y int, rectangle Y-coordinate
    /// @param w int, rectangle width
    /// @param h int, rectangle height
    CRect(int x=0,int y=0,int w=0,int h=0) {
        m_x = x;
        m_y = y;
        m_w = w;
        m_h = h;
    }

    /// returns rectangle X-coordinate
    int x() const {
        return m_x;
    }

    /// returns rectangle Y-coordinate
    int y() const {
        return m_y;
    }

    /// returns rectangle width
    int w() const {
        return m_w;
    }

    /// returns rectangle height
    int h() const {
        return m_h;
    }

    /// sets rectangle X and Y coordinates
    /// @param x int, rectangle X-coordinate
    /// @param y int, rectangle Y-coordinate
    void move(int x,int y) {
        m_x = x;
        m_y = y;
    }

    /// sets rectangle's width and height
    /// @param w int, rectangle width
    /// @param h int, rectangle height
    void size(int w,int h) {
        m_w = w;
        m_h = h;
    }

    /// shifts rectangle X and Y coordinates
    /// @param dx int, rectangle X-coordinate shift
    /// @param dy int, rectangle Y-coordinate shift
    void shift(int dx,int dy) {
        m_x += dx;
        m_y += dy;
    }

    /// shrinks rectangle width and height
    /// @param dw int, rectangle width change
    /// @param dh int, rectangle height change
    void shrink(int dw,int dh) {
        m_w -= dw;
        m_h -= dh;
    }

    /// expands rectangle width and height
    /// @param dw int, rectangle width change
    /// @param dh int, rectangle height change
    void expand(int dw,int dh) {
        m_w += dw;
        m_h += dh;
    }
}
;
/// @}
}

#endif
