/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CScrollBar.h  -  description
                             -------------------
    begin                : 29 December 2004
    copyright            : (C) Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/


#ifndef __CSCROLLBAR_H__
#define __CSCROLLBAR_H__

#include <FL/Fl_Scrollbar.h>
#include <sptk3/sptk.h>
#include <sptk3/gui/CLayoutClient.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// Extended version of Scroll Bar - to add theme support
class SP_EXPORT CScrollBar : public Fl_Scrollbar, public CLayoutClient {
protected:
    int m_pushed;   ///< flag to detect pushed button

    /// Custom draw method to support SPTK themes
    void draw();
public:
    /// Constructor in SPTK style
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CScrollBar(int layoutSize=16,CLayoutAlign layoutAlign=SP_ALIGN_RIGHT)
            : Fl_Scrollbar(0,0,10,10), CLayoutClient(this,layoutSize,layoutAlign) {
        m_pushed = 0;
    }

    /// Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    CScrollBar(int x,int y,int w,int h)
            : Fl_Scrollbar(x,y,w,h), CLayoutClient(this,10,SP_ALIGN_NONE) {
        m_pushed = 0;
    }

    /// Computes the optimal progress bar size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);

    /// Custom handle() method - event processing
    int handle(int event);

    /// @brief Returns widget class name (internal SPTK RTTI).
    virtual std::string className() const {
        return "scroll_bar";
    }
};
/// @}
}

#endif
