/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CSplitter.h  -  description
                             -------------------
    begin                : Tue Jan 27 2004
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CSPLITTER_H__
#define __CSPLITTER_H__

#include <sptk3/sptk.h>
#include <sptk3/gui/CDataControl.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Splitter widget
///
/// CSplitter implements 'grab and resize' bar.
/// It works only with SPTK-style widgets derived from CLayoutClient,
/// and placed on a group derived from CLayoutManager (CGroup,CScroll).
class CSplitter : public CBox {
    Fl_Widget      *m_chainedWidget;           /// Pointer on the widget that will be resized
    CLayoutClient  *m_chainedWidgetLayout;     /// Pointer on the CLayoutClient of the resized widget
    bool            m_dragging;                /// The flag indicating the dragging state
    int             m_lastDragX;               /// Where did we grab the bar (x-coordinate) ?
    int             m_lastDragY;               /// Where did we grab the bar (y-coordinate) ?
protected:
    void findChainedControl();                 /// After the splitter is grabbed finds out which widget is resizing
public:
    /// Constructor in SPTK style
    /// @param label const char *, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CSplitter(const char * label=0,int layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

    /// Custom handle() method to process widget-specific events
    int handle(int event);

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node);

    /// @brief Returns widget class name (internal SPTK RTTI).
    virtual std::string className() const {
        return "splitter";
    }
};
/// @}
}
#endif
