/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTabImage.h  -  description
                             -------------------
    begin                : Thu Nov 09 2006
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTABIMAGE_H__
#define __CTABIMAGE_H__

#include <sptk3/gui/CPngImage.h>
#include <sptk3/CTar.h>
#include <sptk3/cxml>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Typee of the tab image
enum CTabImageIndex {
    TI_INACTIVE=0,
    TI_ACTIVE=1,
    TI_BORDER=2
};

/// @brief An image representing one of three required images for theme tabs
class CTabImage {
    CPngImage*  m_image;             ///< Tab image
    std::string m_name;              ///< Logical name
    int         m_leftFrameWidth;    ///< Left frame width
    int         m_rightFrameWidth;   ///< Right frame width
    int         m_topFrameHeight;    ///< Top frame height
    int         m_bottomFrameHeight; ///< Bottom frame height
    CPngImage::CPatternDrawMode m_backgroundDrawMode; ///< Background draw mode
public:
    /// @brief Constructor
    /// @param tar const sptk::CTar&, the theme tar archive
    /// @param tabImageNode const sptk::CXmlNode*, an XML node with the tab theme info
    CTabImage(const sptk::CTar& tar,const sptk::CXmlNode* tabImageNode);

    /// @brief Destructor
    ~CTabImage() {
        if (m_image) delete m_image;
    }

    /// @brief Custom draw method
    void draw(int x,int y,int w,int h);

    /// @brief Tab name
    std::string name() const { return m_name; }
};

/// Tab theme images collection
class CTabImages : protected std::map<std::string,CTabImage*> {
public:
    /// @brief Constructor
    CTabImages() {}

    /// @brief Loads tab images from tar archive using XML node as an index
    /// @param tar const sptk::CTar&, the theme tar archive
    /// @param tabImagesNode const sptk::CXmlNode*, an XML node with the tabs theme info
    void load(const sptk::CTar& tar,const sptk::CXmlNode* tabImagesNode);

    /// @brief Destructor
    ~CTabImages() { clear(); }

    /// @brief Removes all the tab images from the vector
    void clear();

    /// @brief Returns tab image for the name, or NULL if not found
    CTabImage* tabImage(const char* imageName);
};

/// @}
}
#endif
