/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTabs.h  -  description
                             -------------------
    begin                : Tue Feb 26 2002
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTABS_H__
#define __CTABS_H__

#include <sptk3/sptk.h>
#include <sptk3/gui/CGroup.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

class CTabGroup;

/// @brief Tabs widget
///
/// It can show the tabs in several rows if they are too wide to fit.
class SP_EXPORT CTabs : public CGroup {
    CTabGroup*  m_tabs;            ///< Tabs as a separate group
    int32_t     m_autoColorIndex;  ///< The internal index inside tabs auto-color table

    /// auto-color table
    static const Fl_Color AutoColorTable[16];

    void        removeEmptyLastPage();                             ///< Removes an empty last page prior to adding a new page
protected:
    virtual void prepareNewPage(Fl_Group *page,bool autoColor);    ///< Sets default parameters for a new page
public:
    /// @brief Constructor in SPTK style
    /// @param label const char*, label
    /// @param layoutSize int, widget align in layout
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CTabs(const char* label=0L,int32_t layoutSize=10,CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    /// @param label, const char* label
    CTabs(int x,int y,int w,int h,const char* label = 0);
#endif

    /// @brief Destructor
    ~CTabs();

    /// @brief Creates a new CGroup page
    /// @param label const char*, tab and page label
    /// @param autoColor bool, auto-color assignment on/off
    virtual Fl_Group* newPage(const char* label,bool autoColor=false);

    /// @brief Creates a new CScroll page
    /// @param label const char*, tab and page label
    /// @param autoColor bool, auto-color assignment on/off
    virtual Fl_Group* newScroll(const char* label,bool autoColor=false);

    /// @brief Custom draw method
    virtual void draw();

    /// @brief Show/hide tabs flag
    ///
    /// @param show bool, if true the tabs would be visible
    void showTabs(bool show);

    /// @brief Returns currently selected page
    Fl_Group *page();

    /// Selects page
    uint32_t page(Fl_Widget *page);

    /// @brief Returns tabs count
    uint32_t pageCount() const;

    /// @brief Returns selected page number
    uint32_t pageNumber() const;

    /// @brief Sets selected page by number
    /// @param pageNumber uint32_t, a page number to select
    void pageNumber(uint32_t pageNumber);

    /// @brief Removes a page completely
    /// @param page Fl_Group*, page to remove
    void remove(Fl_Group* page);

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node);

    /// @brief Returns widget class name (internal SPTK RTTI).
    virtual std::string className() const {
        return "tabs";
    }
};
/// @}
}
#endif
