/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CToolBar.h  -  description
                             -------------------
    begin                : Mon Jun 19 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CTOOLBAR_H__
#define __CTOOLBAR_H__

#include <sptk3/gui/CGroup.h>
#include <sptk3/gui/CButton.h>

namespace sptk {

/// @addtogroup gui GUI Classes
/// @{

/// @brief Toolbar widget
///
/// The slightely extended CGroup to work as a toolbar.
/// Most of the toolbar functionality isn't implemented yet.
class SP_EXPORT CToolBar : public CGroup {
    bool    m_colapsed; ///< Flag, true if the toolbar is collapsed
public:

    /// @brief Constructor in SPTK style
    /// @param layoutAlign CLayoutAlign, widget align in layout
    CToolBar(CLayoutAlign layoutAlign=SP_ALIGN_TOP);

#ifdef __COMPATIBILITY_MODE__
    /// @brief Constructor in FLTK style
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    CToolBar(int x,int y,int w,int h);
#endif

    /// @brief Creates and adds a new button to the toolbar
    /// @param buttonKind CButtonKind, button type
    /// @param cb Fl_Callback_p, button callback
    /// @param label const char *, button label
    /// @param buttonAlign CLayoutAlign, button align in the toolbar
    /// @see CButton
    CButton *addButton(CButtonKind buttonKind,Fl_Callback_p cb,const char *label = 0,CLayoutAlign buttonAlign=SP_ALIGN_LEFT);

    /// @brief Resizes the group and inside widgets.
    /// @param x int, x-position
    /// @param y int, y-position
    /// @param w int, width
    /// @param h int, height
    virtual void resize(int x,int y,int w,int h);

    /// @brief Computes the optimal group size
    /// @param w int&, input - width offered by the program, output - width required by widget
    /// @param h int&, input - height offered by the program, output - height required by widget
    /// @returns true if the size is stable (doesn't depend on input sizes)
    virtual bool preferredSize(int& w,int& h);

    /// @brief Flag, true if the toolbar is collapsed
    bool collapsed() const {
        return m_colapsed;
    }

    /// @brief Special handle() function
    int handle(int);

    /// @brief Draws the toolbar
    void draw();

    /// @brief Creates a widget based on the XML node information
    /// @param node CXmlNode*, an XML node with widget information
    static CLayoutClient* creator(CXmlNode* node);

    /// @brief Returns widget class name (internal SPTK RTTI).
    virtual std::string className() const {
        return "toolbar";
    }
};
/// @}
}
#endif
