/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          istring.h  -  description
                             -------------------
    begin                : July 19 2005
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __ISTRING_H__
#define __ISTRING_H__

#include <sptk3/sptk.h>
#include <algorithm>

namespace sptk {
/// @addtogroup utility Utility Classes
/// @{

/// @brief Case-insensitive string.
///
/// Lower case string class istring is really useful if we need
/// a case-independent string map
class istring : public std_string {

    /// Converts string to lower case
	void tolower() {
		std::transform(begin(),end(),begin(),(int(*)(int))::tolower);
	}

public:

    /// Default constructor
    istring() : std_string() {}

    /// Copy constructor
    /// @param s const string &, a source string
    istring(const istring& s) : std_string(s.c_str()) {
        tolower();
    }

    /// Constructor
    /// @param s const char *, a source string
    istring(const char *s) : std_string(s) {
        tolower();
    }

    /// Constructor
    /// @param s const string &, a source string
    istring(const std::string& s) : std_string(s) {
        tolower();
    }

    /// Assignment
    /// @param s const char *, a source string
    /// @returns this object
    istring& operator = (const char *s) {
        assign(s);
        return *this;
    }

    /// Assignment
    /// @param s const string &, a source string
    /// @returns this object
    istring& operator = (const std::string& s) {
        assign(s);
        return *this;
    }
};

/// @}

}

#endif
