/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          string_ext.h  -  description
                             -------------------
    begin                : Thu July 14 2005
    copyright            : (C) 2005 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __STRING_EXT_H__
#define __STRING_EXT_H__

#include <sptk3/sptk.h>
#include <string>
#include <vector>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// Converts string to upper case
std::string upperCase(const std::string& str);

/// Converts string to lower case
std::string lowerCase(const std::string& str);

/// Trims string to remove leading and trailing spaces
std::string trim(const std::string& str);

/// Converts int32_t to string
std::string int2string(int32_t);

/// Converts uint32_t to string
std::string int2string(uint32_t);

/// Converts uint64_t to string
std::string int2string(int64_t);

/// Converts uint64_t to string
std::string int2string(uint64_t);

/// Converts string to integer. The optional default value is used
/// for unsuccessful conversion
int string2int(const std::string& str,int defaultValue=0);

/// Replaces all the pattern in str with replacement
std::string replaceAll(const std::string& str,const std::string& pattern,const std::string& replacement);

/// Capitalizes all the words in string
std::string capitalizeWords(const std::string& str);

/// Converts a string vector into CSV string
void join(std::string& dest, const std::vector<std::string> src, std::string separator);

/// Converts a CSV string into string vector
void split(std::vector<std::string> dest, const std::string& src, std::string separator);

/// Converts a CSV string into string vector
void stringToStringVector(const std::string& src,std::vector<std::string> dest,std::string delimitter);

/// @}

}

#endif
