/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlEntities.h  -  description
                             -------------------
    begin                : Tue Jun 27 2006
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CXMLENTITIES_H__
#define __CXMLENTITIES_H__

#include <string>
#include <map>

namespace sptk {

/// @addtogroup XML
/// @{

/// @brief XML entities
///
/// Maps an XML entity string to a presentation string.
class CXmlEntities : public std::map<std::string,std::string> {
public:

    /// @brief Constructor
    CXmlEntities() { }

    /// @brief Removes named entity
    ///
    /// @param name const char *, entity name to remove
    /// @returns true, if operation succeeds
    void removeEntity(const char *name) {
        remove(name);
    }

    /// @brief Adds entity to map
    ///
    /// If entity named 'name' exists already in map, its value is replaced with 'replacement'
    /// @param name const char *, entity to add/change
    /// @param replacement const char *, value that represents entity
    void setEntity(const char *name, const char *replacement) {
        (*this)[name] = replacement;
    }
};
/// @}
}
#endif
