/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlNodeList.h  -  description
                             -------------------
    begin                : Sun May 22 2003
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CXML_NODELIST_H__
#define __CXML_NODELIST_H__

#include <string>
#include <vector>
#include <algorithm>

namespace sptk {

/// @addtogroup XML
/// @{

class CXmlNode;

/// @brief The vector of CXmlNode *
typedef std::vector<CXmlNode *> CXmlNodeVector;

/// @brief XML node list
///
/// The CXmlNodeList interface provides the an ordered collection of nodes,
/// The items in the NodeList are accessible via an integral index, starting from 0.
class CXmlNodeList : public CXmlNodeVector {
public:
    /// @brief Constructor
    CXmlNodeList() {}

    /// @brief Destructor
    ~CXmlNodeList() {
        clear();
    }

    /// @brief Clears the list of XML nodes and releases all the allocated memory
    void clear();

    /// @brief Finds the first node node in the list with the matching name
    /// @param nodeName const char*, a node name
    /// @returns node iterator, or end()
    iterator findFirst(const char* nodeName);

    /// @brief Finds the first node node in the list with the matching name
    /// @param nodeName const std::string&, a node name
    /// @returns node iterator, or end()
    iterator findFirst(const std::string& nodeName);

    /// @brief Finds the first node node in the list with the matching name
    /// @param nodeName const char*, a node name
    /// @returns node iterator, or end()
    const_iterator findFirst(const char* nodeName) const;

    /// @brief Finds the first node node in the list with the matching name
    /// @param nodeName const std::string&, a node name
    /// @returns node iterator, or end()
    const_iterator findFirst(const std::string& nodeName) const;
/*
    /// Replaces the item
    /// @param pos uint32_t, replace position
    /// @param item CXmlNode *, item to insert
    void replace(uint32_t pos, CXmlNode *item);

    /// Removes the item
    /// @param pos uint32_t, remove item position
    void remove(uint32_t pos);

    /// Removes the item
    /// @param item CXmlNode *, item to remove
    bool remove(CXmlNode *item);
*/
}
;
/// @}
}
#endif
