/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlValue.h  -  description
                             -------------------
    begin                : Wed June 21 2006
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CXMLVALUE_H__
#define __CXMLVALUE_H__

#include <sptk3/cxml>
#include <sptk3/CDateTime.h>

namespace sptk {

/// @addtogroup XML
/// @{

class CXmlNode;
class CXmlDoc;

/// @brief XML value
///
/// A string that has converters to and from most popular types
class SP_EXPORT CXmlValue : public std::string {

public:
    /// @brief Default constructor
    ///
    /// Creates an empty XML attribute
    CXmlValue() {}

    /// @brief Constructor
    ///
    /// Creates an XML attribute from character string
    CXmlValue(const char *v) : std::string(v) {}

    /// @brief Constructor
    ///
    /// Creates an XML attribute from std::string
    CXmlValue(std::string v) : std::string(v) {}

    /// @brief Constructor
    ///
    /// Creates an XML attribute from integer
    CXmlValue(int32_t v) {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from integer
    CXmlValue(uint32_t v) {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from int32_t
    CXmlValue(int64_t v)        {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from uint32_t
    CXmlValue(uint64_t v) {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from double
    CXmlValue(double v) {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from bool
    CXmlValue(bool v) {
        operator = (v);
    }

    /// @brief Constructor
    ///
    /// Creates an XML attribute from CDateTime
    CXmlValue(CDateTime v) : std::string(v) {}

    /// @brief Assignment of the value
    ///
    /// @param v bool, a new value
    CXmlValue& operator = (bool v);

    /// @brief Assignment of the value
    ///
    /// @param v int32_t, a new value
    CXmlValue& operator = (int32_t v);

    /// @brief Assignment of the value
    ///
    /// @param v uint32_t, a new value
    CXmlValue& operator = (uint32_t v);

    /// @brief Assignment of the value
    ///
    /// @param v int64_t, a new value
    CXmlValue& operator = (int64_t v);

    /// @brief Assignment of the value
    ///
    /// @param v int64_t, a new value
    CXmlValue& operator = (uint64_t v);

    /// @brief Assignment of the value
    ///
    /// @param v double, a new value
    CXmlValue& operator = (double v);

    /// @brief Assignment of the value
    ///
    /// @param s const std::string, a new value
    CXmlValue& operator = (const std::string& s) {
        assign(s);
        return *this;
    }

    /// @brief Assignment of the value
    ///
    /// @param s const char *, a new value
    CXmlValue& operator = (const char *s)        {
        assign(s);
        return *this;
    }

    /// @brief Assignment of the value
    ///
    /// @param s CDateTime, a new value
    CXmlValue& operator = (CDateTime s)          {
        assign(s);
        return *this;
    }

    /// @brief Returns the value with the conversion
    operator uint32_t () const                  {
        return atol(c_str());
    }

    /// @brief Returns the value with the conversion
    operator int32_t () const                           {
        return atol(c_str());
    }

    /// @brief Returns the value with the conversion
    operator uint64_t () const                  {
#ifdef __UNIX_COMPILER__
        return atoll(c_str());
#else
        return _atoi64(c_str());
#endif
    }

    /// @brief Returns the value with the conversion
    operator int64_t () const                           {
#ifdef __UNIX_COMPILER__
        return atoll(c_str());
#else
        return _atoi64(c_str());
#endif
    }

    /// @brief Returns the value with the conversion
    operator bool () const;

    /// @brief Returns the value with the conversion
    operator double () const                         {
        return atof(c_str());
    }
};

/// @}
}
#endif
