/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDatabase.cpp  -  description
                             -------------------
    begin                : Wed Dec 15 1999
    copyright            : (C) 1999-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/db/CDatabase.h>
#include <sptk3/db/CQuery.h>
#include <sptk3/CException.h>

#include <algorithm>

using namespace std;
using namespace sptk;

CDatabase::CDatabase(string connectionString)
: CWaiter(), m_connString(connectionString)
{
   m_inTransaction = false;
   m_log = 0;
}

CDatabase::~CDatabase() {
   // To prevent the exceptions, if the database connection 
   // is terminated already
   try {
      while (m_queryList.size()) {
         CQuery *query = (CQuery *)m_queryList[0];
         query->disconnect();
      }
   }
   catch (...) {
   }
}

bool CDatabase::linkQuery(CQuery *q) {
   m_queryList.push_back(q);
   return true;
}

bool CDatabase::unlinkQuery(CQuery *q) {
   CQueryVector::iterator itor = find(m_queryList.begin(),m_queryList.end(),q);
   m_queryList.erase(itor);
   return true;
}

void CDatabase::openDatabase(string newConnectionString) throw (CException) {
   notImplemented("openDatabase");
}

void CDatabase::open(string newConnectionString) throw (CException)  {
   clearStatistics();
   openDatabase(newConnectionString);
   if (m_log) *m_log << "Opened database: " << m_connString << endl;
}

void CDatabase::closeDatabase() throw(CException) {
   notImplemented("closeDatabase");
}

void CDatabase::close() throw (CException) {
   if (active()) {
      if (m_inTransaction) {
         rollbackTransaction();
         m_inTransaction = false;
      }
      for (uint32_t i = 0; i < m_queryList.size(); i++)
         m_queryList[i]->closeQuery(true);
      closeDatabase();
      if (m_log) *m_log << "Closed database: " << m_connString << endl;
   }
}

void* CDatabase::handle() const {
   notImplemented("handle");
   return 0;
}

bool CDatabase::active() const {
   notImplemented("active");
   return true;
}

void CDatabase::beginTransaction() throw(CException) {
   if (m_log) *m_log << "Begin transaction" << endl;
   driverBeginTransaction(); 
}

void CDatabase::commitTransaction() throw(CException) { 
   if (m_log) *m_log << "Commit transaction" << endl;
   driverEndTransaction(true); 
}

void CDatabase::rollbackTransaction() throw(CException) { 
   if (m_log) *m_log << "Rollback transaction" << endl;
   driverEndTransaction(false); 
}

//-----------------------------------------------------------------------------------------------

string CDatabase::queryError(const CQuery *query) const {
   notImplemented("queryError");
   return "";
}

void CDatabase::querySetAutoPrep(CQuery *q,bool pf) { 
   q->m_autoPrepare = pf; 
}

void CDatabase::querySetStmt(CQuery *q,void *stmt)  { 
   q->m_statement = stmt; 
}

void CDatabase::querySetConn(CQuery *q,void *conn)  { 
   q->m_connection = conn;
}

void CDatabase::querySetPrepared(CQuery *q,bool pf) { 
   q->m_prepared = pf; 
}  

void CDatabase::querySetActive(CQuery *q,bool af)   { 
   q->m_active = af; 
}    

void CDatabase::querySetEof(CQuery *q,bool eof)     {
   q->m_eof = eof; 
} 

void CDatabase::queryAllocStmt(CQuery *query) {
   notImplemented("queryAllocStmt");
}

void CDatabase::queryFreeStmt(CQuery *query) {
   notImplemented("queryFreeStmt");
}

void CDatabase::queryCloseStmt(CQuery *query) {
   notImplemented("queryCloseStmt");
}

void CDatabase::queryPrepare(CQuery *query) {
   notImplemented("queryPrepare");
}

void CDatabase::queryUnprepare(CQuery *query) {
   queryFreeStmt(query);
}

void CDatabase::queryExecute(CQuery *query) {
   notImplemented("queryExecute");
}

int CDatabase::queryColCount(CQuery *query) {
   notImplemented("queryColCount");
   return 0;
}

void CDatabase::queryColAttributes(CQuery *query,int16_t column,int16_t descType,int32_t& value) {
   notImplemented("queryColAttributes");
}

void CDatabase::queryColAttributes(CQuery *query,int16_t column,int16_t descType,char *buff,int32_t len) {
   notImplemented("queryColAttributes");
}

void CDatabase::queryBindParameters(CQuery *query) {
   notImplemented("queryBindParameters");
}

void CDatabase::queryOpen(CQuery *query) {
   notImplemented("queryOpen");
}

void CDatabase::queryFetch(CQuery *query) {
   notImplemented("queryFetch");
}

void CDatabase::notImplemented(const char *methodName) const {
   throw CException("Method '"+string(methodName)+"' is not supported by this database driver.");
}

void *CDatabase::queryHandle(CQuery *query) const { 
   return query->m_statement;
}

void CDatabase::queryHandle(CQuery *query,void *handle) { 
   query->m_statement = handle; 
}

void CDatabase::logAndThrow(string method,string error) throw(CException) {
   string errorText("Exception in " + method + ": " + error);
   if (m_log) *m_log << "errorText" << endl;
   throw CException(errorText);
}

void CDatabase::clearStatistics() {
   m_queryStatisticMap.clear();
}

void CDatabase::addStatistics(const std::string& location, double totalDuration, unsigned totalCalls,const std::string& sql) {
   CCallStatisticMap::iterator itor = m_queryStatisticMap.find(location);
   if (itor != m_queryStatisticMap.end()) {
      CCallStatistic& stat = itor->second;
      stat.m_duration += totalDuration;
      stat.m_calls += totalCalls;
      stat.m_sql = sql;
   } else {
      CCallStatistic stat(totalDuration, totalCalls, sql);
      m_queryStatisticMap[location] = stat;
   }
}
