/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDatabaseField.cpp  -  description
                             -------------------
    begin                : Wednesday November 2 2005
    copyright            : (C) 2005-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdlib.h>
#include <sptk3/db/CDatabaseField.h>

using namespace std;
using namespace sptk;

CDatabaseField::CDatabaseField(const std::string fName,int fieldColumn,int fieldType,CVariantType dataType,int fieldLength,int fieldScale)
: CField(fName.c_str()) {
   m_fldType     = fieldType;
   m_fldColumn   = fieldColumn;
   m_fldSize     = fieldLength;
   m_fldScale    = fieldScale;
   visible       = true;
   displayName   = fName;

   m_data.buffer.size = 0;

   switch (dataType) {
      case VAR_BOOL:
         setBool(false);
         width = 6;
         break;

      case VAR_INT:
         setInteger(0);
         width = 10;
         break;

      case VAR_FLOAT:
         setFloat(0);
         width = 16;
         precision = fieldScale;
         break;

      case VAR_STRING:
         setString("");
         checkSize(fieldLength + 1);
         width = fieldLength;
         break;

      case VAR_TEXT:
         setText("");
         checkSize(fieldLength + 1);
         width = fieldLength;
         break;

      case VAR_BUFFER:
         setBuffer("",1);
         checkSize(fieldLength);
         width = 1;
         break;

      case VAR_DATE:
      case VAR_DATE_TIME:
         setDateTime(0.0);
         CField::dataType(dataType);
         width = 10;
         break;

      default:
         setString("");
         checkSize(fieldLength + 1);
         width = fieldLength;
         break;
   }
}

bool CDatabaseField::checkSize(unsigned sz) {
   if (sz > m_data.buffer.size) {
      unsigned newSize = (sz / 16 + 1) * 16;
      char *p = (char *)realloc(m_data.buffer.data,newSize + 1);
      if (!p)
         throw CException("Can't reallocate a buffer");
      m_data.buffer.data = p;
      m_data.buffer.size = newSize;
   }
   return true;
}
