/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CParams.cpp  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/db/CParams.h>
#include <sptk3/CIntList.h>
#include <sptk3/CException.h>

#include <sptk3/string_ext.h>
#include <ctype.h>

using namespace std;
using namespace sptk;

//---------------------------------------------------------------------------
CParam::CParam(char *name) : CVariant() {
   m_name = lowerCase(name);
   m_timeData = new char[80];
   m_callbackLength = 0;
}
//---------------------------------------------------------------------------
CParam& CParam::operator = (const CParam& param) {
   setData(param);
   m_bindParamIndexes = param.m_bindParamIndexes;
   return *this;
}
//---------------------------------------------------------------------------
void CParam::bindAdd(uint32_t bindIndex) {
   m_bindParamIndexes.push_back(bindIndex);
}
//---------------------------------------------------------------------------
string CParam::asXML() const {
   string xml("<param ");
   xml += "name='" + m_name + "' ";
   xml += "type='" + int2string(m_dataType) + "' ";
   xml += "null='" + int2string(m_null) + "' ";
   if (m_dataType & (VAR_STRING|VAR_TEXT)) {
      string txt = replaceAll(replaceAll(m_data.buffer.data,"\n","\\n"),"\r","");
      xml += "value='" + txt + "' ";
   } else {
      xml += "value='" + asString() + "' ";
   }
   xml += "size='" + int2string((uint32_t)m_dataSize) + "' />";
   return xml;
}
//---------------------------------------------------------------------------
void CParam::setString(const char * value,uint32_t maxlen) {
    uint32_t valueLength;
    if (maxlen) valueLength = maxlen;
    else        valueLength = (uint32_t) strlen(value);
    m_null = false;
    if (dataType() == VAR_STRING && m_data.buffer.size >= valueLength+1) {
        if (value) {
            strncpy(m_data.buffer.data,value,valueLength);
            m_data.buffer.data[valueLength] = 0;
        } else {
            m_data.buffer.data[0] = 0;
            m_null = true;
        }
    } else {
        releaseBuffers();
        if (value) {
            if (maxlen) {
                dataSize(maxlen);
                m_data.buffer.size = maxlen + 1;
                m_data.buffer.data = (char *)malloc(m_data.buffer.size);
                strncpy(m_data.buffer.data,value,maxlen);
                m_data.buffer.data[maxlen] = 0;
            } else {
                dataSize(valueLength);
                m_data.buffer.size = dataSize() + 1;
                m_data.buffer.data = strdup(value);
                m_null = false;
            }
        } else {
            m_data.buffer.data = NULL;
            dataSize(0);
            m_data.buffer.size = 0;
            m_null = true;
        }
    }
    dataType(VAR_STRING);
}
//---------------------------------------------------------------------------
CParamList::~CParamList() {
   try {
      clear();
   }
   catch (...) {}
}
//---------------------------------------------------------------------------
void CParamList::clear() {
   for (unsigned i = 0; i < size(); i++) {
      CParam *item = (CParam *)m_items[i];
      delete item;
   }
   m_items.clear();
   m_index.clear();
   m_paramStreamItor = m_items.begin();
}
//---------------------------------------------------------------------------
void CParamList::add(CParam * item) {
   m_items.push_back(item);
   m_index[item->name()] = item;
   m_paramStreamItor = m_items.begin();
}
//---------------------------------------------------------------------------
CParam* CParamList::find(const char *paramName) {
   map<istring,CParam *>::iterator itor = m_index.find(paramName);
   if (itor == m_index.end())
      return 0;
   return itor->second;
}
//---------------------------------------------------------------------------
CParam& CParamList::operator [] (const char *paramName) const {
   map<istring,CParam *>::const_iterator itor = m_index.find(paramName);
   if (itor == m_index.end())
      throw CException("Invalid parameter name: "+string(paramName),__FILE__,__LINE__);
   return *itor->second;
}
//---------------------------------------------------------------------------
void CParamList::remove(uint32_t i) {
   CParam *item = m_items[i];
   m_index.erase(item->name().c_str());
   m_items.erase(m_items.begin()+i);
   delete item;
   m_paramStreamItor = m_items.begin();
}
//---------------------------------------------------------------------------
CParam& CParamList::next() {
   CParam *currentParam = *m_paramStreamItor;
   m_paramStreamItor++;
   if (m_paramStreamItor == m_items.end()) m_paramStreamItor = m_items.begin();
   return *currentParam;
}
//---------------------------------------------------------------------------
void CParamList::enumerate(CParamVector& params) {
    CParamVector::iterator ptor;
    CIntList::iterator btor;
    params.resize(m_items.size()*2);
    if (m_items.empty())
        return;
    uint32_t maxIndex = 0;
    for (ptor = m_items.begin(); ptor != m_items.end(); ptor++) {
        CParam* param = *ptor;
        CIntList& bindIndex = (*ptor)->m_bindParamIndexes;
        for (btor = bindIndex.begin(); btor != bindIndex.end(); btor++) {
            uint32_t index = *btor;
            if (index >= params.size())
                params.resize(index+1);
            params[index] = param;
            if (index > maxIndex)
                maxIndex = index;
        }
    }
    params.resize(maxIndex+1);
}
//---------------------------------------------------------------------------
CParamList& sptk::operator << (CParamList& paramList, const string& data) {
   paramList.next() = data;
   return paramList;
}

CParamList& sptk::operator << (CParamList& paramList, const char *data) {
   paramList.next() = data;
   return paramList;
}

CParamList& sptk::operator << (CParamList& paramList, int data) {
   paramList.next() = data;
   return paramList;
}

CParamList& sptk::operator << (CParamList& paramList, double data) {
   paramList.next() = data;
   return paramList;
}

CParamList& sptk::operator << (CParamList& paramList, CDateTime data) {
   paramList.next() = data;
   return paramList;
}

CParamList& sptk::operator << (CParamList& paramList, const CBuffer& data) {
   paramList.next() = data;
   return paramList;
}
