/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBox.cpp  -  description
                             -------------------
    begin                : Oct 15 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_draw.H>
#include <sptk3/CStrings.h>
#include <sptk3/gui/CBox.h>
#include <sptk3/db/CQuery.h>

using namespace sptk;

//===========================================================================
void CBox::ctor_init(const char *label) {
    m_controlFlags = FGE_MULTILINEENTRY;
    begin();
    m_control = new Fl_Box(x(),y(),w(),h());
    m_control->clear_output();
    end();
    align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_WRAP);
    m_control->callback(CControl::internalCallback);
    m_control->when(FL_WHEN_CHANGED);
    labelWidth(0);
    if (label)
        data(label);
}

CBox::CBox(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CInput("",layoutSize,layoutAlignment,false) {
    ctor_init(label);
}

#ifdef __COMPATIBILITY_MODE__
CBox::CBox(int x,int y,int w,int h,const char *label)
        : CInput(x,y,w,h,"",false) {
    ctor_init(label);
}
#endif

CLayoutClient* CBox::creator(CXmlNode *node) {
    CBox* widget = new CBox("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    widget->dragable( node->getAttribute("drag","N") );
    return widget;
}

void CBox::draw() {
    m_control->callback(callback());
    Fl_Widget::color(m_control->color());
    draw_box();
    m_control->align(Fl_Widget::align());
    m_control->labelcolor(labelColor());
    m_control->label(m_label.c_str());
    m_control->labelfont(labelfont());
    m_control->labelsize(labelsize());
    m_control->labeltype(labeltype());
    m_control->draw();
}

CVariant CBox::data() const {
    return m_label.c_str();
}

void CBox::data(const CVariant v) {
    std::string text = v.asString();
    m_label.resize(text.length());
    if (m_label.length()) {
        unsigned i = 0;
        for (const char *c = text.c_str(); *c; c++) {
            if (*c == '\r')
                continue;
            m_label[i] = *c;
            i++;
        }
        m_label[i] = 0;

        m_label = replaceAll(m_label,"\x0B","  ");
    }
    m_control->label( m_label.c_str() );
}

Fl_Font CBox::textFont() const {
    return m_control->labelfont();
}

void CBox::textFont(Fl_Font f) {
    m_control->labelfont(f);
}

uchar CBox::textSize() const {
    return m_control->labelsize();
}

void CBox::textSize(uchar s) {
    m_control->labelsize(s);
}

unsigned CBox::textAlign() const {
    return m_control->align();
}

void CBox::textAlign(unsigned align) {
    m_control->align(align);
}

int CBox::totalHeight(int ww) const {
    fl_font(m_textFont,m_textSize);
    if (ww < 1)
        ww = w();
    int cw = ww, ch = 0;
    fl_measure(m_label.c_str(), cw, ch);
    return ch;
}

void CBox::resize(int x,int y,int w,int h) {
    m_labelWidth = 0;
    Fl_Group::resize(x,y,w,h);
    m_control->resize(x,y,w,h);
}

bool CBox::preferredSize(int& w,int& h) {
    int cw = 0, ch = 0;
    int dh = Fl::box_dh(box()) * 2;
    int dw = Fl::box_dw(box()) * 2;

    switch (m_layoutAlign) {
    case SP_ALIGN_TOP:
    case SP_ALIGN_BOTTOM:
        if (!(align() & FL_ALIGN_WRAP))
            cw = 0;
        else
            cw = w - dw;
        break;
    case SP_ALIGN_LEFT:
    case SP_ALIGN_RIGHT:
        ch = h - dh;
        break;
    default:
        break;
    }

    fl_font(m_textFont,m_textSize);
    fl_measure(m_label.c_str(), cw, ch);
    int hh = ch + dh;
    int ww = cw + dw;
    if (h < hh)
        h = hh;
    if (w < ww)
        w = ww;
    return false;
}

int CBox::handle(int event) {
    int dx, dy;
    Fl_Window *w;
    switch (event) {
    case FL_PUSH:
        m_xPushed = Fl::event_x();
        m_yPushed = Fl::event_y();
        fireEvent(CE_MOUSE_CLICK,0);
        return 1;
    case FL_DRAG:
        if (m_dragable) {
            dx = Fl::event_x_root();
            dy = Fl::event_y_root();
            w = window();
            w->position(dx - m_xPushed, dy - m_yPushed);
            return 1;
        }
        return 0;
    case FL_RELEASE:
        return 1;
    }
    return Fl_Widget::handle(event);
}
