/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CButton.cpp  -  description
                             -------------------
    begin                : Jan 2, 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>

#include <FL/fl_draw.h>
#include <sptk3/gui/CThemes.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CControl.h>
#include <sptk3/gui/CThemes.h>
#include <sptk3/gui/CLayoutManager.h>

using namespace sptk;
using namespace std;

void CBaseButton::image(CButtonKind buttonKind,CIconSize iconSize,const char *label) {
   const char *name;
   const char *l;
   switch (buttonKind) {
      default:                 name = 0;          l = 0;         break;
      case SP_ADD_BUTTON:      name = "add";      l = "Add";     break;
      case SP_DELETE_BUTTON:   name = "delete";   l = "Delete";  break;
      case SP_EDIT_BUTTON:     name = "edit";     l = "Edit";    break;
      case SP_BROWSE_BUTTON:   name = "browse";   l = "Browse";  break;
      case SP_CALENDAR_BUTTON: name = "calendar"; l = "Date";    break;
      case SP_REFRESH_BUTTON:  name = "refresh";  l = "Refresh"; break;
      case SP_CANCEL_BUTTON:   name = "cancel";   l = "Cancel";  break;
      case SP_HELP_BUTTON:     name = "help";     l = "Help";    break;
      case SP_NO_BUTTON:       name = "no";       l = "No";      break;
      case SP_OK_BUTTON:       name = "ok";       l = "Ok";      break;
      case SP_OPEN_BUTTON:     name = "open";     l = "Open";    break;
      case SP_PRINT_BUTTON:    name = "print";    l = "Print";   break;
      case SP_SAVE_BUTTON:     name = "save";     l = "Save";    break;
      case SP_COPY_BUTTON:     name = "copy";     l = "Insert";  break;
      case SP_LEFT_BUTTON:     name = "left";     l = "Left";    break;
      case SP_NEW_BUTTON:      name = "new";      l = "New";     break;
      case SP_NEXT_BUTTON:     name = "next";     l = "Next";    break;
      case SP_PRINTER_BUTTON:  name = "printer";  l = "Printer"; break;
      case SP_PRIOR_BUTTON:    name = "prior";    l = "Prior";   break;
      case SP_RIGHT_BUTTON:    name = "right";    l = "Right";   break;
      case SP_SEARCH_BUTTON:   name = "search";   l = "Search";  break;
      case SP_SEND_BUTTON:     name = "send";     l = "Send";    break;
      case SP_STEPLEFT_BUTTON: name = "stepleft";  l = "Step Left";  break;
      case SP_STEPRIGHT_BUTTON:name = "stepright"; l = "Step Right"; break;
      case SP_VIEW_BUTTON:     name = "view";     l = "View";    break;
      case SP_HOME_BUTTON:     name = "home";     l = "Home";    break;
      case SP_EXEC_BUTTON:     name = "exec";     l = "Exec";    break;
      case SP_STOP_BUTTON:     name = "stop";     l = "Stop";    break;
      case SP_EXIT_BUTTON:     name = "exit";     l = "Exit";    break;
      case SP_SAVE_AS_BUTTON:  name = "saveas";   l = "Save As.."; break;
      case SP_CONFIGURE_BUTTON:name = "configure";l = "Settings"; break;
   }
   if (!label)
      label = l;
   m_image = 0;
   if (name) 
       m_image = CThemes::getIconImage(name,iconSize);
   if (iconSize == IS_LARGE_ICON)
      this->label(label);
   m_kind = buttonKind;
   m_iconSize = iconSize;
}

void CBaseButton::label(const char *lbl) {
   m_label = lbl;
   Fl_Button::label(m_label.c_str());
}

#ifdef __COMPATIBILITY_MODE__
CBaseButton::CBaseButton(CButtonKind kind,int x,int y,int w,const char *l,CThemeButtonSize tbt)
: Fl_Button(x,y,w,30), CLayoutClient(this,w,SP_ALIGN_NONE) {
   m_default = false;
   m_type = tbt;
   image(kind,IS_LARGE_ICON,l);
   box(FL_THIN_UP_BOX);
   color(FL_LIGHT3);
}
#endif

CBaseButton::CBaseButton(CButtonKind kind,CLayoutAlign layoutAlign,bool is_small,const char *l,CThemeButtonSize tbt)
: Fl_Button(0,0,20,20), CLayoutClient(this,20,layoutAlign) {
   m_default = false;
   m_type = tbt;
   if (is_small)
      image(kind,IS_SMALL_ICON);
   else image(kind,IS_LARGE_ICON,l);
   box(FL_THIN_UP_BOX);
}

CBaseButton::CBaseButton(const char *l,CLayoutAlign layoutAlign,CThemeButtonSize tbt)
: Fl_Button(0,0,20,20), CLayoutClient(this,20,layoutAlign) {
   m_default = false;
   m_type = tbt;
   m_image = 0L;
   label(l);
   box(FL_THIN_UP_BOX);
}

void CBaseButton::defaultButton(bool isDefault) {
   m_default = isDefault;
   if (isDefault)
      box(FL_UP_BOX);
   else box(FL_THIN_UP_BOX);
   redraw();
}

void CBaseButton::drawFocusLine(int xs,int ys,int xe,int ye) {
    // Since FLTK under Win32 doesn't support the dotted line,
    // draw it with dots.
   int xp = xs;
   int yp = ys;
   int xsign = ((xs < xe) - (xs > xe));
   int ysign = ((ys < ye) - (ys > ye));
   if (xsign == 0 && ysign == 0) return;
   int dx = xsign * 2;
   int dy = ysign * 2;
   for ( ; xp * xsign <= xe * xsign && yp * ysign <= ye * ysign; xp += dx, yp += dy ) {
      fl_point(xp,yp);
   }
}

void CBaseButton::drawFocus(bool usingTheme) {
   fl_color(FL_BLACK);
   int r = 0;
   if (usingTheme) r = CThemes::buttonFocusRadius();
   if (r > h() / 2) r = h() / 2;
   int xx = x()+3;
   int yy = y()+3;
   int ww = w()-7;
   int hh = h()-7;
   if (m_default) {
      xx += 2;
      yy += 2;
      ww -= 6;
      hh -= 5;
   }
   drawFocusLine(xx+r,yy,xx+ww-r,yy);
   drawFocusLine(xx+ww-r,yy,xx+ww,yy+r);
   drawFocusLine(xx+ww,yy+r,xx+ww,yy+hh-r);
   drawFocusLine(xx+ww,yy+hh-r,xx+ww-r,yy+hh);
   drawFocusLine(xx+ww-r,yy+hh,xx+r,yy+hh);
   drawFocusLine(xx+r,yy+hh,xx,yy+hh-r);
   drawFocusLine(xx,yy+hh-r,xx,yy+r);
   drawFocusLine(xx,yy+r,xx+r,yy);
}

void CBaseButton::draw() {
   if (m_kind != SP_UNDEFINED_BUTTON)
     image(m_kind,m_iconSize,m_label.c_str());
#if FL_MAJOR_VERSION > 1
   align(FL_ALIGN_INSIDE);
   image(m_image);
   Fl_Button::draw();
#else
   align(Fl_Align(align()|FL_ALIGN_INSIDE));
   if (type() == FL_HIDDEN_BUTTON) return;

   int xx = x();
   int yy = y();
   int offset = 0;
   if (m_default) offset = 2;
   int X = xx  + offset;
   int Y = yy  + offset;
   int W = w() - offset*2;
   int H = h() - offset*2;

   bool highlited = (Fl::belowmouse() == this) && active_r();

   bool usingTheme = CThemes::drawButton(m_type, X, Y, W, H, highlited, value()!=0, m_default);
   if (!usingTheme) {
      if (m_default) {
         draw_box(FL_THIN_DOWN_BOX,FL_BLACK);
         draw_box(value()?FL_FLAT_BOX:box(), X, Y, W, H, color());
      } else {
         draw_box(value()?FL_THIN_DOWN_BOX:box(), X, Y, W, H, color());
      }
   }

   fl_push_clip(X+1,Y+1,W-2,H-2);
   if (!m_image)
      draw_label();
   else {
      int pressedOffset = value()?1:0;
      int xx = X + 4 + pressedOffset;
      int yy = Y + (H - m_image->h()) / 2 + pressedOffset;
      int ww = W - 6;
      int hh = H - 6;
      int tx = xx + m_image->w() + 2;
      int tw = ww - (tx - xx) - 2;
      if (!label()) { // center image
         xx = x() + (w() - m_image->w() - 1) / 2 + pressedOffset;
         m_image->draw(xx,yy,m_image->w(),m_image->h(),0,0);
      } else {
         m_image->draw(xx,yy,ww,hh,0,0);
      }
      fl_color(FL_BLACK);
      fl_font(FL_HELVETICA,FL_NORMAL_SIZE);
      int wt = tw;
      int ht = 0;
      fl_measure(label(),wt,ht);
      int ty = Y + H / 2 - ht / 2;
      fl_draw(label(), tx, ty, wt, ht, Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_TOP));
      if (Fl::focus() == this) {
         //draw_focus(FL_FLAT_BOX, x() + 2, y() + 2, w() - 4, h() - 4);
         drawFocus(usingTheme);
      }
   }
   fl_pop_clip();
#endif
}

int CBaseButton::handle(int event) {
   switch (event) {
      case FL_FOCUS:
         {
            CControl *control = 0L;
            try {
               control = dynamic_cast<CControl *>(parent());
            }
            catch (...) {}

            if (!control && parent())
               try {
               control = dynamic_cast<CControl *>(parent()->parent());
               if (control)
                  control->notifyFocus();
            }
            catch (...) {}
         }
         break;

      case FL_ENTER:
      case FL_LEAVE:
         redraw();
         break;
   }
   return Fl_Button::handle(event);
}

bool CBaseButton::preferredSize(int& w,int& h) {
   h = 0;

   int bh = 0;
   int bw = 0;
   int br = 0;
   CThemes::sizeButton(m_type,bw,bh);
   if (bw) {
      br = CThemes::buttonFocusRadius();
      if (m_default) {
         bh += 4;
         bw += 4;
      }
   }

   if (m_image)
      h = m_image->h() + 6;

   fl_font(labelfont(),labelsize());
   int lw = 0, lh = 0;
   fl_measure(label(),lw,lh);
   if (lh + 4 > h) h = lh + 4;
   if (lw) lw += 8;
   int ww = lw + 6 + br;
   if (m_image)
      ww += m_image->w();
   if (m_default) {
      h += 4;
      ww += 4;
   }
   if (bh) h = bh;
   if (w < ww) w = ww;
   if (w < bw) w = bw;
   return false;
}

CLayoutClient* CButton::creator(CXmlNode *node) {
    CButton* button = new CButton("",SP_ALIGN_TOP,THM_BUTTON_NORMAL);
    button->load(node);
    return button;
}

void CButton::load(const CXmlNode *node) {
    CLayoutClient::load(node,LXM_LAYOUT);
    string iconName = node->getAttribute("icon");
    string iconType = node->getAttribute("size","large");
    CIconSize iconSize;
    switch (iconType[0]) {
        case 'c': iconSize = IS_COMBO_ICON; break;
        case 's': iconSize = IS_SMALL_ICON; break;
        default:
        case 'l': iconSize = IS_LARGE_ICON; break;
    }
    if (!iconName.empty()) {
        Fl_Image* image = CThemes::getIconImage(iconName,iconSize);
        buttonImage(image);
    }
}

void CButton::save(CXmlNode *node) const {
    CLayoutClient::save(node,LXM_LAYOUT);
}
