/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CCheckButtons.cpp  -  description
                             -------------------
    begin                : Thi Dec 12 2002
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/Fl_Round_Button.H>
#include <FL/fl_draw.H>

#include <stdio.h>
#include <string.h>
#include <sptk3/gui/CDataControl.h>
#include <sptk3/gui/CCheckButtons.h>
#include <sptk3/gui/CThemes.h>
#include <sptk3/gui/CScroll.h>
#include <sptk3/CStrings.h>

#include <stdio.h>

#include <algorithm>

using namespace std;
using namespace sptk;

void CCheckButton::draw() {
    int bh = 0;
    int bw = 0;
    CThemes::sizeButton(THM_BUTTON_CHECK,bw,bh);
    bool highlited = value() != 0;
    if (CThemes::drawCheckButton(x(),y(),highlited,highlited)) {
        if (!m_label.empty()) {
            fl_color(labelcolor());
            fl_font(labelfont(),labelsize());
            fl_draw(m_label.c_str(),x()+bw+4,y()+2,w()-bw-6,h()-4,Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_WRAP));
            if (Fl::focus() == this)
                draw_focus(FL_FLAT_BOX, x()+bw+2,y()+1,w()-bw-2,h()-1);
        }
    } else
        Fl_Check_Button::draw();
}

bool CCheckButton::preferredSize(int& w,int& h) {
    int bh = 12;
    int bw = 12;
    CThemes::sizeButton(THM_BUTTON_CHECK,bw,bh);
    fl_font(labelfont(),labelsize());
    w -= bw + 6;
    fl_measure(m_label.c_str(),w,h);
    w += bw + 6;
    if (h < bh)
        h = bh;
    if (h < labelsize())
        h = labelsize();
    return false;
}

int CCheckButton::handle(int event) {
    if (event == FL_FOCUS) {
        CControl *control = dynamic_cast<CControl *>(parent());
        if (!control && parent())
            control = dynamic_cast<CControl *>(parent()->parent());
        if (control)
            control->notifyFocus();
    }
    return Fl_Button::handle(event);
}

void CCheckButtons::checkButtonsCallback(Fl_Widget *w,void *) {
    Fl_Button      *b = (Fl_Button *) w;
    Fl_Group       *g = b->parent();
    CCheckButtons  *c = (CCheckButtons *)g->parent();
    c->controlDataChanged();
    g->redraw();
}

CCheckButtons::CCheckButtons(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CRadioButtons (label,layoutSize,layoutAlignment) {}

#ifdef __COMPATIBILITY_MODE__
CCheckButtons::CCheckButtons(int x,int y,int w,int h,const char *l)
        : CRadioButtons (x,y,w,h,l) {}
#endif

Fl_Button *CCheckButtons::createButton(const char *label,int sz,CLayoutAlign layoutAlignment) {
    Fl_Button *btn = new CCheckButton(label,sz,layoutAlignment);
    btn->callback(CCheckButtons::checkButtonsCallback);
    return btn;
}

CVariant CCheckButtons::data() const {
    string     result;
    CScroll    *group = (CScroll *)m_control;
    unsigned    cnt = group->children();
    for (unsigned i = 0; i < cnt; i++) {
        Fl_Button *b = dynamic_cast<Fl_Button *>(group->child(i));
        if (!b)
            continue;
        if (b->value()) {
            if (b == m_otherButton) {
                result += m_otherInput->value();
            } else {
                result += b->label();
            }
            result += "|";
        }
    }
    int len = result.length();
    if (len)
        result[len-1] = 0;

    return result;
}

void CCheckButtons::data(const CVariant s) {
    deselectAllButtons();
    if (m_otherButton)
        m_otherInput->value("");
    string st = s;
    CStrings sl(st,"|");
    unsigned cnt = sl.size();
    CScroll *g = (CScroll *)m_control;
    for (unsigned i = 0; i < cnt; i++) {
        int ndx = buttonIndex(sl[i].c_str());
        if (ndx > -1) {
            Fl_Button *b = (Fl_Button *)g->child(ndx);
            b->value(1);
        } else {
            if (m_otherButton) {
                m_otherButton->value(1);
                m_otherInput->value(sl[i].c_str());
            }
        }
    }
}

void CCheckButtons::getSelections(CIntList& selection) const {
    selection.clear();
    CScroll  *group = (CScroll *)m_control;
    unsigned    cnt = group->children();
    for (unsigned i = 0; i < cnt; i++) {
        Fl_Button *b = dynamic_cast<Fl_Button *>(group->child(i));
        if (!b)
            continue;
        if (b->value()) {
            uint32_t id = (uint32_t)(long)b->user_data();
            selection.push_back(id);
        }
    }
}

void CCheckButtons::setSelections(const CIntList& selection) {
    deselectAllButtons();
    CScroll  *group = (CScroll *)m_control;
    unsigned    cnt = group->children();
    for (unsigned i = 0; i < cnt; i++) {
        Fl_Button *b = dynamic_cast<Fl_Button *>(group->child(i));
        if (!b)
            continue;
        uint32_t id = (uint32_t)(long)b->user_data();
        CIntList::const_iterator itor = std::find(selection.begin(),selection.end(),id);
        if (itor != selection.end())
            b->value(1);
    }
}

CLayoutClient* CCheckButtons::creator(CXmlNode *node) {
    CCheckButtons* widget = new CCheckButtons("",20,SP_ALIGN_NONE);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}
