/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          ccolumn.cpp  -  description
                             -------------------
    begin                : Thu Apr 27 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdlib.h>

#include <FL/Fl.H>
#include <sptk3/sptk.h>
#include <sptk3/gui/CColumn.h>
#include <sptk3/CException.h>
#include <sptk3/string_ext.h>

using namespace std;
using namespace sptk;

CColumn::CColumn(const std::string cname,CVariantType type,short cwidth,bool cvisible) {
    m_name = cname;
    m_type = type;
    m_visible = cvisible;
    if (cwidth < 0) {
        m_autoWidth = true;
        m_width = 0;
    } else {
        m_autoWidth = false;
        m_width = cwidth;
    }
}

CColumn::CColumn(const CColumn& col) {
    m_name = col.m_name;
    m_type = col.m_type;
    m_width = col.m_width;
    m_visible = col.m_visible;
    m_autoWidth = col.m_autoWidth;
}

void CColumn::load(const CXmlNode& node) {
    m_name = node.getAttribute("caption");
    m_type = (CVariantType)(unsigned) node.getAttribute("type");
    m_width = node.getAttribute("width");
    m_visible = node.getAttribute("visible");
    m_autoWidth = node.getAttribute("auto_width");
}

void CColumn::save(CXmlNode& node) const {
    node.clear();
    node.name("column");
    node.setAttribute("caption",m_name);
    node.setAttribute("type",m_type);
    node.setAttribute("width",m_width);
    node.setAttribute("visible",m_visible);
    node.setAttribute("auto_width",m_autoWidth);
}

int CColumnList::indexOf(const char *colname) const {
    unsigned cnt = size();
    for (unsigned i = 0; i < cnt; i++) {
        const CColumn& column = operator[](i);
        if (column.name() == colname)
            return i;
    }
    return -1;
}

void CColumnList::load(const CXmlNode& node) {
    CXmlNode::const_iterator itor = node.begin();
    CXmlNode::const_iterator iend = node.end();
    resize(node.size());
    for (; itor != iend; itor++) {
        try {
            CXmlNode& columnNode = *(*itor);
            unsigned columnIndex = columnNode.getAttribute("index");
            if (columnIndex >= size())
                continue;
            CColumn& column = (*this)[columnIndex];
            column.load(columnNode);
        } catch (...) {}
    }
}

void CColumnList::save(CXmlNode& node) const {
    node.clear();
    node.name("columns");
    unsigned counter = size();
    for (unsigned i = 0; i < counter; i++) {
        try {
            const CColumn& column = (*this)[i];
            CXmlNode& columnNode = *(new CXmlElement(node,"column"));
            column.save(columnNode);
            columnNode.setAttribute("index",i);
        } catch (...) {}
    }
}
