/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cdblistview.cpp  -  description
                            -------------------
    begin                : Fri Mar 31 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CFieldList.h>
#include <sptk3/CException.h>
#include <sptk3/gui/CDBListView.h>
#include <sptk3/gui/CMessageDialog.h>

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <stdio.h>

#include <sptk3/DebugMacros.h>

using namespace std;
using namespace sptk;

CDBListView::CDBListView(const char *label,int layoutSize,CLayoutAlign layoutAlignment)
: CListView(label,layoutSize,layoutAlignment) {
   m_fastRefreshEnabed = false;
   m_maxRecords = 0;
   m_recordsLimited = false;
}

#ifdef __COMPATIBILITY_MODE__    
CDBListView::CDBListView(int x, int y, int w, int h, const char *label)
: CListView(x, y, w, h, label) {
   m_fastRefreshEnabed = false;
   m_maxRecords = 0;
   m_recordsLimited = false;
}
#endif

CDBListView::~CDBListView() {
   m_fastRefreshQuery.close();
   m_fullRefreshQuery.close();
}

CLayoutClient* CDBListView::creator(CXmlNode *node) {
    CDBListView* widget = new CDBListView("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

void CDBListView::database(CDatabase *db) {
   if (m_dataMode == LV_DATA_UNDEFINED)
      m_dataMode = LV_DATA_KEY;
   m_fastRefreshQuery.connect(db);
   m_fullRefreshQuery.connect(db);
   m_recordCountQuery.connect(db);
}

CDatabase *CDBListView::database() const {
   return m_fullRefreshQuery.database();
}

void CDBListView::sql(string _sql,string _recordCountSql,string _fastRefreshSQL) {
   m_fullRefreshQuery.sql(_sql);
   m_fastRefreshQuery.sql(_fastRefreshSQL);
   m_recordCountQuery.sql(_recordCountSql);
   m_fastRefreshEnabed = _fastRefreshSQL.length() > 0;
}

string CDBListView::sql() {
   return m_fullRefreshQuery.sql();
}

CParam& CDBListView::param(const char *paramName,CRefreshKind refreshKind) {
   if (m_fastRefreshEnabed && refreshKind == LV_REFRESH_FAST )
      return m_fastRefreshQuery.param(paramName);
   else  return m_fullRefreshQuery.param(paramName);
}

void CDBListView::setup(CDatabase *db,string _sql,string _keyField) {
   database(db);
   sql(_sql);
   keyField(_keyField);
}

void CDBListView::keyField(string keyField) {
   m_keyField = keyField;
}

void CDBListView::refreshData(CRefreshKind refreshKind) {
   m_lastRefresh = CDateTime::Now();

   fl_cursor(FL_CURSOR_WAIT);
   Fl::check();

   unsigned recordsEstimated = 0;
   if (m_recordCountQuery.sql().length()) {
      m_recordCountQuery.open();
      recordsEstimated = m_recordCountQuery[uint32_t(0)].asInteger();
      m_recordCountQuery.close();
   }

   CQuery *query = &m_fullRefreshQuery;
   if (m_fastRefreshEnabed && refreshKind == LV_REFRESH_FAST)
      query = &m_fastRefreshQuery;
   else  refreshKind = LV_REFRESH_FULL;

   fill(*query,m_keyField,m_maxRecords,recordsEstimated,refreshKind);
}
