/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDataControl.cpp  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Help_View.H>
#include <ctype.h>

#include <stdio.h>
#include <stdlib.h>
#include <sptk3/CStrings.h>
#include <sptk3/gui/CDataControl.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CCalendar.h>
#include <sptk3/db/CQuery.h>

#include <sptk3/gui/CEditor.h>
#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CDateControl.h>
#include <sptk3/gui/CRadioButtons.h>
#include <sptk3/gui/CCheckButtons.h>
#include <sptk3/gui/CComboBox.h>

using namespace sptk;

CControl *sptk::createControl(int controlKind,std::string label,std::string fieldName,int size) {
    CControl   *control = NULL;

    const char *lbl = label.c_str();

    switch (controlKind) {
    case DCV_BOX:
        control = new CBox(lbl,size);
        break;
    case DCV_HTMLBOX:
        control = new CHtmlBox(lbl,size);
        break;
    case DCV_RADIOBUTTONS:
        control = new CRadioButtons(lbl,size);
        break;
    case DCV_STRING:
        control = new CInput(lbl,size);
        break;
    case DCV_MEMO:
        control = new CMemoInput(lbl,size);
        break;
    case DCV_INTEGER:
        control = new CIntegerInput(lbl);
        break;
    case DCV_FLOAT:
        control = new CFloatInput(lbl);
        break;
    case DCV_DATETIME:
        control = new CDateTimeInput(lbl);
        break;
    case DCV_DATEINTERVAL:
        control = new CDateIntervalInput(lbl);
        break;
    case DCV_DATE:
        control = new CDateInput(lbl);
        break;
    case DCV_TIME:
        control = new CTimeInput(lbl);
        break;
    case DCV_PHONE:
        control = new CPhoneNumberInput(lbl);
        break;
    case DCV_LISTBOX:
        control = new CListBox(lbl,size);
        break;
    case DCV_CHECKBUTTONS:
        control = new CCheckButtons(lbl);
        break;
    case DCV_COMBO:
        control = new CComboBox(lbl,size);
        break;
    case DCV_INTVALUECOMBO:
        control = new CComboBox(lbl,size);
        break;
    default:
        throw CException("Unsupported control type, field name '"+fieldName+"'.");
    }

    control->fieldName(fieldName.c_str());

    return control;
}
