/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDateControl.cpp - description
                             -------------------
    begin                : Mar 19 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.H>

#include <sptk3/gui/CDateControl.h>
#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CControl.h>
#include <sptk3/gui/CThemes.h>

using namespace std;
using namespace sptk;

void CDateControl::date_cb(Fl_Widget *w,void *data) {
   CDateControl *dateControl = (CDateControl *)w->parent();
   dateControl->do_callback(dateControl,dateControl->user_data());
}

CDateControl::CDateControl(int x,int y,int w,int h,const char *label) 
: Fl_Group(x,y,w,h,label) {
   box(FL_DOWN_BOX);
   m_input = new CInput_;
   m_input->mask(CDateTime::dateFormat);
   m_input->box(FL_FLAT_BOX);
   m_input->callback(date_cb);
   m_input->when(FL_WHEN_CHANGED);
   m_button = new CSmallButton(SP_CALENDAR_BUTTON);
   m_button->visible_focus(false);
   end();
}

void CDateControl::resize(int x,int y,int w,int h) {
   int editorWidth;
   int buttonWidth;
   preferredSize(w,h,editorWidth,buttonWidth);
   Fl_Group::resize(x,y,w,h);
   int dx = Fl::box_dx(box());
   int dy = Fl::box_dy(box());
   int dh = Fl::box_dh(box());
   m_input->resize(x+dx,y+dy,editorWidth,h-dh);
   m_button->resize(x+editorWidth+dx,y+dy,buttonWidth,h-dh);
}

void CDateControl::preferredHeight(int& hh) {
   CControl *parentControl = dynamic_cast<CControl *>(parent());
   int bw = 18;
   int bh = 18;
   int dh = Fl::box_dh(box());
   CThemes::sizeButton(THM_BUTTON_COMBO,bw,bh);
   if (parentControl) {
      hh = parentControl->textSize() + 8;
      if (hh < bh) hh = bh;
   }
   hh += dh;
}

void CDateControl::preferredSize(int& ww,int& hh,int& editorWidth,int& buttonWidth) {
   CControl *parentControl = dynamic_cast<CControl *>(parent());
   editorWidth = 70;
   int bw = 18;
   int bh = 18;
   CThemes::sizeButton(THM_BUTTON_COMBO,bw,bh);
   int dw = Fl::box_dw(box());
   if (parentControl) {
      fl_font(parentControl->textFont(),parentControl->textSize());
      editorWidth = (int) fl_width("00/00/00000");
   }
   buttonWidth = bw;
   ww = editorWidth+bw+dw;
   preferredHeight(hh);
}
