/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDateIntervalInput.cpp  -  description
                             -------------------
    begin                : Fri Oct 27 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.H>

#include <stdio.h>
#include <stdlib.h>
#include <sptk3/CStrings.h>
#include <sptk3/gui/CDateIntervalInput.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CCalendar.h>
#include <sptk3/gui/CDateControl.h>

using namespace std;
using namespace sptk;

void CDateIntervalInput::intervalCalendarButtonPressed(Fl_Widget *btn,void *data) {
    CDateIntervalInput *intervalInput = (CDateIntervalInput *)btn->parent()->parent();
    intervalInput->showCalendar(btn);
}

void CDateIntervalInput::ctor_init() {
    m_calendarWindow = new CPopupCalendar(0L);
    m_separator = "..";

    begin();

    m_firstDateInput = new CDateControl(0,0,10,10);
    m_firstDateInput->callback(CControl::internalCallback);
    m_firstDateInput->when(FL_WHEN_CHANGED);
    m_firstDateInput->button()->callback(intervalCalendarButtonPressed);

    m_control = m_firstDateInput->input();

    m_secondDateInput = new CDateControl(0,0,10,10,"to");
    m_secondDateInput->align(FL_ALIGN_LEFT);
    m_secondDateInput->callback(CControl::internalCallback);
    m_secondDateInput->when(FL_WHEN_CHANGED);
    m_secondDateInput->button()->callback(intervalCalendarButtonPressed);

    end();
}

CDateIntervalInput::CDateIntervalInput(const char *label,int layoutSize,CLayoutAlign layoutAlignment)
        : CInput(label,layoutSize,layoutAlignment,false) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CDateIntervalInput::CDateIntervalInput(int x,int y,int w,int h,const char * label)
        : CInput(x,y,w,h,label,false) {
    ctor_init();
}
#endif

CLayoutClient* CDateIntervalInput::creator(CXmlNode *node) {
    CDateIntervalInput* widget = new CDateIntervalInput("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

void CDateIntervalInput::showCalendar(Fl_Widget *btn) {
    CDateTime originalDate;
    CDateControl *dateControl;
    if (btn == m_firstDateInput->button()) {
        originalDate = beginOfInterval();
        dateControl = m_firstDateInput;
    } else {
        originalDate = endOfInterval();
        dateControl = m_secondDateInput;
    }
    m_calendarWindow->attachTo(dateControl);
    m_calendarWindow->date(originalDate);
    bool rc = m_calendarWindow->showModal();
    if (rc && m_calendarWindow->date() != originalDate) {
        if (btn == m_firstDateInput->button()) {
            beginOfInterval(m_calendarWindow->date());
        } else {
            endOfInterval(m_calendarWindow->date());
        }
        dateControl->do_callback();
    }
}

void CDateIntervalInput::separator(std::string s) {
    m_separator = s;
}

void CDateIntervalInput::resize(int x,int y,int w,int h) {
    int twidth = 16;
    fl_font(textFont(),textSize());
    int editorWidth = (int) fl_width("00/00/00000") + h;
    int maxWidth = 2 * editorWidth + twidth + m_labelWidth;

    if (w > maxWidth)
        w = maxWidth;

    CControl::resize(x,y,w,h);
    h = this->h();

    w -= m_labelWidth;
    x += m_labelWidth;
    if (m_menuButton)
        m_menuButton->resize(x,y,w,h);

    m_firstDateInput->resize(x,y,editorWidth,h);
    x += editorWidth + twidth;
    m_secondDateInput->resize(x,y,editorWidth,h);
}

CVariant CDateIntervalInput::data() const {
    CDateTime   dt1 = beginOfInterval();
    CDateTime   dt2 = endOfInterval();

    if ((double)dt1 == 0.0 && (double)dt2 == 0.0)
        return "";

    std::string result = m_firstDateInput->input()->value();
    result += m_separator;
    result += m_secondDateInput->input()->value();
    return result;
}

void CDateIntervalInput::data(const CVariant s) {
    string     firstStringData(s.asString().c_str());
    string     secondStringData;

    size_t pos = firstStringData.find(m_separator.c_str());
    size_t separatorLength = m_separator.length();

    if (!separatorLength)
        throw CException("Can't work with an empty dates separator");

    if (pos != string::npos && pos > 0) {
        firstStringData[pos] = 0;
        secondStringData = firstStringData.substr(pos+separatorLength,20);
    }
    m_firstDateInput->input()->value(firstStringData.c_str());
    m_secondDateInput->input()->value(secondStringData.c_str());
}

bool CDateIntervalInput::valid() const {
    CDateTime   dt1 = beginOfInterval();
    CDateTime   dt2 = endOfInterval();

    if ((double)dt1 != 0 && (double)dt2 != 0 && dt1 > dt2)
        return false;

    return true;
}

CDateTime CDateIntervalInput::beginOfInterval() const {
    return CDateTime(m_firstDateInput->input()->value());
}

void CDateIntervalInput::beginOfInterval(const CDateTime bi) {
    m_firstDateInput->input()->value(bi.dateString().c_str());
}

CDateTime CDateIntervalInput::endOfInterval() const {
    return CDateTime(m_secondDateInput->input()->value());
}

void CDateIntervalInput::endOfInterval(const CDateTime ei) {
    m_secondDateInput->input()->value(ei.dateString().c_str());
}
