/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDateTimeInput.cpp  -  description
                             -------------------
    begin                : Fri Oct 27 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.H>
#include <sptk3/gui/CDateTimeInput.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CInput.h>
#include <sptk3/gui/CDateControl.h>
#include <sptk3/gui/CCalendar.h>

using namespace sptk;

void CDateTimeBaseInput::calendarButtonPressed(Fl_Widget *btn,void *data) {
    CDateTimeBaseInput *dateInput = (CDateTimeBaseInput *)btn->parent()->parent();
    if (!dateInput)
        return;
    CDateTime dt = dateInput->dateTimeValue();
    dateInput->showCalendar(btn);
    if (dt != dateInput->dateTimeValue())
        dateInput->m_dateInput->do_callback();
}

void CDateTimeBaseInput::ctor_init() {
    m_dateInput = 0L;
    m_timeInput = 0L;
    m_minValue = double(0);
    m_maxValue = double(0);
    m_calendarWindow = 0L;
    m_timeInput = (CInput_ *)m_control;
}

CDateTimeBaseInput::CDateTimeBaseInput(const char * label,int layoutSize,CLayoutAlign layoutAlignment,bool autoCreate)
        : CInput(label,layoutSize,layoutAlignment,autoCreate) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CDateTimeBaseInput::CDateTimeBaseInput(int x,int y,int w,int h,const char * label,bool autoCreate)
        : CInput(x,y,w,h,label,autoCreate) {
    ctor_init();
}
#endif

void CDateTimeBaseInput::setLimits(bool limited,CDateTime min,CDateTime max) {
    m_limited = limited;
    m_minValue = min;
    m_maxValue = max;
}

void CDateTimeBaseInput::load(CQuery *loadQuery) {
    if (!m_fieldName.length())
        return; // no field name - no data loaded
    if (!m_fieldName.length())
        return;
    CField& fld = (*loadQuery)[m_fieldName.c_str()];
    dateTimeValue(fld.asDateTime());
}

void CDateTimeBaseInput::save(CQuery *updateQuery) {
    if (!m_fieldName.length())
        return; // no field name - no data saved
    CParam& param = updateQuery->param(m_fieldName.c_str());
    CDateTime dt = dateTimeValue();
    param.setDateTime( dt );
}

void CDateTimeBaseInput::load(const CXmlNode *node,CLayoutXMLmode xmlMode) {
    CControl::load(node,xmlMode);
}

void CDateTimeBaseInput::save(CXmlNode *node,CLayoutXMLmode xmlMode) const {
    CControl::save(node,xmlMode);
}

bool CDateTimeBaseInput::valid() const {
    try {
        double val = dateTimeValue();
        if (m_limited)
            return val >= double(m_minValue) && val <= double(m_maxValue);
        return true;
    } catch (...) {
        return false;
    }
}

CDateTime CDateTimeBaseInput::dateTimeValue() const {
    return data();
}

void CDateTimeBaseInput::dateTimeValue(CDateTime dt) {
    if (kind() == DCV_DATE)
        data(dt.dateString());
    else
        data(dt.timeString());
}

bool CDateTimeBaseInput::showCalendar(Fl_Widget *btn) {
    if (!m_calendarWindow)
        return false;
    m_calendarWindow->attachTo(btn->parent());
    m_calendarWindow->date(dateTimeValue());
    bool rc = m_calendarWindow->showModal();
    if (rc) {
        if (m_calendarWindow->date() != dateTimeValue()) {
            dateTimeValue(m_calendarWindow->date());
            do_callback();
        }
    }
    return rc;
}
//===========================================================================
void CDateInput::ctor_init() {
    begin();
    m_dateInput = new CDateControl(0,0,10,10);
    m_control = m_dateInput->input();
    m_dateInput->callback(CControl::internalCallback);
    m_dateInput->when(FL_WHEN_CHANGED);
    m_dateInput->button()->callback(calendarButtonPressed);
    m_calendarWindow = new CPopupCalendar(m_control);
    end();
    m_control->color(FL_LIGHT3);
}

CDateInput::CDateInput(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CDateTimeBaseInput(label,layoutSize,layoutAlignment,false) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CDateInput::CDateInput(int x,int y,int w,int h,const char * label)
        : CDateTimeBaseInput(x,y,w,h,label,false) {
    ctor_init();
}
#endif

CDateInput::~CDateInput() {
    delete m_calendarWindow;
}

CLayoutClient* CDateInput::creator(CXmlNode *node) {
    CDateInput* widget = new CDateInput("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

void CDateInput::resize(int xx,int yy,int ww,int hh) {
    CControl::resize(xx,yy,ww,hh);
    m_dateInput->resize(xx+m_labelWidth,yy,ww-m_labelWidth,hh);
}

CVariant CDateInput::data() const {
    CDateTime   dt(m_dateInput->input()->value());
    CVariant    rc;
    rc.setDate(dt);
    return rc;
}

void CDateInput::data(const CVariant s) {
    CDateTime dt = s;
    m_dateInput->input()->value(dt.dateString().c_str());
}

void CDateInput::preferredHeight(int& h) const {
    CDateTimeBaseInput::preferredHeight(h);
    m_dateInput->preferredHeight(h);
}
//===========================================================================
CTimeInput::CTimeInput(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CDateTimeBaseInput(label,layoutSize,layoutAlignment,true) {
    m_timeInput = (CInput_ *)m_control;
    m_timeInput->mask(CDateTime::shortTimeFormat);
}

#ifdef __COMPATIBILITY_MODE__
CTimeInput::CTimeInput(int x,int y,int w,int h,const char * label)
        : CDateTimeBaseInput(x,y,w,h,label,true) {
    m_timeInput = (CInput_ *)m_control;
    m_timeInput->mask(CDateTime::shortTimeFormat);
}
#endif

CLayoutClient* CTimeInput::creator(CXmlNode *node) {
    CTimeInput* widget = new CTimeInput("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

void CTimeInput::preferredWidth(int& w) const {
    w = int(7 * fl_width('W')) + m_labelWidth + 6;
}
//===========================================================================
void CDateTimeInput::ctor_init() {
    begin();
    m_dateInput = new CDateControl(0,0,10,10);
    m_dateInput->callback(CControl::internalCallback);
    m_dateInput->when(FL_WHEN_CHANGED);
    m_dateInput->button()->callback(calendarButtonPressed);
    m_dateInput->color(FL_LIGHT3);

    m_control = m_dateInput->input();

    m_timeInput = new CInput_("");
    m_timeInput->callback(CControl::internalCallback);
    m_timeInput->when(FL_WHEN_CHANGED);
    m_timeInput->color(FL_LIGHT3);

    m_calendarWindow = new CPopupCalendar(m_dateInput);
    end();
}

CDateTimeInput::CDateTimeInput(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CDateTimeBaseInput(label,layoutSize,layoutAlignment,false) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CDateTimeInput::CDateTimeInput(int x,int y,int w,int h,const char * label)
        : CDateTimeBaseInput(x,y,w,h,label,false) {
    ctor_init();
}
#endif

CLayoutClient* CDateTimeInput::creator(CXmlNode *node) {
    CDateTimeInput* widget = new CDateTimeInput("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

void CDateTimeInput::resize(int x,int y,int w,int h) {
    CControl::resize(x,y,w,h);
    w -= m_labelWidth;
    x += m_labelWidth;

    int hh = textSize() + 6;

    if (m_menuButton)
        m_menuButton->resize(x,y,w,hh);

    m_dateInput->resize(x,y,w/2-2,hh);
    x += m_dateInput->w() + 4;

    fl_font(textFont(),textSize());
    int timeWidth = (int) fl_width("00:00AM") + 6;
    m_timeInput->resize(x,y,timeWidth,m_dateInput->h());
}

void CDateTimeInput::dateTimeValue(CDateTime dt) {
    m_dateInput->input()->value(dt.dateString().c_str());
    m_timeInput->value(dt.timeString().c_str());
}

CDateTime CDateTimeInput::dateTimeValue() const {
    CDateTime   dt = CDateTime( m_dateInput->input()->value() );
    CDateTime   tm = CDateTime( m_timeInput->value() );
    return dt + tm;
}

CVariant CDateTimeInput::data() const {
    return std::string(m_dateInput->input()->value()) + " " + std::string(m_timeInput->value());
}

void CDateTimeInput::data(const CVariant s) {
    dateTimeValue(s.asDateTime());
}
