/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cgroup.cpp  -  description
                             -------------------
    begin                : Sun May 18 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <sptk3/gui/CGroup.h>
#include <sptk3/gui/CControl.h>
#include <stdio.h>

using namespace std;
using namespace sptk;

void CGroup::ctor_init(const char *alabel) {
    if (!alabel)
        alabel = "";
    m_drawClipped = true;
    label(alabel);
}

CGroup::CGroup(const char *alabel,int layoutSize,CLayoutAlign layoutAlign)
        : Fl_Group(0,0,layoutSize,layoutSize), CLayoutManager(this,layoutSize,layoutAlign) {
    ctor_init(alabel);
}

#ifdef __COMPATIBILITY_MODE__
CGroup::CGroup(int x,int y,int w,int h,const char *alabel)
        : Fl_Group(x,y,w,h), CLayoutManager(this,w,SP_ALIGN_NONE) {
    ctor_init(alabel);
}
#endif

CLayoutClient* CGroup::creator(CXmlNode* node) {
    CGroup* widget = new CGroup("",10,SP_ALIGN_TOP);
    widget->loadLayout(node,LXM_LAYOUTDATA);
    return widget;
}

void CGroup::resize(int xx,int yy,int ww,int hh) {
    Fl_Widget::resize(xx,yy,ww,hh);

    int offsetX = frameWidth();
    int offsetY = offsetX;

    const char *lbl = m_label.c_str();
    if (align() & FL_ALIGN_INSIDE && lbl && lbl[0] ) {
        int tw = ww - 8, th = 0;
        fl_font(labelfont(),labelsize());
        fl_measure(lbl,tw,th);
        offsetY = th;
    }

    autoLayout(xx,yy,ww,hh,true);
}

bool CGroup::preferredSize(int& ww,int& hh) {
    int offsetX = frameWidth();
    int offsetY = offsetX;

    const char *lbl = m_label.c_str();
    if (align() & FL_ALIGN_INSIDE && lbl && lbl[0] ) {
        int tw = ww - 8, th = 0;
        fl_font(labelfont(),labelsize());
        fl_measure(lbl,tw,th);
        offsetY = th;
    }

    return autoLayout(offsetX,offsetY,ww,hh,false);
}

void CGroup::draw() {
    string lbl = label();
    label("");

    Fl_Boxtype b = box();

    if (damage() & ~FL_DAMAGE_CHILD) { // redraw the entire thing:
        if (!m_frame)
            fl_draw_box(box(),x(),y(),w(),h(),color());
        paintBackground();
    }

    box(FL_NO_BOX);

    if (m_drawClipped)
        fl_push_clip(x()+Fl::box_dx(b),y()+Fl::box_dy(b),w()-Fl::box_dw(b),h()+Fl::box_dh(b));

    Fl_Group::draw();

    if (m_drawClipped)
        fl_pop_clip();

    label(lbl.c_str());

    int tw = w() - 8, th = 0;
    fl_font(labelfont(),labelsize());
    fl_measure(m_label.c_str(),tw,th);

    if (align() & FL_ALIGN_INSIDE && lbl[0]) {
        int xt = x() + 8;
        switch (align() & (FL_ALIGN_LEFT|FL_ALIGN_RIGHT|FL_ALIGN_CENTER)) {
        case FL_ALIGN_LEFT:
            break;
        case FL_ALIGN_RIGHT:
            xt = x() + w() - 8 - (tw + 4);
            break;
        case FL_ALIGN_CENTER:
            xt = x() + (w() - (tw + 4)) / 2;
            break;
        }
        fl_draw_box(FL_FLAT_BOX,xt,y(),tw+4,th+2,color());
        fl_color(labelcolor());
        fl_draw(m_label.c_str(),xt,y()+2,tw+4,th,FL_ALIGN_CENTER);
    } else {
        draw_label();
    }
    box(b);
}
