/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CDataControl.cpp  -  description
                             -------------------
    begin                : Tue Apr 4 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

//#include <FL/Fl.H>
//#include <FL/fl_draw.H>
#include <FL/Fl_Help_View.H>

#include <sptk3/gui/CHtmlBox.h>

using namespace sptk;

//===========================================================================
void CHtmlBox::ctor_init(const char *label) {
    m_controlFlags = FGE_MULTILINEENTRY;
    m_control = new Fl_Help_View(x(),y(),w(),h(),"");
    add(m_control);
    m_control->box(FL_FLAT_BOX);
    m_control->align(FL_ALIGN_LEFT | FL_ALIGN_INSIDE | FL_ALIGN_WRAP);
    m_control->callback(CControl::internalCallback);
    m_control->when(FL_WHEN_CHANGED);
    labelWidth(0);
    if (label)
        data(label);
}

CHtmlBox::CHtmlBox(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CInput("",layoutSize,layoutAlignment,false) {
    ctor_init(label);
}

#ifdef __COMPATIBILITY_MODE__
CHtmlBox::CHtmlBox(int x,int y,int w,int h,const char *label)
        : CInput(x,y,w,h,"",false) {
    ctor_init(label);
}
#endif

CLayoutClient* CHtmlBox::creator(CXmlNode *node) {
    CHtmlBox* widget = new CHtmlBox("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

CVariant CHtmlBox::data() const {
    return ((Fl_Help_View *)m_control)->value();
}

void CHtmlBox::data(const CVariant v) {
    ((Fl_Help_View *)m_control)->value(v.getString());
}

Fl_Font CHtmlBox::textFont() const {
    return Fl_Font(((Fl_Help_View *)m_control)->textfont());
}

void  CHtmlBox::textFont(Fl_Font f) {
    ((Fl_Help_View *)m_control)->textfont(f);
}

uchar CHtmlBox::textSize() const {
    return ((Fl_Help_View *)m_control)->textsize();
}

void  CHtmlBox::textSize(uchar s) {
    ((Fl_Help_View *)m_control)->textsize(s);
}

int CHtmlBox::totalHeight() const {
    return ((Fl_Help_View *)m_control)->size();
}

bool CHtmlBox::preferredSize(int& w,int& h) {
    Fl_Help_View *hv = (Fl_Help_View *)m_control;
    hv->resize(hv->x(),hv->y(),w,h);
    int totalHeight = hv->size();

    int hh = totalHeight + 10;
    if (hh < (int)labelHeight())
        hh = labelHeight();
    if (h < hh)
        h = hh;
    if (w < (int)m_labelWidth + 16)
        w = m_labelWidth + 16;

    return false;
}
