/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CLayoutClient.cpp  -  description
                             -------------------
    begin                : Thu Aug 17 2006
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/gui/CLayoutClient.h>
#include <sptk3/gui/CLayoutManager.h>
#include <sptk3/gui/CControl.h>
#include <sptk3/istring.h>
#include <stdexcept>

using namespace std;
using namespace sptk;

CLayoutClient::CLayoutClient(Fl_Widget* widget,int layoutSize,CLayoutAlign ca) {
    m_lastPreferredW = m_lastPreferredH = 0;
    m_layoutSize = layoutSize;
    m_layoutAlign = ca;
    m_widget = widget;
    if (widget->label()) {
        m_label = widget->label();
        widget->label(m_label.c_str());
    }
}

void CLayoutClient::load(const CXmlNode* node,CLayoutXMLmode xmlMode) {
    if (xmlMode & (int) LXM_LAYOUT) {
        CLayoutAlign layoutAlign;
        istring alignName = node->getAttribute("layout_align");
        switch (alignName[0]) {
        case 'b':
            layoutAlign = SP_ALIGN_BOTTOM;
            break;
        case 'l':
            layoutAlign = SP_ALIGN_LEFT;
            break;
        case 'r':
            layoutAlign = SP_ALIGN_RIGHT;
            break;
        case 'c':
            layoutAlign = SP_ALIGN_CLIENT;
            break;
        case 't':
            layoutAlign = SP_ALIGN_TOP;
            break;
        default:
            layoutAlign = SP_ALIGN_NONE;
            {
                int x = node->getAttribute("x","-1");
                int y = node->getAttribute("y","-1");
                int w = node->getAttribute("w","-1");
                int h = node->getAttribute("h","-1");
                if (x > -1 && y > -1)
                    m_widget->position(x,y);
                if (w > -1 && h > -1)
                    m_widget->Fl_Widget::size(w,h);
            }
            break;
        }
        m_layoutAlign = layoutAlign;
        name(node->getAttribute("name"));
        label(node->getAttribute("label"));

        if (layoutAlign != SP_ALIGN_NONE) {
            int layoutSize = node->getAttribute("layout_size");
            if (layoutSize)
                m_layoutSize = layoutSize;
        }

        string boxTypeName = node->getAttribute("box");
        if (boxTypeName.empty())
            boxTypeName = node->getAttribute("frame");
        if (!boxTypeName.empty()) {
            map<string,Fl_Boxtype>::const_iterator btor = CLayoutManager::boxTypeNames().find(boxTypeName);
            if (btor != CLayoutManager::boxTypeNames().end())
                m_widget->box(btor->second);
        }

        if ( ! (bool) node->getAttribute("visible","Y") )
            m_widget->hide();
        else
            m_widget->show();
        if ( ! (bool) node->getAttribute("enable","Y") )
            m_widget->deactivate();
        else
            m_widget->activate();
    }
    if (xmlMode & (int) LXM_DATA) {
        CControl* control = dynamic_cast<CControl*>(m_widget);
        if (control)
            control->load(node,LXM_DATA);
    }
}

void CLayoutClient::save(CXmlNode* node,CLayoutXMLmode xmlMode) const {
    if (!node->isElement())
        throw runtime_error("Node must be an element");
    string className = "widget";
    CLayoutClient* layoutClient = dynamic_cast<CLayoutClient*>(m_widget);
    if (layoutClient)
        className = layoutClient->className();
    node->name(className);

    if (xmlMode & (int) LXM_LAYOUT) {
        if (!m_name.empty())
            node->setAttribute("name",name());
        if (!m_label.empty())
            node->setAttribute("label",label());
        if (!m_widget->visible())
            node->setAttribute("visible",m_widget->visible());
        if (!m_widget->active())
            node->setAttribute("enable",m_widget->active());

        string layoutAlignStr;
        switch (m_layoutAlign) {
        case SP_ALIGN_TOP:
            layoutAlignStr = "top";
            break;
        case SP_ALIGN_BOTTOM:
            layoutAlignStr = "bottom";
            break;
        case SP_ALIGN_LEFT:
            layoutAlignStr = "left";
            break;
        case SP_ALIGN_RIGHT:
            layoutAlignStr = "right";
            break;
        case SP_ALIGN_CLIENT:
            layoutAlignStr = "client";
            break;
        case SP_ALIGN_NONE:
            break;
        }

        if (layoutAlignStr.empty()) {
            node->setAttribute("x",m_widget->x());
            node->setAttribute("y",m_widget->y());
            node->setAttribute("w",m_widget->w());
            node->setAttribute("h",m_widget->h());
        } else {
            node->setAttribute("layout_align",layoutAlignStr);
            node->setAttribute("layout_size",layoutSize());
        }
    } else {
        if (!m_name.empty())
            node->setAttribute("name",name());
        else
            if (!m_label.empty())
                node->setAttribute("label",label());
    }
    if (xmlMode & (int) LXM_DATA) {
        CControl* control = dynamic_cast<CControl*>(m_widget);
        if (control)
            control->save(node,LXM_DATA);
    }
}
