/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CListViewSelection.cpp  -  description
                             -------------------
    begin                : Jul 14 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/gui/CListViewSelection.h>
#include <string>
#include <algorithm>

using namespace sptk;

void CSelection::select(CPackedStrings *row) {
   if (row) {
      row->flags |= CLV_SELECTED;
      m_selectedRows.push_back(row);
   }
}

void CSelection::deselect(CPackedStrings *row) {
   if (row) {
      row->flags &= ~CLV_SELECTED;
      remove(row);
   }
}

void CSelection::deselectAll() {
   unsigned cnt = m_selectedRows.size();
   for (unsigned i=0; i < cnt; i++) {
      CPackedStrings *row = (CPackedStrings *)m_selectedRows[i];
      row->flags &= ~CLV_SELECTED;
   }
   m_selectedRows.clear();
}

void CSelection::remove(CPackedStrings *row) { 
   CPSVector::iterator itor = std::find(m_selectedRows.begin(),m_selectedRows.end(),row);
   if (itor != m_selectedRows.end())
      m_selectedRows.erase(itor); 
}

void CSelection::clear() {
   m_selectedRows.clear();
}

CPackedStrings *CSelection::findKey(int keyValue) const {
   unsigned cnt = m_selectedRows.size();
   CPackedStrings *row = 0L;
   for (unsigned i=0; i < cnt; i++) {
      row = (CPackedStrings *)m_selectedRows[i];
      if (row->argument() == keyValue)
         return row;
   }
   return 0L;
}

CPackedStrings *CSelection::findCaption(std::string caption) const {
   unsigned cnt = m_selectedRows.size();
   CPackedStrings *row = 0L;
   for (unsigned i=0; i < cnt; i++) {
      row = (CPackedStrings *)m_selectedRows[i];
      if ((*row)[0] == caption)
         return row;
   }
   return 0L;
}
