/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMemoInput.cpp  -  description
                             -------------------
    begin                : Oct 16 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include <sptk3/CStrings.h>
#include <sptk3/gui/CMemoInput.h>
#include <sptk3/gui/CEditor.h>

using namespace sptk;

void CMemoInput::ctor_init() {
    m_controlFlags = FGE_MULTILINEENTRY;
    begin();
    CEditor *e = new CEditor(50,SP_ALIGN_NONE);
    e->wrap_mode(true,0);
    m_control = e;
    m_control->callback(CControl::internalCallback);
    m_control->when(FL_WHEN_CHANGED);
    end();
}

CMemoInput::CMemoInput(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CInput(label,layoutSize,layoutAlignment,false) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CMemoInput::CMemoInput(int x,int y,int w,int h,const char * label)
        : CInput(x,y,w,h,label,false) {
    ctor_init();
}
#endif

CLayoutClient* CMemoInput::creator(CXmlNode *node) {
    CMemoInput* widget = new CMemoInput("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

CVariant CMemoInput::data() const {
    return ((CEditor *)m_control)->textBuffer()->text();
}

void CMemoInput::data(const CVariant s) {
    std::string strValue = replaceAll(s.asString(),"\r","");
    ((CEditor *)m_control)->textBuffer()->text(strValue.c_str());
}

Fl_Font CMemoInput::textFont() const {
    return ((CEditor *)m_control)->textfont();
}

void CMemoInput::textFont(Fl_Font f) {
    ((CEditor *)m_control)->textfont(f);
}

uchar CMemoInput::textSize() const {
    return ((CEditor *)m_control)->textsize();
}

void CMemoInput::textSize(uchar s) {
    ((CEditor *)m_control)->textsize(s);
}

void CMemoInput::save(CQuery *updateQuery) {
    if (!m_fieldName.length())
        return;
    CParam& param = updateQuery->param(m_fieldName.c_str());
    param.setText(data().getString());
}

bool CMemoInput::preferredSize(int& w,int& h) {
    if (h < 30)
        h = 30;
    if (w < 30)
        w = 30;
    return false;
}
