/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMessageDialog.cpp  -  description
                             -------------------
    begin                : Thu Oct 28 2004
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.h>
#include <sptk3/gui/CMessageDialog.h>
#include <sptk3/string_ext.h>

using namespace std;
using namespace sptk;

CAskDialog::CAskDialog(const char *label,int w)
: CDialog(w,100,label) {

   int textSize = int(Fl::h() / 64 + 0.5);
   if (textSize < 14) textSize = 14;

   CGroup *grp = new CGroup("",65,SP_ALIGN_LEFT);
   m_imageBox = new Fl_Box(8, 8, 50, 50);
   m_imageBox->box(FL_NO_BOX);
   grp->end();

   m_textBox = new CHtmlBox("",30,SP_ALIGN_CLIENT);
   m_textBox->labelWidth(0);
   m_textBox->textSize(textSize);
   m_textBox->color(FL_GRAY);

   m_inputBox = new CInput("",10,SP_ALIGN_BOTTOM);
   m_inputBox->labelWidth(0);
   m_inputBox->textSize(textSize);
   m_inputBox->hide();

   layoutGrowMode(LGM_AUTO_GROW);

   end();
}

bool CAskDialog::execute(string msg) {
   string htmlMessage(msg);
   if (upperCase(htmlMessage).find("<HTML>") == string::npos) {
      htmlMessage = "<HTML><BODY>" + replaceAll(htmlMessage,"\n","<BR>") + "</BODY></HTML>";
   }
   m_textBox->data(htmlMessage);
   relayout();
   return showModal();
}	

bool CInputDialog::execute(string msg,string& inputText) {
   m_inputBox->data(inputText.c_str());
   if (CAskDialog::execute(msg)) {
      inputText = m_inputBox->data().asString().c_str();
      return true;
   }
   return false;
}

int sptk::spAsk(string message) {
   CAskDialog ask("Please, confirm");
   ask.icon(CThemes::getIconImage("question",IS_DIALOG_ICON));
   return ask.execute(message);
}

int sptk::spInformation(string message) {
   CMessageDialog ask("Information");
   ask.icon(CThemes::getIconImage("info",IS_DIALOG_ICON));
   return ask.execute(message);
}

int sptk::spWarning(string message) {
   CMessageDialog ask("Warning");
   ask.icon(CThemes::getIconImage("warning",IS_DIALOG_ICON));
   return ask.execute(message);
}

int sptk::spError(string message) {
   CMessageDialog ask("Error");
   ask.icon(CThemes::getIconImage("error",IS_DIALOG_ICON));
   return ask.execute(message);
}

int sptk::spInput(string message,string& inputText) {
   CInputDialog ask("Please, input");
   ask.icon(CThemes::getIconImage("question",IS_DIALOG_ICON));
   return ask.execute(message,inputText);
}
