/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CPngImage.cpp  -  description
                             -------------------
    begin                : 3 Sep 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.H>
#include <sptk3/gui/CPngImage.h>
#include <zlib.h>
#include <png.h>
#ifdef __UNIX_COMPILER__
#include <unistd.h>
#endif

using namespace sptk;
using namespace std;

typedef struct {
    unsigned       read_offset;
    const CBuffer* buffer;
} CMemHandle;

static void png_read(png_structp pp, png_bytep buf, png_size_t len) {
    CMemHandle *p = (CMemHandle *)pp->io_ptr;
    const CBuffer* buffer = p->buffer;
    png_size_t tail = buffer->size() - p->read_offset;
    if (len > tail)
        len = tail;
    memcpy(buf,p->buffer->data()+p->read_offset,len);
    p->read_offset += len;
}

void CPngImage::load(const CBuffer& imagedata) {
    int i;
    int channels;
    png_structp pp; // PNG read pointer
    png_infop info; // PNG info pointer
    png_bytep *rows; // PNG row pointers
    CMemHandle p;
    //png_byte junk[16];

    pp = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    info = png_create_info_struct(pp);

    p.read_offset = 0;
    p.buffer = &imagedata;

    png_set_read_fn(pp, (void *)&p, png_read);
    png_read_info(pp, info);

    if (info->color_type == PNG_COLOR_TYPE_PALETTE)
        png_set_expand(pp);
    if (info->color_type & PNG_COLOR_MASK_COLOR)
        channels = 3;
    else
        channels = 1;

    if ((info->color_type & PNG_COLOR_MASK_ALPHA) || info->num_trans)
        channels++;

    w((int)(info->width));
    h((int)(info->height));
    d(channels);

    if (info->bit_depth < 8) {
        png_set_packing(pp);
        png_set_expand(pp);
    } else if (info->bit_depth == 16)
        png_set_strip_16(pp);

#if defined(HAVE_PNG_GET_VALID) && defined(HAVE_PNG_SET_TRNS_TO_ALPHA)
    // Handle transparency...
    if (png_get_valid(pp, info, PNG_INFO_tRNS))
        png_set_tRNS_to_alpha(pp);
#endif // HAVE_PNG_GET_VALID && HAVE_PNG_SET_TRNS_TO_ALPHA

    array = new uchar[w() * h() * d()];
    alloc_array = 1;

    // Allocate pointers...
    rows = new png_bytep[h()];

    for (i = 0; i < h(); i ++)
        rows[i] = (png_bytep)(array + i * w() * d());

    // Read the image, handling interlacing as needed...
    for (i = png_set_interlace_handling(pp); i > 0; i --)
        png_read_rows(pp, rows, NULL, h());

#ifdef WIN32
    // Some Windows graphics drivers don't honor transparency when RGB == white
    if (channels == 4) {
        // Convert RGB to 0 when alpha == 0...
    uchar *ptr = (uchar *)array;
    for (i = w() * h(); i > 0; i --, ptr += 4)
        if (!ptr[3])
            ptr[0] = ptr[1] = ptr[2] = 0;
    }
#endif // WIN32

    // Free memory and return...
    delete[] rows;

    png_destroy_read_struct(&pp, &info, NULL);
}

CPngImage::CPngImage(const CBuffer& imagedata)
        : Fl_RGB_Image(0, 0, 0) {
    load(imagedata);
}

CPngImage::CPngImage(const Fl_RGB_Image* image)
        : Fl_RGB_Image(0, 0, 0) {
    unsigned arraySize = image->w() * image->h() * image->d();
    array = new uchar[arraySize];
    alloc_array = 1;
    memcpy((void*)array,image->array,arraySize);
    w(image->w());
    h(image->h());
    d(image->d());
    ld(image->ld());
}

CPngImage::CPngImage(string fileName)
    : Fl_RGB_Image(0, 0, 0) {
    try {
        CBuffer imageData;
        imageData.loadFromFile(fileName);
        load(imageData);
    } catch (...) {}
}

static Fl_RGB_Image* subRGBImage(Fl_RGB_Image* image,unsigned offsetX,unsigned offsetY,unsigned width,unsigned height) {
    if (offsetX > (unsigned) image->w())
        offsetX = image->w();
    if (offsetY > (unsigned) image->h())
        offsetY = image->h();
    if (offsetX + width > (unsigned) image->w())
        width = (unsigned)image->w() - offsetX;
    if (offsetY + height > (unsigned) image->h())
        height = (unsigned)image->h() - offsetY;
    unsigned  bytesPerRow = image->w() * image->d();
    unsigned  newBytesPerRow = width * image->d();
    unsigned  totalBytes = height * newBytesPerRow;
    const uchar* imageArray = image->array;
    uchar* newArray = new uchar[totalBytes];
    Fl_RGB_Image* newImage = new Fl_RGB_Image((uchar *)newArray,width,height,image->d());
    newImage->alloc_array = 1;
    //unsigned maxRow = offsetY + height;
    //unsigned maxCol = offsetX + width;

    const uchar *imageRow = imageArray + offsetY * bytesPerRow + offsetX * image->d();
    for (unsigned row = 0; row < height; row++) {
        memcpy(newArray,imageRow,newBytesPerRow);
        newArray += newBytesPerRow;
        imageRow += bytesPerRow;
    }
    return newImage;
}

void CPngImage::cutStretchDraw(int srcX,int srcY,int srcW,int srcH,int destX,int destY,int destW,int destH) {
    if (destW <= 0 || destH <= 0) return;
    Fl_RGB_Image* img = subRGBImage(this,srcX,srcY,srcW,srcH);
    Fl_RGB_Image* stretched = (Fl_RGB_Image*)img->copy(destW,destH);
    stretched->draw(destX,destY);
    delete img;
    delete stretched;
}

void CPngImage::cutTileDraw(int srcX,int srcY,int srcW,int srcH,int destX,int destY,int destW,int destH) {
    if (destW <= 0 || destH <= 0) return;
    Fl_RGB_Image* img = subRGBImage(this,srcX,srcY,srcW,srcH);
    fl_push_clip(destX,destY,destW,destH);
    for (int y = destY; y < destY + destH; y += srcH)
        for (int x = destX; x < destX + destW; x += srcW)
            img->draw(x,y);
    fl_pop_clip();
    delete img;
}

void CPngImage::drawResized(int xx,int yy,int ww,int hh,int cornerWidth,CPatternDrawMode drawMode,bool drawBackground) {
    int xSideSpace = w() - cornerWidth * 2;
    if (xSideSpace < 2)
        xSideSpace = 2;
    int cornerSizeX = (w() - xSideSpace) / 2;

    int ySideSpace = h() - cornerWidth * 2;
    if (ySideSpace < 2)
        ySideSpace = 2;
    int cornerSizeY = (h() - ySideSpace) / 2;

    if (cornerSizeX > ww/2)
        cornerSizeX = ww/2;
    if (cornerSizeY > hh/2)
        cornerSizeY = hh/2;
    for (int destY = yy, srcY = 0; destY < yy + hh; destY += hh-cornerSizeY, srcY += h()-cornerSizeY )
        for (int destX = xx, srcX = 0; destX < xx + ww; destX += ww-cornerSizeX, srcX += w()-cornerSizeX )
            draw(destX,destY,cornerSizeX,cornerSizeY,srcX,srcY);

    if (drawMode == PDM_STRETCH) {
        cutStretchDraw(cornerSizeX,0,w()-cornerSizeX*2,cornerSizeY,xx+cornerSizeX,yy,ww-cornerSizeX*2,cornerSizeY);
        cutStretchDraw(cornerSizeX,h()-cornerSizeY,w()-cornerSizeX*2,cornerSizeY,xx+cornerSizeX,yy+hh-cornerSizeY,ww-cornerSizeX*2,cornerSizeY);
        cutStretchDraw(0,cornerSizeY,cornerSizeX,h()-cornerSizeY*2,xx,yy+cornerSizeY,cornerSizeX,hh-cornerSizeY*2);
        cutStretchDraw(w()-cornerSizeX,cornerSizeY,cornerSizeX,h()-cornerSizeY*2,xx+ww-cornerSizeX,yy+cornerSizeY,cornerSizeX,hh-cornerSizeY*2);
        if (drawBackground)
            cutStretchDraw(cornerSizeX,cornerSizeY,w()-cornerSizeX*2,h()-cornerSizeY*2,xx+cornerSizeX,yy+cornerSizeY,ww-cornerSizeX*2,hh-cornerSizeY*2);
    } else {
        cutTileDraw(cornerSizeX,0,w()-cornerSizeX*2,cornerSizeY,xx+cornerSizeX,yy,ww-cornerSizeX*2,cornerSizeY);
        cutTileDraw(cornerSizeX,h()-cornerSizeY,w()-cornerSizeX*2,cornerSizeY,xx+cornerSizeX,yy+hh-cornerSizeY,ww-cornerSizeX*2,cornerSizeY);
        cutTileDraw(0,cornerSizeY,cornerSizeX,h()-cornerSizeY*2,xx,yy+cornerSizeY,cornerSizeX,hh-cornerSizeY*2);
        cutTileDraw(w()-cornerSizeX,cornerSizeY,cornerSizeX,h()-cornerSizeY*2,xx+ww-cornerSizeX,yy+cornerSizeY,cornerSizeX,hh-cornerSizeY*2);
        if (drawBackground)
            cutTileDraw(cornerSizeX,cornerSizeY,w()-cornerSizeX*2,h()-cornerSizeY*2,xx+cornerSizeX,yy+cornerSizeY,ww-cornerSizeX*2,hh-cornerSizeY*2);
    }

}
