/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CRadioButtons.cpp  -  description
                             -------------------
    begin                : Thi Dec 12 2002
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.H>
#include <FL/Fl_Round_Button.H>
#include <FL/fl_draw.H>

#include <sptk3/gui/CScroll.h>

#include <stdio.h>
#include <string.h>

#include <sptk3/gui/CDataControl.h>
#include <sptk3/gui/CRadioButtons.h>
#include <sptk3/gui/CThemes.h>

using namespace std;
using namespace sptk;

namespace sptk {

class CRadioButton : public Fl_Round_Button, public CLayoutClient {
protected:
    virtual void draw();
public:
    CRadioButton(const char * label,int layoutSize,CLayoutAlign layoutAlignment);
    int handle(int);
    virtual bool preferredSize(int& w,int& h);
};

}

CRadioButton::CRadioButton(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : Fl_Round_Button(0,0,layoutSize,layoutSize,label), CLayoutClient(this,layoutSize,layoutAlignment) {}

void CRadioButton::draw() {
    int bh = 0;
    int bw = 0;
    CThemes::sizeButton(THM_BUTTON_RADIO,bw,bh);
    bool highlited = value() != 0;
    if (CThemes::drawRadioButton(x(),y(),highlited,highlited)) {
        if (!m_label.empty()) {
            fl_color(labelcolor());
            fl_font(labelfont(),labelsize());
            fl_draw(m_label.c_str(),x()+bw+4,y()+2,w()-bw-6,h()-4,Fl_Align(FL_ALIGN_LEFT|FL_ALIGN_WRAP));
            if (Fl::focus() == this)
                draw_focus(FL_FLAT_BOX, x()+bw+2,y()+1,w()-bw-2,h()-1);
        }
    } else
        Fl_Round_Button::draw();
}

bool CRadioButton::preferredSize(int& w,int& h) {
    int bh = 12;
    int bw = 12;
    CThemes::sizeButton(THM_BUTTON_RADIO,bw,bh);

    fl_font(labelfont(),labelsize());
    w -= bw + 6;
    fl_measure(m_label.c_str(),w,h);
    w += bw + 6;
    if (h < labelsize())
        h = labelsize();
    if (h < bh)
        h = bh;
    return false;
}

int CRadioButton::handle(int event) {
    if (event == FL_FOCUS) {
        CControl *control = dynamic_cast<CControl *>(parent());
        if (!control && parent())
            control = dynamic_cast<CControl *>(parent()->parent());
        if (control)
            control->notifyFocus();
    }
    return Fl_Button::handle(event);
}

void CRadioButtons::radioButtonsCallback(Fl_Widget *w,void *) {
    Fl_Button      *b = (Fl_Button *) w;
    Fl_Group       *g = b->parent();
    CRadioButtons  *r = (CRadioButtons *)g->parent();
    r->deselectAllButtons();
    b->value(1);
    g->redraw();
    r->controlDataChanged();
}

CRadioButtons::CRadioButtons(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CButtonGroup (label,layoutSize,layoutAlignment) {}

#ifdef __COMPATIBILITY_MODE__
CRadioButtons::CRadioButtons(int x,int y,int w,int h,const char *l)
        : CButtonGroup (x,y,w,h,l) {}
#endif

CLayoutClient* CRadioButtons::creator(CXmlNode *node) {
    CRadioButtons* widget = new CRadioButtons("",10,SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

Fl_Button *CRadioButtons::createButton(const char *label,int sz,CLayoutAlign layoutAlignment) {
    Fl_Button *btn = new CRadioButton(label,sz,layoutAlignment);
    btn->callback(CRadioButtons::radioButtonsCallback);
    btn->when(FL_WHEN_CHANGED);
    return btn;
}

int32_t CRadioButtons::intValue() const {
    CScroll *group = (CScroll *)m_control;
    uint32_t    cnt = group->children();
    for (uint32_t i = 0; i < cnt; i++) {
        Fl_Button *b = dynamic_cast<Fl_Button *>(group->child(i));
        if (!b)
            continue;
        if (b->value())
            return (int32_t)(long) b->user_data();
    }
    return 0;
}

void CRadioButtons::intValue(int32_t v) {
    deselectAllButtons();
    CScroll *group = (CScroll *)m_control;
    unsigned    cnt = group->children();
    for (unsigned i = 0; i < cnt; i++) {
        Fl_Button *b = dynamic_cast<Fl_Button *>(group->child(i));
        if (!b)
            continue;
        if (long(b->user_data()) == v) {
            b->value(1);
            break;
        }
    }
}
