/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CScrollBar.cpp -  description
                             -------------------
    begin                : 29 December 2004
    copyright            : (C) Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#if defined(WIN32)
#pragma warning (disable: 4786)
#endif

#include <FL/Fl.h>
#include <FL/fl_draw.h>
#include <sptk3/gui/CScrollBar.h>
#include <sptk3/gui/CThemes.h>

//#include <stdio.h>

using namespace sptk;

int CScrollBar::handle(int event) {
    int area;  // area of scrollbar
    int X=x(), Y=y(), W=w(), H=h();

    // adjust slider area to be inside the arrow buttons:
    if (horizontal()) {
        if (W >= 3*H) {
            X += H;
            W -= 2*H;
        }
    } else {
        if (H >= 3*W) {
            Y += W;
            H -= 2*W;
        }
    }

    // which widget part is highlighted?
    int relx;
    int ww;
    if (horizontal()) {
        relx = Fl::event_x()-X;
        ww = W;
    } else {
        relx = Fl::event_y()-Y;
        ww = H;
    }
    if (relx < 0)
        area = 1;
    else if (relx >= ww)
        area = 2;
    else {
        int S = int(slider_size()*ww+.5);
        int T = (horizontal() ? H : W)/2+1;
        if (S < T)
            S = T;
        double val =
            (maximum()-minimum()) ? (value()-minimum())/(maximum()-minimum()) : 0.5;
        int sliderx;
        if (val >= 1.0)
            sliderx = ww-S;
        else if (val <= 0.0)
            sliderx = 0;
        else
            sliderx = int(val*(ww-S)+.5);
        if (Fl::event_button() == FL_MIDDLE_MOUSE)
            area = 8;
        else if (relx < sliderx)
            area = 5;
        else if (relx >= sliderx+S)
            area = 6;
        else
            area = 8;
    }

    switch (event) {
    case FL_ENTER:
    case FL_LEAVE:
        break;
    case FL_RELEASE:
        m_pushed = 0;
        break;
    case FL_PUSH:
        if (m_pushed)
            break;
        if (area != 8)
            m_pushed = area;
        break;
    }
    return Fl_Scrollbar::handle(event);
}

void CScrollBar::draw() {
    // Use the theme support to draw the scroll bars
    CThemes::drawScrollBar(x(),y(),w(),h(),(float)minimum(),(float)maximum(),(float)value(),slider_size(),m_pushed,active_r()!=0);
}

bool CScrollBar::preferredSize(int& w,int& h) {
    if (!CThemes::sizeScrollBar(w,h)) {
        if (horizontal()) // Horizontal scroll bar
            h = 16;
        else  // Vertical scroll bar
            w = 16;
    }
    return false;
}
