/***************************************************************************
                         SIMPLY POWERFUL TOOLKIT (SPTK)
                         CSplitter.cpp -  description
                            -------------------
   begin                : Tue Jan 27 2004
   copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
   email                : alexeyp@gmail.com
***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <stdio.h>
#include <FL/Fl.h>
#include <FL/Fl_Window.h>
#include <FL/fl_draw.h>
#include <sptk3/gui/CSplitter.h>
#include <sptk3/gui/CGroup.h>

using namespace std;
using namespace sptk;

CSplitter::CSplitter( const char * label, int layoutSize, CLayoutAlign layoutAlign )
: CBox( label, layoutSize, layoutAlign ) {
   m_chainedWidget = 0L;
   m_dragging = false;
   box(FL_THIN_UP_BOX);
}

CLayoutClient* CSplitter::creator(CXmlNode *node) {
  CSplitter* widget = new CSplitter("",10,SP_ALIGN_TOP);
  widget->load(node,LXM_LAYOUTDATA);
  return widget;
}

int CSplitter::handle( int event ) {

   switch ( event ) {
      case FL_ENTER:
         switch (m_layoutAlign) {
            case SP_ALIGN_TOP:
            case SP_ALIGN_BOTTOM:
               fl_cursor( FL_CURSOR_NS );
               break;
            case SP_ALIGN_RIGHT:
            case SP_ALIGN_LEFT:
               fl_cursor( FL_CURSOR_WE );
               break;
            default:
               break;
         }
         break;

      case FL_LEAVE:
         fl_cursor( FL_CURSOR_ARROW ); 
         break;

      case FL_PUSH:
         m_lastDragX = Fl::event_x();
         m_lastDragY = Fl::event_y();
         m_dragging = true;
         findChainedControl();
         return true;

      case FL_RELEASE:
         m_dragging = false;
         return true;

      case FL_DRAG:
         if (m_chainedWidget) {
            //puts("DRAGGING");
            int dx = Fl::event_x() - m_lastDragX;
            int dy = Fl::event_y() - m_lastDragY;
            m_lastDragX = Fl::event_x();
            m_lastDragY = Fl::event_y();
            int newW = m_chainedWidget->w();
            int newH = m_chainedWidget->h();
            switch (m_layoutAlign) {
               case SP_ALIGN_TOP:
                  newH += dy;
                  m_chainedWidgetLayout->preferredSize(newW,newH);
                  m_chainedWidgetLayout->layoutSize(newH);
                  break;
               case SP_ALIGN_BOTTOM:
                  newH -= dy;
                  m_chainedWidgetLayout->preferredSize(newW,newH);
                  m_chainedWidgetLayout->layoutSize(newH);
                  break;
               case SP_ALIGN_LEFT:
                  newW += dx;
                  m_chainedWidgetLayout->preferredSize(newW,newH);
                  m_chainedWidgetLayout->layoutSize(newW);
                  break;
               case SP_ALIGN_RIGHT:
                  newW -= dx;
                  m_chainedWidgetLayout->preferredSize(newW,newH);
                  m_chainedWidgetLayout->layoutSize(newW);
                  break;
               default:
                  break;
            }
            CLayoutManager *parentManager = dynamic_cast<CLayoutManager *>(parent());
            if (parentManager) {
               parentManager->relayout();
               CGroup *parentGroup = dynamic_cast<CGroup *>(parent());
               if (parentGroup) {
                  //puts("Group");
                  parentGroup->redraw();
               } else {
                  //puts("Window");
                  window()->redraw();
               }
            }
         }
         return true;
   }

   return CBox::handle( event );
}

void CSplitter::findChainedControl() {
   m_chainedWidget = 0;
   try {
      if (!dynamic_cast<CLayoutManager *>(parent()))
         return;
      Fl_Group *group = parent();
      unsigned cnt = group->children();
      Fl_Widget *priorWidget = 0;
      Fl_Widget *nextWidget = 0;
      CLayoutClient *priorWidgetLayout = 0;
      CLayoutClient *nextWidgetLayout = 0;
      unsigned index = (unsigned)-1;
      for (unsigned i = 1; i < cnt; i++) {
         if (group->child(i) == this) {
            index = i;
            break;
         }
      }
      if (index < cnt - 1) {
         try {
            priorWidget = group->child(index-1);
            priorWidgetLayout = dynamic_cast<CLayoutClient *>(group->child(index-1));
         }
         catch (...) {}
         try {
            nextWidget = group->child(index+1);
            nextWidgetLayout = dynamic_cast<CLayoutClient *>(group->child(index+1));
         }
         catch (...) {}
      }
      if (priorWidget && priorWidgetLayout->layoutAlign() != SP_ALIGN_CLIENT) {
         m_chainedWidget = priorWidget;
         m_chainedWidgetLayout = priorWidgetLayout;
      } else {
         m_chainedWidget = nextWidget;
         m_chainedWidgetLayout = nextWidgetLayout;
      }

      if (m_chainedWidget) {
         if (m_chainedWidgetLayout->layoutAlign() == SP_ALIGN_NONE) {
            m_chainedWidget = 0;
            return;
         }
         layoutAlign(m_chainedWidgetLayout->layoutAlign());
      }
   } catch (...) {
      return;
   }
}
