/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CToolBar.cpp  -  description
                             -------------------
    begin                : Mon Jun 19 2000
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.h>
#include <FL/Fl_Window.h>
#include <sptk3/gui/CToolBar.h>

using namespace std;
using namespace sptk;

CToolBar::CToolBar(CLayoutAlign layoutAlignment)
        : CGroup("",5,layoutAlignment) {
    m_colapsed = false;
    layoutSpacing(4);
    box(FL_UP_BOX);
}

#ifdef __COMPATIBILITY_MODE__
CToolBar::CToolBar(int x,int y,int w,int h)
        : CGroup(x, y, w, h) {
    m_colapsed = false;
    layoutSpacing(3);
    box(FL_UP_BOX);
}
#endif

CLayoutClient* CToolBar::creator(CXmlNode *node) {
    CToolBar* widget = new CToolBar(SP_ALIGN_TOP);
    widget->load(node,LXM_LAYOUTDATA);
    return widget;
}

CButton *CToolBar::addButton(CButtonKind buttonKind,Fl_Callback_p cb,const char *label,CLayoutAlign buttonAlign) {
    CButton *b = new CButton(buttonKind,buttonAlign,label);
    if (cb)
        b->callback(cb);
    return b;
}

#define HIDE_BAR_WIDTH 8
#define HIDE_BAR_WIDTH_COLLAPSED 30

void CToolBar::resize(int xx,int yy,int ww,int hh) {
    if (m_colapsed) {
        Fl_Widget::resize(xx,yy,ww,HIDE_BAR_WIDTH+Fl::box_dh(box()));
    } else {
        Fl_Widget::resize(xx,yy,ww,hh);
        xx += HIDE_BAR_WIDTH;
        ww -= HIDE_BAR_WIDTH;
        autoLayout(xx,yy,ww,hh,true);
    }
}

bool CToolBar::preferredSize(int& ww,int& hh) {
    if (m_colapsed) {
        hh = HIDE_BAR_WIDTH+Fl::box_dh(box());
        int wm = HIDE_BAR_WIDTH_COLLAPSED+Fl::box_dw(box());
        if (ww < wm)
            ww = wm;
    } else {
        ww -= HIDE_BAR_WIDTH + 1;
        autoLayout(0,0,ww,hh,false);
        ww += HIDE_BAR_WIDTH + 1;
    }
    return false;
}

int CToolBar::handle(int event) {
    if (event == FL_PUSH) {
        int xzone = HIDE_BAR_WIDTH;
        if (m_colapsed)
            xzone = HIDE_BAR_WIDTH_COLLAPSED;

        if (Fl::event_x() <= x() + xzone) {
            m_colapsed = !m_colapsed;
            CLayoutManager *parentLayoutManager = dynamic_cast<CLayoutManager *>(parent());
            //CLayoutManager *parentLayoutManager = dynamic_cast<CLayoutManager *>(window());
            if (parentLayoutManager)
                parentLayoutManager->relayout();
            else 
                relayout();
            window()->redraw();
            return true;
        }
    }
    return CGroup::handle(event);
}

void CToolBar::draw() {
    int dx = Fl::box_dx(box());
    int dy = Fl::box_dy(box());
    int dh = Fl::box_dh(box());
    int xs = x()+dx+1;
    int ys = y()+dy+1;
    int hs = h()-dh-2;
    if (m_colapsed) {
        draw_box();
        //fl_draw_box(FL_THIN_UP_BOX,xs,ys,HIDE_BAR_WIDTH_COLLAPSED,hs,color());
        //xs += 1;
        //ys += 1;
        int xe = xs + HIDE_BAR_WIDTH_COLLAPSED - 4;
        for (int xx = xs; xx < xe; xx+= 3) {
            fl_draw_box(FL_THIN_UP_BOX,xx,ys,2,2,FL_WHITE);
            fl_draw_box(FL_THIN_UP_BOX,xx+1,ys+3,2,2,FL_WHITE);
        }
    } else {
        CGroup::draw();
        int ye = ys + hs - 2;
        for (int yy = ys; yy < ye; yy+= 3) {
            fl_draw_box(FL_THIN_UP_BOX,xs,yy,2,2,FL_WHITE);
            fl_draw_box(FL_THIN_UP_BOX,xs+3,yy+1,2,2,FL_WHITE);
        }
    }
}
