/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTreeView.cpp  -  description
                             -------------------
    begin                : June 21 2003
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/Fl.h>

#include <sptk3/gui/CDataControl.h>
#include <sptk3/gui/CTreeControl.h>
#include <sptk3/gui/CTreeView.h>

using namespace std;
using namespace sptk;

void CTreeView::internal_callback(Fl_Widget *widget,void *) {
    CTreeView *tree = (CTreeView *)widget->parent();
    tree->do_callback();
}

void CTreeView::ctor_init() {
    m_controlFlags = FGE_MULTILINEENTRY;
    m_treeControl = new CTreeControl("",10,SP_ALIGN_NONE);
    m_control = m_treeControl;
    m_control->callback(CControl::internalCallback);
    labelWidth(0);
    end();
}

CTreeView::CTreeView(const char * label,int layoutSize,CLayoutAlign layoutAlignment)
        : CControl(label,layoutSize,layoutAlignment) {
    ctor_init();
}

#ifdef __COMPATIBILITY_MODE__
CTreeView::CTreeView(int x,int y,int w,int h,const char *label)
        : CControl(x,y,w,h,label) {
    ctor_init();
}
#endif

CTreeView::~CTreeView() {}

CLayoutClient* CTreeView::creator(CXmlNode *node) {
    CTreeView* widget = new CTreeView("",10,SP_ALIGN_TOP);
    widget->load(node);
    return widget;
}

std::string CTreeView::selectedPath() const {
    std::string result;
    CTreeItem *node = m_treeControl->selected();
    if (!node) return "";
    do {
        result = "/" + std::string(node->label()) + result;
        node = node->parentItem();
    } while (node && node->parentItem());
    return result;
}

void CTreeView::resize(int x,int y,int w,int h) {
    CControl::resize(x,y,w,h);

    w -= m_labelWidth;
    x += m_labelWidth;

    if (m_menuButton)
        m_menuButton->resize(x,y,w,this->h());

    m_treeControl->resize(x,y,w,h);
}

void CTreeView::load(CQuery *loadQuery) {
    if (!m_fieldName.length()) return;
    CField& fld = (*loadQuery)[m_fieldName.c_str()];
    data(fld);
}

void CTreeView::save(CQuery *updateQuery) {
    if (!m_fieldName.length()) return;
    CParam& param = updateQuery->param(m_fieldName.c_str());
    param = data();
}

void CTreeView::load(const CXmlNode *node) {
    m_treeControl->load(node);
}

void CTreeView::save(CXmlNode *node) const {
    m_treeControl->save(node);
}

CVariant CTreeView::data() const {
    CTreeItem *node = m_treeControl->selected();
    if (node) return (int32_t)(long) node->user_data();
    return int32_t(0);
}

void CTreeView::data(const CVariant v) {
    CTreeItem *node = m_treeControl->findData((void *)v.asInteger());
    if (node)
        m_treeControl->makeVisible(node);
}
