/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWindow.cpp  -  description
                             -------------------
    begin                : Sun May 18 2003
    copyright            : (C) 2000-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/cxml>
#include <sptk3/gui/CWindow.h>
#include <FL/Fl.H>

using namespace std;
using namespace sptk;

void CWindow::resize(int xx,int yy,int ww,int hh) {
    Fl_Double_Window::resize(xx,yy,ww,hh);
    int www = ww, hhh = hh;
    // For a window, layout is always located at 0,0 coordinates
    autoLayout(0,0,www,hhh,true);
    bool limited = false;
    if (www > Fl::w())      {
        www = Fl::w();
        limited = true;
    }
    if (hhh > Fl::h())      {
        hhh = Fl::h();
        limited = true;
    }
    if (xx + www > Fl::w()) {
        xx = Fl::w() - www;
        limited = true;
    }
    if (yy + hhh > Fl::h()) {
        yy = Fl::h() - hhh;
        limited = true;
    }

    if (limited || ww != www || hh != hhh)
        Fl_Double_Window::resize(xx,yy,www,hhh);

    if (m_shapeExtension)
        resizingShape(ww,hh);
}

bool CWindow::preferredSize(int& ww,int& hh) {
    return autoLayout(0,0,ww,hh,false);
}

void CWindow::hide() {
    m_borderCleared = false;
    m_shapeChanged = true;
    Fl_Double_Window::hide();
}

void CWindow::draw() {
    if (m_shapeExtension && m_shapeChanged) {
        resizingShape(w(),h());
        shapeApply();
    }

    int savex = x();
    x(0);
    int savey = y();
    y(0);

    Fl_Boxtype b = box();
    if (damage() & ~FL_DAMAGE_CHILD) { // redraw the entire thing:
        draw_box();
        paintBackground();
    }

    box(FL_NO_BOX);
    Fl_Double_Window::draw();
    box(b);

    y(savey);
    x(savex);
}

void CWindow::load(const CXmlNode* node,CLayoutXMLmode xmlMode) throw(CException) {
    CLayoutManager::loadLayout(node,xmlMode);
    loadPosition(node);
}

void CWindow::save(CXmlNode* node,CLayoutXMLmode xmlMode) const {
    CLayoutManager::saveLayout(node,xmlMode);
    savePosition(node);
}

void CWindow::loadPosition(const CXmlNode* node) {
    int hh = node->getAttribute("height");
    if (!hh)
        hh = node->getAttribute("h");
    int ww = node->getAttribute("width");
    if (!ww)
        ww = node->getAttribute("w");
    if (hh > 0 && ww > 0)
        resize(node->getAttribute("x","0"),node->getAttribute("y","0"),ww,hh);
}

void CWindow::savePosition(CXmlNode* node) const {
    node->setAttribute("x",x());
    node->setAttribute("y",y());
    node->setAttribute("width",w());
    node->setAttribute("height",h());
    node->setAttribute("label",label());
    node->name(className());
}

int CWindow::handle(int event) {
    int rc = Fl_Window::handle(event);

    if (!m_shapeExtension)
        return rc;

    switch ( event ) {

    case FL_PUSH:
        m_pushedX = Fl::event_x();
        m_pushedY = Fl::event_y();
        m_resizingZone = mouseZone(m_pushedX,m_pushedY);
        break;

    case FL_DRAG:
        if (m_resizingZone) {
            int x,y;
            Fl::get_mouse(x,y);
            changeSize(x,y);
        } else {
            int new_x, new_y;
            new_x = Fl::event_x() + x() - m_pushedX;
            new_y = Fl::event_y() + y() - m_pushedY;
            position(new_x,new_y);
        }
        return 1;
    }

    if (shapeCursorHandle(event))
        return 1;

    return rc;
}
