/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CTar.cpp  -  description
                             -------------------
    begin                : Fri Sep 1 2006
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.

    This module creation was sponsored by Total Knowledge (http://www.total-knowledge.com).
    Author thanks the developers of CPPSERV project (http://www.total-knowledge.com/progs/cppserv)
    for defining the requirements for this class.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CTar.h>
#include <errno.h>
#include <string.h>
#include "libtar.h"
#include <sptk3/CException.h>

#ifdef __UNIX_COMPILER__
#include <unistd.h>
#else
#include <io.h>
#endif

using namespace std;
using namespace sptk;

extern "C" {
   typedef int (*CMemOpenCallback)  (const char*,int,...);
   typedef int (*CMemCloseCallback) (int);
   typedef int (*CMemReadCallback)  (int,void*,size_t);
   typedef int (*CMemWriteCallback) (int,const void*,size_t);
}

#ifdef _MSC_VER
#define lseek _lseek
#endif

static tartype_t memtype;
int            CTar::lastTarHandle;
CTarHandleMap *CTar::tarHandleMap;

CMemoryTarHandle* CTar::tarMemoryHandle(int handle) {
    CTarHandleMap::iterator itor = tarHandleMap->find(handle);
    if (itor == tarHandleMap->end())
        return 0;
    return itor->second;
}

int CTar::mem_open(const char *name, int x, ...) {
    lastTarHandle++;
    CMemoryTarHandle* memHandle = new CMemoryTarHandle;
    (*tarHandleMap)[lastTarHandle] = memHandle;
    return lastTarHandle;
}

int CTar::mem_close(int handle) {
    CTarHandleMap::iterator itor = tarHandleMap->find(handle);
    if (itor == tarHandleMap->end())
        return -1;
    CMemoryTarHandle* memHandle = itor->second;
    delete memHandle;
    tarHandleMap->erase(itor);
    return 0;
}

int CTar::mem_read(int x, void *buf, size_t len) {
    CMemoryTarHandle* memHandle = tarMemoryHandle(x);
    if (!memHandle) return -1;
    if (memHandle->position + len > memHandle->sourceBufferLen)
        len = memHandle->sourceBufferLen - memHandle->position;
    if (buf)
        memcpy(buf, memHandle->sourceBuffer + memHandle->position, len);
    memHandle->position += (uint32_t)len;
    return (int) len;
}

int CTar::mem_write(int x, const void *buf, size_t len) {
    return -1;
}

CTar::CTar() {
    m_tar = 0;
    if (!tarHandleMap) {
        memtype.openfunc  = (CMemOpenCallback)  mem_open;
        memtype.closefunc = (CMemCloseCallback) mem_close;
        memtype.readfunc  = (CMemReadCallback)  mem_read;
        memtype.writefunc = (CMemWriteCallback) mem_write;
        tarHandleMap = new CTarHandleMap;
    }
}

bool CTar::loadFile() throw(CException) {
    TAR* tar = (TAR*)m_tar;
    // Read file header
    int rc = th_read(tar);
    if (rc > 0) return false; // End of archive
    if (rc < 0) throwError(m_fileName);

    string fileName = th_get_pathname(tar);
    uint32_t fileSize = th_get_size(tar);

    if (fileSize) {
        CBuffer *buffer = new CBuffer(fileSize + 1);
        char *buf = buffer->data();

        uint32_t offset = 0;
        while (offset != fileSize) {
            int k = tar->type->readfunc(tar->fd, buf + offset, unsigned(fileSize - offset));
            if (k < 0) {
                delete buffer;
                throw CException("Error reading file '"+fileName+"' from tar archive");
            }
            offset += k;
        }
        buf[fileSize] = '\0';
        buffer->bytes(fileSize);

        m_fileNames.push_back(fileName);
        m_files[fileName] = buffer;

        off_t emptyTail = off_t(T_BLOCKSIZE - fileSize % T_BLOCKSIZE);
        if (m_memoryRead) {
            mem_read(tar->fd,NULL,emptyTail);
        } else {
            if (lseek(tar->fd,emptyTail,SEEK_CUR) == (off_t) -1)
                throwError(m_fileName);
        }
    }
    return true;
}

void CTar::throwError(string fileName) throw(CException) {
    char* ptr = strerror(errno);
    if (fileName.empty())
        throw CException(ptr);
    throw CException(fileName+": " + string(ptr));
}

void CTar::read(const char* fileName) throw(CException) {
    m_fileName = fileName;
    m_memoryRead = false;
    TAR* tar;
    int rc = tar_open(&tar,(char*)fileName,0,0,0,TAR_GNU);
    if (rc < 0) throwError(fileName);
    m_tar = tar;
while (loadFile()) {}
    tar_close((TAR*)m_tar);
    m_tar = 0;
}

void CTar::read(const CBuffer& tarData) throw(CException) {
    m_fileName = "";
    m_memoryRead = true;
    TAR* tar;
    tar_open(&tar,(char *)"memory",&memtype,0,0,TAR_GNU);
    CMemoryTarHandle* memHandle = tarMemoryHandle(tar->fd);
    if (!memHandle)
        throw CException("Can't open the archive",__FILE__,__LINE__);
    memHandle->sourceBuffer = tarData.data();
    memHandle->sourceBufferLen = tarData.bytes();
    m_tar = tar;
while (loadFile()) {}
    tar_close((TAR*)m_tar);
    m_tar = 0;
}

void CTar::clear() {
    for (CFileCollection::iterator itor = m_files.begin(); itor != m_files.end(); itor++)
        delete itor->second;
    m_fileName = "";
    m_files.clear();
    m_fileNames.clear();
}

const CBuffer& CTar::file(std::string fileName) const throw(CException) {
    CFileCollection::const_iterator itor = m_files.find(fileName);
    if (itor == m_files.end())
        throw CException("File '"+fileName+"' isn't found",__FILE__,__LINE__);
    return *(itor->second);
}
