/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFrame.cpp  -  description
                             -------------------
    begin                : Thu Sep 14 2006
    copyright            : (C) 2006-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#include <sptk3/CFrame.h>

using namespace std;
using namespace sptk;

void CFrames::clear() {
    for (CFrameMap::iterator itor = m_frames.begin(); itor != m_frames.end(); itor++) {
        CFrame* frame = itor->second;
        delete frame;
    }
    m_frames.clear();
    m_fltkFrames.clear();
}

const CStrings    CFrames::frameTypeNames("up frame|thin up frame|thin down frame|down frame","|");
const Fl_Boxtype  CFrames::frameTypes[] = {
            FL_UP_FRAME, FL_THIN_UP_FRAME, FL_THIN_DOWN_FRAME, FL_DOWN_FRAME
        };

void CFrames::load(CTar& tar,CXmlNode* framesNode) {
    clear();
    CXmlNode::iterator itor = framesNode->begin();
    CXmlNode::iterator iend = framesNode->end();
    for (; itor != iend; itor++) {
        CXmlNode* frameNode = *itor;
        if (frameNode->name() != "frame")
            continue;
        string fileName = frameNode->getAttribute("image");
        if (fileName.empty())
            continue;
        string frameTypeStr = frameNode->getAttribute("type");
        string frameName    = frameNode->getAttribute("name",frameTypeStr.c_str());
        if (frameTypeStr.empty())
            frameTypeStr = frameName;
        unsigned frameTypeInt = frameTypeNames.indexOf(frameTypeStr);
        unsigned frameWidth = frameNode->getAttribute("width","1");
        unsigned cornerZone = frameNode->getAttribute("corner","1");
        Fl_Boxtype frameType = FL_NO_BOX;
        CFrame::CFrameKind kind = CFrame::USER_EXTENDED;
        if (frameTypeInt < 4) {
            frameType = frameTypes[frameTypeInt];
            kind = CFrame::FLTK_STANDARD;
        }
        CPngImage::CPatternDrawMode drawMode = CPngImage::PDM_STRETCH;
        if (frameNode->getAttribute("mode") == "tile")
            drawMode = CPngImage::PDM_TILE;
        const CBuffer& imageData = tar.file(fileName);
        registerFrame(frameName,new CFrame(imageData,frameWidth,cornerZone,drawMode,kind),frameType);
    }
}

void CFrames::registerFrame(std::string frameName,CFrame* frame,Fl_Boxtype frameType) {
    CFrame* oldFrame = find(frameName);
    if (oldFrame)
        delete oldFrame;
    if (frameType != FL_NO_BOX) {
        m_fltkFrames[frameType] = frame;
    }
    m_frames[frameName] = frame;
}

CFrame* CFrames::find(Fl_Boxtype frameType) const {
    try {
        if (m_fltkFrames.empty())
            return NULL;
        CFltkFrameMap::const_iterator itor = m_fltkFrames.find(frameType);
        if (itor == m_fltkFrames.end())
            return NULL;
        return itor->second;
    } catch (...) {
        return NULL;
    }
}

CFrame* CFrames::find(std::string frameName) const {
    CFrameMap::const_iterator itor = m_frames.find(frameName);
    if (itor == m_frames.end())
        return NULL;
    return itor->second;
}
