/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CIcon.cpp  -  description
                             -------------------
    begin                : Sun Aug 20 2006
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.h>
#include <sptk3/gui/CPngImage.h>
#include <FL/Enumerations.h>
#include <sptk3/sptk.h>
#include <sptk3/gui/CIcon.h>

#include <iostream>

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#endif

using namespace std;
using namespace sptk;

void CIcon::load(const CBuffer& imageData) {
    m_image = new CPngImage(imageData);
    if (!m_image->data()) {
        delete m_image;
        m_image = 0L;
    }
}

void CIconMap::clear() {
    for (iterator itor=begin(); itor!=end(); itor++) {
        CIcon* icon = itor->second;
        if (m_shared)
            delete icon;
        else
        if (!icon->m_shared)
            delete icon;
    }
    map<string,CIcon*>::clear();
}

void CIconMap::insert(CIcon* icon) {
    iterator it = find(icon->name());
    if (it != end()) {
        CIcon* oldIcon = it->second;
        delete oldIcon;
        it->second = icon;
    } else {
        (*this)[icon->name()] = icon;
    }
}

void CIconMap::load(CTar& tar,CXmlNode* iconsNode) {
    clear();
    for (CXmlNode::iterator itor = iconsNode->begin(); itor != iconsNode->end(); itor++) {
        CXmlNode* node = *itor;
        if (node->name() != "icon")
            continue;
        string iconName = node->getAttribute("name");
        string fileName = node->getAttribute("image");
        if (iconName.empty())
            continue;
        try {
            const CBuffer& imageData = tar.file(fileName);
            CIcon* icon = new CIcon(iconName);
            icon->load(imageData);
            if (icon->image()) {
		//cerr << "Load [" << iconName << "]: " << fileName << endl;
                insert(icon);
            } else {
		cerr << "Can't load " << fileName << endl;
                delete icon;
            }
        }
        catch (exception& e) {
            cerr << "ERROR: " << e.what() << endl;
        }
    }
}
