/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CThemes.cpp  -  description
                             -------------------
    begin                : Fri Nov 08 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <FL/fl_draw.h>
#include <FL/Enumerations.h>
#include <sptk3/sptk.h>
#include <sptk3/CDirectoryDS.h>
#include <sptk3/gui/CButton.h>
#include <sptk3/gui/CThemes.h>
#include <sptk3/gui/CTreeControl.h>
#include <sptk3/gui/CPngImage.h>
#include <sptk3/string_ext.h>

#include <sptk3/gui/default_icons.h>

#include <iostream>

#ifdef _WIN32
    #include <winsock2.h>
    #include <windows.h>
#endif

using namespace std;
using namespace sptk;

extern void fl_internal_boxtype(Fl_Boxtype,Fl_Box_Draw_F*);

string         CThemes::m_name;
CFontsVector   CThemes::screenFonts;
CFontsMap      CThemes::m_fonts;
CFrames        CThemes::m_frames;
map<string,Fl_Color> CThemes::m_colorNames;
CTabImages     CThemes::m_tabImages;

Fl_Box_Draw_F *CThemes::m_flatBox;
Fl_Box_Draw_F *CThemes::m_thinUpBox;
Fl_Box_Draw_F *CThemes::m_upBox;
Fl_Box_Draw_F *CThemes::m_thinDownBox;
Fl_Box_Draw_F *CThemes::m_downBox;

Fl_Box_Draw_F *CThemes::m_thinUpFrame;
Fl_Box_Draw_F *CThemes::m_upFrame;
Fl_Box_Draw_F *CThemes::m_thinDownFrame;
Fl_Box_Draw_F *CThemes::m_downFrame;

CPngImage      *CThemes::m_background[7];
CPngImage      *CThemes::m_progressBar[2];
CScrollBarImages CThemes::m_scrollBar[2];
CButtonImage   CThemes::m_normalButtons[4];
CButtonImage   CThemes::m_comboButtons[4];
CButtonImage   CThemes::m_checkButtons[4];
CButtonImage   CThemes::m_radioButtons[4];
CButtonImage   CThemes::m_largeButtons[4];
bool           CThemes::m_flatButtons;

CXmlDoc       *CThemes::m_registry;
CTar           CThemes::m_tar;
CIconMap       CThemes::m_icons[4]; /// Four different sets of icons
bool           CThemes::m_desaturateInactiveButtons;
int            CThemes::m_buttonFocusRadius;

CThemes        themes;

CButtonImage::CButtonImage(CPngImage *img) {
    m_colorImage = m_grayImage = 0L;
    set
        (img,false);
}

void CButtonImage::clear() {
    if (m_colorImage)
        delete m_colorImage;
    if (m_grayImage)
        delete m_grayImage;
    m_colorImage = m_grayImage = 0L;
}

CButtonImage::~CButtonImage() {
    clear();
}

void CButtonImage::set(CPngImage *img,bool desaturateInactive) {
    clear();
    if (img) {
        m_colorImage = new CPngImage(img);
        m_grayImage = new CPngImage(img);
        if (desaturateInactive)
            m_grayImage->desaturate();
    }
}

static CPngImage *loadValidatePNGImage(string fileName) {
    try {
        const CBuffer& imageBuffer = CThemes::m_tar.file(fileName);
        CPngImage *img = new CPngImage(imageBuffer);
        if (img) {
            if (img->data())
                return img;
            delete img;
            img = 0L;
        }
        return img;
    } catch (...) {}
    return 0;
}

void CScrollBarImages::load(int buttonSize,CThemeScrollBarType atype) {
    m_buttonSize = buttonSize;
    string scrollBarDir;
    if (atype == THM_SCROLLBAR_VERTICAL)
        scrollBarDir += "vscrollbar";
    else
        scrollBarDir += "hscrollbar";
    clear();
    for (unsigned i = 0; i < 4; i++)
        m_images[i] = loadValidatePNGImage(scrollBarDir+int2string(i)+".png");
}

void CScrollBarImages::clear() {
    for (unsigned i = 0; i < 4; i++)
        if (m_images[i]) {
            delete m_images[i];
            m_images[i] = 0L;
        }
}

CThemes::CThemes() {
    m_flatBox = Fl::get_boxtype(FL_FLAT_BOX);
    m_thinUpBox = Fl::get_boxtype(FL_THIN_UP_BOX);
    m_upBox = Fl::get_boxtype(FL_UP_BOX);
    m_thinDownBox = Fl::get_boxtype(FL_THIN_DOWN_BOX);
    m_downBox = Fl::get_boxtype(FL_DOWN_BOX);

    m_thinUpFrame = Fl::get_boxtype(FL_THIN_UP_FRAME);
    m_upFrame = Fl::get_boxtype(FL_UP_FRAME);
    m_thinDownFrame = Fl::get_boxtype(FL_THIN_DOWN_FRAME);
    m_downFrame = Fl::get_boxtype(FL_DOWN_FRAME);

    m_registry = new CXmlDoc;
    m_desaturateInactiveButtons = false;
    m_buttonFocusRadius = 0;

    for (unsigned i = 0; i < 7; i++)
        m_background[i] = 0L;

    if (!screenFonts.size()) {
        int k = Fl::set_fonts("*");
        for (int i = 0; i < k; i++) {
            int attributes = 0;
            string fontName = Fl::get_font_name((Fl_Font)i,&attributes);
            CFont* font = new CFont(fontName,10,0,i,attributes);
            screenFonts.push_back(font);
        }
    }

    /// Default widget colors
    m_colorNames["foreground"]  = FL_FOREGROUND_COLOR;
    m_colorNames["background"]  = FL_BACKGROUND2_COLOR;
    m_colorNames["inactive"]    = FL_INACTIVE_COLOR;
    m_colorNames["selection"]   = FL_SELECTION_COLOR;

    set("Default");
}

CThemes::~CThemes() {
    if (m_registry)
        delete m_registry;
}

CIcon* CThemes::getIcon(std::string iconName,CIconSize iconSize) {
    CIconMap::iterator itor = m_icons[iconSize].find(iconName);
    if (itor == m_icons[iconSize].end()) {
        cerr << "Can't find icon '" << iconName << "' for size " << iconSize << endl;
        return 0L;
    }
    return itor->second;
}

CPngImage* CThemes::getIconImage(std::string iconName,CIconSize iconSize) {
    CIcon* icon = getIcon(iconName,iconSize);
    if (icon)
        return icon->image();
    return 0;
}

void CThemes::registerIcon(CIcon* icon,CIconSize iconSize) {
    m_icons[iconSize].insert(icon);
}

const CStrings& CThemes::searchDirectories() {
    static CStrings sd;
    if (sd.size())
        return sd;
#ifdef _WIN32

    char windir[128];
    GetWindowsDirectory(windir,128);
    sd.push_back(windir + string("\\.themes.sptk"));
#else

    sd.push_back("/usr/share/sptk");
    sd.push_back("/usr/local/share/sptk");
#endif

    return sd;
}

void CThemes::reset() {
    Fl::set_boxtype(FL_FLAT_BOX,m_flatBox,0,0,0,0);
    Fl::set_boxtype(FL_THIN_UP_BOX,m_thinUpBox,1,1,2,2);
    Fl::set_boxtype(FL_UP_BOX,m_upBox,2,2,4,4);
    Fl::set_boxtype(FL_THIN_DOWN_BOX,m_thinDownBox,1,1,2,2);
    Fl::set_boxtype(FL_DOWN_BOX,m_downBox,2,2,4,4);

    Fl::set_boxtype(FL_THIN_UP_FRAME,m_thinUpFrame,1,1,2,2);
    Fl::set_boxtype(FL_UP_FRAME,m_upFrame,2,2,4,4);
    Fl::set_boxtype(FL_THIN_DOWN_FRAME,m_thinDownFrame,1,1,2,2);
    Fl::set_boxtype(FL_DOWN_FRAME,m_downFrame,2,2,4,4);

    unsigned i = 0;
    for (; i < 4; i++)
        m_normalButtons[i].clear();

    for (i = 0; i < 2; i++) {
        m_comboButtons[i].clear();
        m_radioButtons[i].clear();
        m_checkButtons[i].clear();
    }
    m_scrollBar[THM_SCROLLBAR_VERTICAL].clear();
    m_scrollBar[THM_SCROLLBAR_HORIZONTAL].clear();

    m_icons[IS_COMBO_ICON].clear();
    m_icons[IS_SMALL_ICON].clear();
    m_icons[IS_LARGE_ICON].clear();
}

CButtonImage *CThemes::sizeToButtonImages(CThemeButtonSize sz) {
    switch (sz) {
    case THM_BUTTON_COMBO:
        return m_comboButtons;
    case THM_BUTTON_CHECK:
        return m_checkButtons;
    case THM_BUTTON_RADIO:
        return m_radioButtons;
    case THM_BUTTON_LARGE:
        return m_largeButtons;
    case THM_BUTTON_NORMAL:
        break;
    }
    return m_normalButtons;
}

void CThemes::replaceButton(CThemeButtonSize sz,CThemeButtonType btndx,string fileName) {
    CPngImage *img = loadValidatePNGImage(fileName);
    CButtonImage *buttons = sizeToButtonImages(sz);
    if (img)
        buttons[btndx].set(img,m_desaturateInactiveButtons);
    else
        buttons[btndx].clear();
}

void CThemes::replaceImage(CPngImage *images[],int ndx,string fileName) {
    if (images[ndx])
        delete images[ndx];
    images[ndx] = loadValidatePNGImage(fileName);
}

void CThemes::set(string themeName) {
    if (themeName == "")
        themeName = "Default";

    if (themeName == m_name)
        return;

    reset();

    const CStrings& dirs = searchDirectories();
    bool defaultTheme = false;
    for (unsigned dn = 0; dn < dirs.size() && !defaultTheme; dn++) {
        string fileName = dirs[dn] + "/" + themeName + ".tar";

        defaultTheme = lowerCase(themeName) == "default";
        m_registry->clear();

        m_desaturateInactiveButtons = false;
        int scrollBarButtonSize = 0;
        if (!defaultTheme) {
            try {
                m_tar.read(fileName);
                m_registry->load(m_tar.file("theme.ini"));
                m_name = themeName;
            } catch(...) {
                continue;
            }
        } else {
            CBuffer defaultThemeBuffer(default_icons,default_icons_len);
            m_tar.read(defaultThemeBuffer);
            m_registry->load(m_tar.file("theme.ini"));
            m_name = themeName;
        }

        try {
            CXmlNode::iterator itor = m_registry->begin();
            for (; itor != m_registry->end(); itor++) {
                CXmlNode* iconsNode = *itor;
                if (iconsNode->name() != "icons")
                    continue;
                string iconsSizeStr = iconsNode->getAttribute("size","large") + " ";
                CIconSize iconsSize;
                switch (iconsSizeStr[0]) {
                case 'c':
                    iconsSize = IS_COMBO_ICON;
                    break;
                case 's':
                    iconsSize = IS_SMALL_ICON;
                    break;
                case 'd':
                    iconsSize = IS_DIALOG_ICON;
                    break;
                default:
                    iconsSize = IS_LARGE_ICON;
                    break;
                }
                m_icons[iconsSize].load(m_tar,iconsNode);
            }

            CXmlNode* buttonsNode = m_registry->findOrCreate("buttons",false);
            m_desaturateInactiveButtons = buttonsNode->getAttribute("DesaturateInactive","N");
            m_buttonFocusRadius = buttonsNode->getAttribute("FocusRadius","0");
            m_flatButtons = buttonsNode->getAttribute("FlatInactive","N");

            CXmlNode* tabsNode = m_registry->findOrCreate("tabs",false);
            m_tabImages.load(m_tar,tabsNode);

            CXmlNode* fontsTopic = m_registry->findOrCreate("fonts",false);
            m_fonts.clear();
            for (itor = fontsTopic->begin(); itor != fontsTopic->end(); itor++) {
                CXmlNode* fontInfo = *itor;
                string fontName = fontInfo->getAttribute("name","Arial");
                CFont* screenFont = screenFonts.find(fontName);
                if (!screenFont)
                    continue;
                string object = fontInfo->getAttribute("object","Arial");
                CFont* font = new CFont(fontName,
                                        fontInfo->getAttribute("size","10"),
                                        fontInfo->getAttribute("color","0"),
                                        screenFont->index(),
                                        screenFont->attributes()
                                       );
                m_fonts[object] = font;
            }
            CFontsMap::iterator ftor = m_fonts.find("default");
            if (ftor == m_fonts.end()) {
                CFont* font = screenFonts[0];
                m_fonts["default"] = new CFont(font->name(),10,0,font->index(),font->attributes());
            }

            Fl::get_system_colors();
            CXmlNode* colorsTopic = m_registry->findOrCreate("colors",false);
            for (itor = colorsTopic->begin(); itor != colorsTopic->end(); itor++) {
                CXmlNode* color = *itor;
                istring colorName = color->getAttribute("name","");
                unsigned red = color->getAttribute("red","0");
                unsigned blue = color->getAttribute("blue","0");
                unsigned green = color->getAttribute("green","0");
                map<string,Fl_Color>::iterator citor = m_colorNames.find(colorName);
                if (citor == m_colorNames.end())
                    continue;
                Fl_Color acolor = citor->second;
                Fl::set_color(acolor,red,green,blue);
            }

            CXmlNode* framesNode = m_registry->findOrCreate("frames",false);
            m_frames.load(m_tar,framesNode);

        } catch (...) {}

        unsigned i;
        for (i = 0; i < 7; i++)
            replaceImage(m_background,i,"background"+int2string(i)+".png");

        for (i = 0; i < 2; i++)
            replaceImage(m_progressBar,i,"progress"+int2string(i)+".png");

        for (i = 0; i < 4; i++)
            replaceButton(THM_BUTTON_NORMAL,CThemeButtonType(i),"button"+int2string(i)+".png");

        for (i = 0; i < 2; i++) {
            replaceButton(THM_BUTTON_COMBO,CThemeButtonType(i),"combo_button"+int2string(i)+".png");
            replaceButton(THM_BUTTON_CHECK,CThemeButtonType(i),"check_button"+int2string(i)+".png");
            replaceButton(THM_BUTTON_RADIO,CThemeButtonType(i),"radio_button"+int2string(i)+".png");
            replaceButton(THM_BUTTON_LARGE,CThemeButtonType(i),"large_button"+int2string(i)+".png");
        }

        CTreeItem::treeOpened   = getIconImage("tree_opened",IS_SMALL_ICON);            ///< Default image of the opened tree
        CTreeItem::treeClosed   = getIconImage("tree_closed",IS_SMALL_ICON);            ///< Default image of the closed tree
        CTreeItem::folderOpened = getIconImage("folder_opened",IS_SMALL_ICON);          ///< Default image of the opened floder
        CTreeItem::folderClosed = getIconImage("folder_closed",IS_SMALL_ICON);          ///< Default image of the closed floder
        CTreeItem::document     = getIconImage("document",IS_SMALL_ICON);               ///< Default image of the document
		if (!CTreeItem::folderOpened)
			CTreeItem::folderOpened = CTreeItem::treeOpened;
		if (!CTreeItem::folderClosed)
			CTreeItem::folderClosed = CTreeItem::treeClosed;

        try {
            CXmlNode& topic = *m_registry->findFirst("scrollbars",false);
            if (&topic)
                scrollBarButtonSize = topic.getAttribute("ButtonSize","16");
        } catch (...) {}

        m_scrollBar[THM_SCROLLBAR_VERTICAL].load(scrollBarButtonSize,THM_SCROLLBAR_VERTICAL);
        m_scrollBar[THM_SCROLLBAR_HORIZONTAL].load(scrollBarButtonSize,THM_SCROLLBAR_HORIZONTAL);

        Fl::set_boxtype(FL_FLAT_BOX,drawFlatBox,0,0,0,0);

        Fl_Boxtype      stdBoxes[] =  { FL_UP_BOX,   FL_THIN_UP_BOX,   FL_DOWN_BOX,   FL_THIN_DOWN_BOX };
        Fl_Boxtype      stdFrames[] = { FL_UP_FRAME, FL_THIN_UP_FRAME, FL_DOWN_FRAME, FL_THIN_DOWN_FRAME };
        Fl_Box_Draw_F*  boxFuncs[] =  { drawUpBox,   drawThinUpBox,    drawDownBox,   drawThinDownBox };
        Fl_Box_Draw_F*  frameFuncs[]= { drawUpFrame, drawThinUpFrame,  drawDownFrame, drawThinDownFrame };
        int             defaultFrameWidth[] = { 2, 1, 2, 1 };
        for (i = 0; i < 4; i++) {
            Fl_Boxtype frameType = stdFrames[i];
            int        frameWidth = defaultFrameWidth[i];
            CFrame* frame = m_frames.find(frameType);
            if (frame) {
                frameWidth = frame->frameWidth();
                Fl::set_boxtype(frameType,frameFuncs[i],frameWidth,frameWidth,frameWidth*2,frameWidth*2);
            }
            Fl::set_boxtype(stdBoxes[i],boxFuncs[i],frameWidth,frameWidth,frameWidth*2,frameWidth*2);
        }

        break;
    }
}

CPngImage *CThemes::imageForColor(Fl_Color clr) {
    if (clr >= FL_DARK3 && clr <= FL_LIGHT3) {
        int ndx = 0;
        switch (clr) {
        case FL_DARK3:
            break;
        case FL_DARK2:
            ndx = 1;
            break;
        case FL_DARK1:
            ndx = 2;
            break;
        case FL_BACKGROUND_COLOR:
            ndx = 3;
            break;
        case FL_LIGHT1:
            ndx = 4;
            break;
        case FL_LIGHT2:
            ndx = 5;
            break;
        case FL_LIGHT3:
            ndx = 6;
            break;
        default:
            return 0;
        }
        return m_background[ndx];
    }
    return 0L;
}

void CThemes::paintBackground(int xx,int yy,int ww,int hh,CPngImage *image) {
    if (!image)
        return;
    if (ww < 0 || hh < 0)
        return;
    int dx = image->w();
    int dy = image->h();
    fl_push_clip(xx,yy,ww,hh);
    int maxx = xx + ww;
    int maxy = yy + hh;
    /// Image alignments
    yy = (yy / dy) * dy;
    xx = (xx / dx) * dx;
    for (int xi = xx; xi < maxx; xi += dx) {
        for (int yi = yy; yi < maxy; yi += dy) {
            image->draw(xi,yi,dx,dy,0,0);
        }
    }
    fl_pop_clip();
}

void CThemes::drawThemeBox(int x,int y,int w,int h,Fl_Color clr,Fl_Boxtype frameType) {
    CPngImage *image = imageForColor(clr);
    if (image) {
        paintBackground(x+Fl::box_dx(frameType),y+Fl::box_dy(frameType),w-Fl::box_dw(frameType),h-Fl::box_dh(frameType),image);
    } else {
        fl_color(clr);
        fl_rectf(x,y,w,h);
    }
    fl_draw_box(frameType,x,y,w,h,clr);
}

void CThemes::drawFlatBox(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeBox(x,y,w,h,clr,FL_NO_BOX);
}

void CThemes::drawThinUpBox(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeBox(x,y,w,h,clr,FL_THIN_UP_FRAME);
}

void CThemes::drawUpBox(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeBox(x,y,w,h,clr,FL_UP_FRAME);
}

void CThemes::drawThinDownBox(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeBox(x,y,w,h,clr,FL_THIN_DOWN_FRAME);
}

void CThemes::drawDownBox(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeBox(x,y,w,h,clr,FL_DOWN_FRAME);
}

void CThemes::drawThemeFrame(int x,int y,int w,int h,Fl_Boxtype frameType) {
    CFrame* frame = m_frames.find(frameType);
    if (frame)
        frame->drawResized(x,y,w,h,false);
    else
        fl_draw_box(frameType,x,y,w,h,FL_BLACK);
}

void CThemes::drawThinUpFrame(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeFrame(x,y,w,h,FL_THIN_UP_FRAME);
}

void CThemes::drawUpFrame(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeFrame(x,y,w,h,FL_UP_FRAME);
}

void CThemes::drawThinDownFrame(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeFrame(x,y,w,h,FL_THIN_DOWN_FRAME);
}

void CThemes::drawDownFrame(int x,int y,int w,int h,Fl_Color clr) {
    drawThemeFrame(x,y,w,h,FL_DOWN_FRAME);
}

bool CThemes::sizeButton(CThemeButtonSize sz,int& w,int& h) {
    CButtonImage *buttons = sizeToButtonImages(sz);
    CPngImage *image = buttons[0].m_colorImage;
    if (!image)
        return false;
    if (sz == THM_BUTTON_COMBO) {
        w = image->w();
    } else {
        if (w < image->h() + CThemes::m_buttonFocusRadius)
            w = image->h() + CThemes::m_buttonFocusRadius;
    }
    h = image->h();
    return true;
}

bool CThemes::drawButton(CThemeButtonSize sz,int x,int y,int& w,int& h,bool highlited,bool pressed,bool defaultButton) {
    int ndx = 0;
    if (defaultButton) {
        x -= 2;
        y -= 2;
        ndx += 2;
    }

    if (pressed)
        ndx++;

    CPngImage *image;
    CButtonImage *buttons = sizeToButtonImages(sz);
    if (highlited)
        image = buttons[ndx].m_colorImage;
    else
        image = buttons[ndx].m_grayImage;

    if (!image || image->w() < 10) {
        return false;
    }

    h = image->h();

    if (m_flatButtons && !highlited) {
        if (defaultButton) {
            fl_draw_box(FL_DOWN_BOX,x,y,w,h,FL_BACKGROUND_COLOR);
        } else {
            fl_draw_box(FL_FLAT_BOX,x,y,w,h,FL_BACKGROUND_COLOR);
        }
    } else
        image->drawResized(x,y,w,h,10,CPngImage::PDM_STRETCH,true);

    return true;
}

bool CThemes::drawCheckOrRadioButton(CButtonImage* buttonImages,int x,int y,bool checked,bool highlited) {
    CPngImage *image;
    int ndx = 0;
    if (checked)
        ndx = 1;
    if (highlited)
        image = buttonImages[ndx].m_colorImage;
    else
        image = buttonImages[ndx].m_grayImage;

    if (!image || image->w() < 10)
        return false;

    image->draw(x,y);

    return true;
}

bool CThemes::drawCheckButton(int x,int y,bool checked,bool highlited) {
    return drawCheckOrRadioButton(m_checkButtons,x,y,checked,highlited);
}

bool CThemes::drawRadioButton(int x,int y,bool checked,bool highlited) {
    return drawCheckOrRadioButton(m_radioButtons,x,y,checked,highlited);
}

bool CThemes::sizeProgressBar(int& w,int& h) {
    CPngImage *image = m_progressBar[0];
    if (image && image->data()) {
        w = image->w();
        h = image->h();
        return true;
    } else {
        w = h = 0;
        return false;
    }
}

bool CThemes::drawProgressBar(int x,int y,int w,float percent) {
    if (percent > 100)
        percent = 100;
    if (percent < 0)
        percent = 0;
    if (m_progressBar[0] && m_progressBar[0]->data()) {
        int frameHeight = m_progressBar[0]->h();
        int frameWidth = m_progressBar[0]->w();
        int barHeight = m_progressBar[1]->h();
        int barWidth = m_progressBar[1]->w();
        int xspace = (frameWidth - barWidth) / 2;
        int yspace = (frameHeight - barHeight) / 2;
        int psize  = int((w - xspace * 2) * percent / 100 + 0.5);
        m_progressBar[0]->drawResized(x,y,w,frameHeight,10,CPngImage::PDM_STRETCH,true);
        m_progressBar[1]->drawResized(x+xspace,y+yspace,psize,barHeight,8,CPngImage::PDM_STRETCH,true);
        return true;
    } else {
        return false;
    }
}

bool CThemes::sizeScrollBar(int& w,int& h) {
    bool vertical = w < h;
    CScrollBarImages *images = &m_scrollBar[THM_SCROLLBAR_HORIZONTAL];
    if (vertical)
        images = &m_scrollBar[THM_SCROLLBAR_VERTICAL];
    CPngImage* normalImage = images->m_images[CScrollBarImages::NORMAL];
    if (normalImage) {
        if (vertical) {
            w = normalImage->w();
            if (h < images->buttonSize() * 2 + 10)
                h = images->buttonSize() * 2 + 10;
        } else {
            h = normalImage->h();
            if (w < images->buttonSize() * 2 + 10)
                w= images->buttonSize() * 2 + 10;
        }
        return true;
    }
    if (vertical) {
        w = 16;
        if (h < 44)
            h = 44;
    } else {
        h = 16;
        if (w < 44)
            w = 44;
    }
    return false;
}

bool CThemes::drawScrollBar(int x,int y,int w,int h,float minimum,float maximum,float value,float sliderSize,int activeZone,bool active) {
    bool vertical = w < h;

    // Selecting the group of images
    CScrollBarImages *images = &m_scrollBar[THM_SCROLLBAR_HORIZONTAL];
    if (vertical)
        images = &m_scrollBar[THM_SCROLLBAR_VERTICAL];

    // Painting the background
    int bsize = images->buttonSize();
    if (!bsize)
        bsize = 14;
    CPngImage* normalImage = (*images)[CScrollBarImages::NORMAL];
    CPngImage* activeImage = (*images)[CScrollBarImages::ACTIVE];
    if (normalImage && activeImage) {
        normalImage->drawResized(x,y,w,h,bsize,CPngImage::PDM_STRETCH,true);
    } else {
        fl_draw_box(FL_THIN_DOWN_BOX,x,y,w,h,FL_DARK3);
    }

    // Painting the buttons
    CPngImage *firstButtonSrc = normalImage;
    CPngImage *secondButtonSrc = normalImage;
    switch (activeZone) {
    case 1:
        firstButtonSrc = activeImage;
        break;
    case 2:
        secondButtonSrc = activeImage;
        break;
    }

    if (normalImage) {
        // Images available
        if (vertical) {
            firstButtonSrc->draw(x,y,normalImage->w(),bsize,0,0);
            secondButtonSrc->draw(x,y+h-bsize,normalImage->w(),bsize,0,normalImage->h()-bsize-1);
        } else {
            firstButtonSrc->draw(x,y,bsize,normalImage->h(),0,0);
            secondButtonSrc->draw(x+w-bsize,y,bsize,normalImage->h(),normalImage->w()-bsize-1,0);
        }
    } else {
        // Images not available
        if (vertical) {
            fl_draw_box(activeZone==1?FL_FLAT_BOX:FL_THIN_UP_BOX,x+1,y+1,w-2,bsize,FL_BACKGROUND_COLOR);
            fl_draw_box(activeZone==2?FL_FLAT_BOX:FL_THIN_UP_BOX,x+1,y+h-bsize-1,w-2,bsize,FL_BACKGROUND_COLOR);
            if (active)
                fl_color(FL_BLACK);
            else
                fl_color(fl_inactive(FL_BLACK));
            int w1 = (w-4)/3;
            if (w1 < 1)
                w1 = 1;
            int x1 = x+(w-2*w1-1)/2;
            int yy1 = y+(w-w1-1)/2;
            fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
            yy1 += h-w;
            fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
        } else {
            fl_draw_box(activeZone==1?FL_FLAT_BOX:FL_THIN_UP_BOX,x+1,y+1,bsize,h-2,FL_BACKGROUND_COLOR);
            fl_draw_box(activeZone==2?FL_FLAT_BOX:FL_THIN_UP_BOX,x+w-bsize-1,y+1,bsize,h-2,FL_BACKGROUND_COLOR);
            if (active)
                fl_color(FL_BLACK);
            else
                fl_color(fl_inactive(FL_BLACK));
            int w1 = (h-4)/3;
            if (w1 < 1)
                w1 = 1;
            int x1 = x+(h-w1-1)/2;
            int yy1 = y+(h-2*w1-1)/2;
            fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1, yy1);
            x1 += (w-h);
            fl_polygon(x1, yy1, x1, yy1+2*w1, x1+w1, yy1+w1);
        }
    }

    // Draw the slider
    double val;
    if (minimum == maximum)
        val = 0.5;
    else {
        val = (value-minimum)/(maximum-minimum);
        if (val > 1.0)
            val = 1.0;
        else if (val < 0.0)
            val = 0.0;
    }

    int ww = (vertical ? h : w) - bsize * 2;
    int S = int(sliderSize * ww+.5);
    if (S < 10)
        S = 10;
    int xx = int(val*(ww-S)+.5) + bsize;

    int xsl, ysl, wsl, hsl;
    CPngImage *sliderImage = (*images)[CScrollBarImages::SLIDER];
    CPngImage *markImage = (*images)[CScrollBarImages::MARK];
    if (vertical) {
        ysl = y+xx;
        hsl = S;
        wsl = w;
        if (sliderImage)
            wsl = sliderImage->w();
        xsl = x + (w + 1 - wsl) / 2;
        if (hsl > 0 && sliderImage) {
            sliderImage->drawResized(xsl,ysl,wsl,hsl,bsize,CPngImage::PDM_STRETCH,true);
            if (markImage && markImage->h() <= hsl + bsize*3/2)
                markImage->draw(xsl+(wsl-markImage->w())/2,ysl+(hsl-markImage->h())/2);
        }
    } else {
        xsl = x+xx;
        wsl = S;
        hsl = h;
        if (sliderImage)
            hsl = sliderImage->h();
        ysl = y + (h + 1 - hsl) / 2;
        if (wsl > 0 && sliderImage) {
            sliderImage->drawResized(xsl,ysl,wsl,hsl,bsize,CPngImage::PDM_STRETCH,true);
            if (markImage && markImage->w() <= wsl + bsize*3/2)
                markImage->draw(xsl+(wsl-markImage->w())/2,ysl+(hsl-markImage->h())/2);
        }
    }
    if (!normalImage && wsl>0 && hsl>0) {
        fl_draw_box(FL_THIN_UP_BOX,xsl+1,ysl+1,wsl-2,hsl-2,FL_BACKGROUND_COLOR);
    }

    return true;
}

CStrings CThemes::availableThemes() {
    CStrings  themes;
    themes.push_back("Default");
    const CStrings& dirs = searchDirectories();
    for (unsigned i = 0; i < dirs.size(); i++) {
        try {
            CDirectoryDS   dir;
            dir.directory(dirs[i]);

            dir.showPolicy(DDS_HIDE_DIRECTORIES|DDS_HIDE_DOT_FILES);
            dir.pattern("*.tar");
            dir.open();
            while (!dir.eof()) {
                std::string fileName = dir["Name"];
                size_t pos = fileName.rfind(".tar");
                if (pos != string::npos) {
                    string themeName = fileName.substr(0,pos);
                    if (themes.indexOf(themeName) < 0)
                        themes.push_back(themeName);
                }
                dir.next();
            }
            dir.close();
        } catch (...) {
            continue;
        }
    }
    return themes;
}

bool CThemes::drawTab(int x,int y,int w,int h,bool active) {
    CTabImage* tabImage;
    if (active)
        tabImage = m_tabImages.tabImage("active");
    else
        tabImage = m_tabImages.tabImage("inactive");
    if (!tabImage)
        return false;
    tabImage->draw(x,y,w,h);
    return true;
}

bool CThemes::drawTabFrame(int x,int y,int w,int h) {
    return false;
}
