/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBase64.cpp  -  description
                             -------------------
    begin                : July 19 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CBase64.h>

using namespace std;
using namespace sptk;

// Following structures are originally from mutt
// Best regards to mutt developers!
static int Index_64[128] = {
                               -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
                               -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
                               -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, 63,-1,-1,-1,
                               52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1,-1,-1,-1,
                               -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
                               15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
                               -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
                               41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
                           };

static char B64Chars[64] = {
                               'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
                               'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
                               'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
                               't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7',
                               '8', '9', '+', '/'
                           };

#define base64val(c) Index_64[(uint32_t)(c)]
#define base64chars(c) B64Chars[(uint32_t)(c & 0x3F)]

void CBase64::encode(CBuffer& bufDest, const CBuffer& bufSource) {
    char   c;
    char*  current = bufSource.data();
    uint32_t len = bufSource.bytes();

    while (len >= 3) {
        c = base64chars((current[0] & 0xFC) >> 2);
        bufDest.append(c);

        c = base64chars(((current[0] & 0x03) << 4) | ((current[1] & 0xF0) >> 4));
        bufDest.append(c);

        c = base64chars(((current[1] & 0x0F) << 2) | ((current[2] & 0xC0) >> 6));
        bufDest.append(c);

        c = base64chars(current[2] & 0x3F);
        bufDest.append(c);

        len     -= 3;
        current += 3;   /* move pointer 3 characters forward */
    }

    /// Now we should clean up remainder
    if (len > 0) {
        c = base64chars(current[0] >> 2);
        bufDest.append(&c, 1);
        if (len > 1) {
            c = base64chars(((current[0] & 0x03) << 4) | (current[1] >> 4));
            bufDest.append(&c, 1);
            c = base64chars((current[1] & 0x0f) << 2);
            bufDest.append(&c, 1);
            c = '=';
            bufDest.append(&c, 1);
        } else {
            c = base64chars((current[0] & 0x03) << 4);
            bufDest.append(&c, 1);
            c = '=';
            bufDest.append(&c, 1);
            c = '=';
            bufDest.append(&c, 1);
        }
    }
}

void CBase64::encode(string& strDest, const CBuffer& bufSource) {
    CBuffer bufOut;
    encode(bufOut, bufSource);

    strDest = bufOut.data();
}

static int internal_decode(CBuffer &bufDest, const unsigned char *src, uint32_t src_len) {
    const unsigned char *current = src;
    unsigned char c;
    int ch, j=0;

    if (!src_len)
        return 0;

    if ((src_len % 4) != 0)
        throw CException("Can't decode Base64 data - Source buffer length must be dividable by 4");

    bufDest.reset();

    for (unsigned i=0; i < src_len; i++) {
        ch = current[i];

        if (ch == '=')
            break;
        if (ch == ' ')
            ch = '+';
        ch = Index_64[ch];
        if (ch < 0)
            continue;

        switch (i % 4) {
        case 0:
            c = (unsigned char)((ch << 2) & 0xFF);
            bufDest.append((char *)&c, 1);
            break;
        case 1:
            bufDest.data()[j] |= ((ch >> 4) & 0xFF);
            j++;
            if (current[i+1] != '=') {
                c = (unsigned char)((ch << 4) & 0xFF);
                bufDest.append((char *)&c, 1);
            } /* if */
            break;
        case 2:
            bufDest.data()[j] |= ((ch >> 2) & 0x0f);
            j++;
            if (current[i+1] != '=') {
                c = (unsigned char)((ch << 6) & 0xFF);
                bufDest.append((char *)&c, 1);
            }
            break;
        case 3:
            bufDest.data()[j] |= ch;
            j++;
            break;
        }
    }
    return j;
}

int CBase64::decode(CBuffer &bufDest, const CBuffer& bufSource) throw(CException) {
    return internal_decode(bufDest, (const unsigned char *)bufSource.data(), bufSource.bytes());
}

int CBase64::decode(CBuffer &bufDest, const string& strSource) throw(CException) {
    return internal_decode(bufDest,(const unsigned char *)strSource.c_str(),(uint32_t)strSource.length());
}
