/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CBaseLog.cpp  -  description
                             -------------------
    begin                : Mon Jan 30 2006
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
 
    This module creation was sponsored by Total Knowledge (http://www.total-knowledge.com).
    Author thanks the developers of CPPSERV project (http://www.total-knowledge.com/progs/cppserv)
    for defining the requirements for this class.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <sptk3/CBaseLog.h>

using namespace std;
using namespace sptk;

const CLogPriority
sptk::CLP_DEBUG(LOG_DEBUG,"debug"),
sptk::CLP_INFO(LOG_INFO,"info"),
sptk::CLP_NOTICE(LOG_NOTICE,"notice"),
sptk::CLP_WARNING(LOG_WARNING,"warning"),
sptk::CLP_ERROR(LOG_ERR,"err"),
sptk::CLP_CRITICAL(LOG_CRIT,"crit"),
sptk::CLP_ALERT(LOG_ALERT,"alert"),
sptk::CLP_PANIC(LOG_EMERG,"panic");
//==========================================================================================
CLogStreamBuf::CLogStreamBuf() {
    m_parent = 0;
    m_bytes = 0;
    m_size = 1024;
    m_buffer = (char *)malloc(m_size);
    m_priority = &CLP_NOTICE;
    m_date = CDateTime::Now();
}

streambuf::int_type CLogStreamBuf::overflow(streambuf::int_type c) {
    bool bufferOverflow = m_bytes > m_size - 2;
    bool lineBreak = c <= 13;

    if (lineBreak || bufferOverflow) {
        if (m_bytes) {
            m_buffer[m_bytes] = 0;
            if (m_parent && m_priority->id() <= m_parent->minPriority().id())
                m_parent->saveMessage(m_date,m_buffer,m_bytes,m_priority);
            if (!bufferOverflow) {
                m_priority = &m_parent->defaultPriority();
                m_date = CDateTime::Now();
            }
            m_bytes = 0;
        }
    }
    if (!lineBreak) {
        if (m_bytes == 0)
            m_date = CDateTime::Now();
        m_buffer[m_bytes] = (char) c;
        m_bytes++;
    }
    return traits_type::not_eof(c);
}
//==========================================================================================
CBaseLog& sptk::operator << (CBaseLog &stream, const CLogPriority &priority) {
    stream.priority(priority);
    return stream;
}

void CBaseLog::option(CLogOption option,bool flag) {
    if (flag)
        m_options |= option;
    else
        m_options &= ~option;
}
//==========================================================================================
bool CFunctionLogger::m_quietMode;

CFunctionLogger::CFunctionLogger(CBaseLog& log,std::string functionName)
        : m_log(log), m_functionName(functionName) {
    m_log << string(m_log.m_indent,' ') << m_functionName << " started\n";
    m_log.m_indent += 2;
    m_started = CDateTime::Now();
}

CFunctionLogger::~CFunctionLogger() {
    m_log.m_indent -= 2;
    CDateTime ended = CDateTime::Now();
    unsigned seconds = unsigned((ended - m_started) * 3600 * 24);
    unsigned minutes = seconds / 60;
    seconds %= 60;

    m_log << string(m_log.m_indent,' ')
    << m_functionName << " completed (elapsed "
    << minutes << ":" << (seconds <= 9 ? "0" : "") << seconds << ")\n";
}
