/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          cfields.cpp  -  description
                             -------------------
    begin                : Wed Dec 15 1999
    copyright            : (C) 1999-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CField.h>
#include <sptk3/CException.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

using namespace std;
using namespace sptk;

CField::CField(const char *name) {
   m_name = name;
   displayName = name;
   width = -1;
   flags = 4; // FL_ALIGN_LEFT
   dataSize(0);
   visible = true;
   precision = 3; // default precision, only affects floating point fields
}

string CField::asString() const throw(CException) {
   char print_buffer[32];
   if (m_null) return "";
   switch (dataType()) {
      case VAR_BOOL:
         if (m_data.intData)
	    return "true";
	 else
	    return "false";
      case VAR_INT:
         sprintf(print_buffer,"%i",m_data.intData);
         return string(print_buffer);
      case VAR_INT64:
#if SIZEOF_LONG == 8
         sprintf(print_buffer,"%li",m_data.int64Data);
#else
         sprintf(print_buffer,"%lli",m_data.int64Data);
#endif
         return string(print_buffer);
      case VAR_FLOAT:
         {
            char formatString[10];
            sprintf(formatString,"%%0.%if",precision);
            sprintf(print_buffer,formatString,m_data.floatData);
            return string(print_buffer);
         }
      case VAR_STRING:
      case VAR_TEXT:
      case VAR_BUFFER:     
         if (!m_data.buffer.data)
            return "";
         return m_data.buffer.data;
      case VAR_DATE:       return CDateTime(m_data.floatData).dateString();
      case VAR_DATE_TIME:   
         {
            CDateTime dt(m_data.floatData);
            return dt.dateString() + " " + dt.timeString();
         }
      case VAR_IMAGE_PTR:
         sprintf(print_buffer,"%p",(char *)m_data.imagePtr);
         return string(print_buffer);
      case VAR_IMAGE_NDX:
         sprintf(print_buffer,"%i",m_data.imageNdx);
         return string(print_buffer);
      default:             
         throw CException("Can't convert field for that type");
   }
}

void CField::toXML(CXmlNode& node,bool compactXmlMode) const {
   string value = asString();
   if (!value.empty()) {
      CXmlElement* element = 0;
      if (dataType() == VAR_TEXT) {
         element = new CXmlElement(node,fieldName());
         new CXmlCDataSection(*element,value);
      } else {
    	 if (compactXmlMode) {
            node.setAttribute(fieldName(),value);
         } else {
	        element = new CXmlElement(node,"field");
	        element->value(value);
         }
      }
      if (!compactXmlMode) {
         element->setAttribute("name",fieldName());
         element->setAttribute("type",CVariant::typeName(dataType()));
          element->setAttribute("size",int2string((uint32_t)dataSize()));
      }
   }
}
