/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFileLog.cpp  -  description
                             -------------------
    begin                : Tue Jan 31 2006
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
 
    This module creation was sponsored by Total Knowledge (http://www.total-knowledge.com).
    Author thanks the developers of CPPSERV project (http://www.total-knowledge.com/progs/cppserv)
    for defining the requirements for this class.
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CFileLog.h>

using namespace std;
using namespace sptk;

void CFileLog::saveMessage(CDateTime date,const char *message,uint32_t sz,const CLogPriority *priority) throw(CException) {
    if (m_options&CLO_ENABLE) {
        if (!m_fileStream.is_open()) {
            m_fileStream.open(m_fileName.c_str(),ofstream::out|ofstream::app);
            if (!m_fileStream.is_open())
                throw CException("Can't append or create log file '"+m_fileName+"'",__FILE__,__LINE__);
        }

        if (m_options & CLO_DATE)
            m_fileStream << date.dateString() << " ";

        if (m_options & CLO_TIME)
            m_fileStream << date.timeString(true) << " ";

        if (m_options & CLO_PRIORITY)
            m_fileStream << "[" << priority->name() << "] ";

        m_fileStream << message << endl;
    }
    
    if (m_options & CLO_STDOUT) {
        if (m_options & CLO_DATE)
            cout << date.dateString() << " ";

        if (m_options & CLO_TIME)
            cout << date.timeString(true) << " ";

        if (m_options & CLO_PRIORITY)
            cout << "[" << priority->name() << "] ";

        cout << message << endl;
    }

    if (m_fileStream.bad())
        throw CException("Can't write to log file '"+m_fileName+"'",__FILE__,__LINE__);
}

CFileLog::~CFileLog() {
    m_buffer.flush();
    if (m_fileStream.is_open())
        m_fileStream.close();
}

void CFileLog::reset() throw(CException) {
    if (m_fileStream.is_open())
        m_fileStream.close();
	if (m_fileName.empty())
        throw CException("File name isn't defined",__FILE__,__LINE__);
    m_fileStream.open(m_fileName.c_str(),ofstream::out|ofstream::trunc);
    if (!m_fileStream.is_open())
        throw CException("Can't open log file '"+m_fileName+"'",__FILE__,__LINE__);
}
