/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CFtpDS.h - description
                             -------------------
    begin                : Mar 19 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CFtpDS.h>
#include <sptk3/CException.h>
#include <sptk3/CSmallPixmapIDs.h>

#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#ifndef FL_ALIGN_RIGHT
#define FL_ALIGN_RIGHT 8
#endif

using namespace std;
using namespace sptk;

static char *next_dir_item(char *p,char **result) {
   char *start = p;
   for (; *start == ' '; start++);
   *result = start;
   start = (char *) strchr(start,' ');
   *start = 0;
   return start + 1;
}

static const CStrings
   month_names("Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Nov|Dec","|");

CFieldList *parse_file_info_string(string& file_info) {
   char *ptr = (char *) file_info.c_str();

   char *permissions = 0L;
   char *refcount = 0L;
   char *user_name = 0L;
   char *group_name = 0L;
   char *size = 0L;
   char *month = 0L, *day = 0L, *year = 0L, *time = 0L;
   char *file_name = 0L;
   bool  is_directory = false;
   bool  is_executable = false;

   CDateTime dt;
   CSmallPixmapType pixmapType = SXPM_DOCUMENT; 

   if (isdigit(*ptr)) {
      // MS Dos style
      char *dtime = 0L;

      ptr = next_dir_item(ptr,&time);     
      ptr = next_dir_item(ptr, &dtime);

      ptr = next_dir_item(ptr,&size);

      if (strstr(size,"DIR"))
         is_directory = true;
      time[2] = 0;
      time[5] = 0;

      int month   = strtol(time, NULL, 10);
      int day     = strtol(time+3, NULL, 10);
      int year = strtol(time+6, NULL, 10);

      bool pm = false;
      if(strstr(dtime, "PM")) pm = true;
      dtime[2] = 0;
      dtime[5] = 0;
      int hour = strtol(dtime, NULL, 10);
      int min     = strtol(dtime+3, NULL, 10);     
      if(pm) hour+=12;

      if(year<50) year+=2000;
      else year+=1900;

      CDateTime dosDate(year, (short)month, (short)day, (short)hour, (short)min);
      dt = dosDate;
   } else {
      ptr = next_dir_item(ptr,&permissions); 
        // Unix style
      if (permissions[0] == 'd')
         is_directory = true;
      else if (strchr(permissions,'x'))
         is_executable = true;
      ptr = next_dir_item(ptr,&refcount);
      ptr = next_dir_item(ptr,&user_name);
      ptr = next_dir_item(ptr,&group_name);
      ptr = next_dir_item(ptr,&size);
      ptr = next_dir_item(ptr,&month);
      ptr = next_dir_item(ptr,&day);
      ptr = next_dir_item(ptr,&year);
      int m = month_names.indexOf(month) + 1;
      if (m >= 0) {
         int d = atoi(day);
         int y = atoi(year);
         CDateTime unixDate(y,m,d);
         dt = unixDate;
      }
   }

   if (is_directory)
      pixmapType = SXPM_FOLDER;
   else if (is_executable)
      pixmapType = SXPM_EXECUTABLE;

    // skip to the file name
   for (; *ptr == ' '; ptr++);
   file_name = ptr;

   CFieldList *df = new CFieldList(false);

   df->push_back("",false).setImageNdx(pixmapType);
   df->push_back("Name",false)     = file_name;
   df->push_back("Size",false)     = (uint32_t) atoi(size);
   df->push_back("Modified",false) = dt;

   (*df)[uint32_t(0)].width = 3;
   (*df)[uint32_t(1)].width = 30;
   (*df)[uint32_t(2)].width = 10;
   (*df)[uint32_t(2)].flags = FL_ALIGN_RIGHT;
   (*df)[uint32_t(3)].width = 16;

   return df;
}

// read the folder() and move item into the first entry
bool sptk::CFtpDS::open() {
   clear();

   // Connect to the server
   m_ftp.host(m_host,m_port);
   m_ftp.user(m_user);
   m_ftp.password(m_password);
   m_ftp.open();

   // Select the folder
   if (m_folder.length())
      m_ftp.cmd_cd(m_folder);

   CStrings dirlist;
   m_ftp.cmd_list(dirlist);
    //dirlist.print();

   if (dirlist.size()) {
      int cnt = (int) dirlist.size();
      if (m_callback)
         m_callback(cnt,0);
      for (int i = 0; i < cnt; i++) {
         CFieldList *df = parse_file_info_string(dirlist[i]);
         if (df)
            m_list.push_back(df);

         if (m_callback)
            m_callback(cnt,i);
      }
   }

   if (m_callback)
      m_callback(100,100);

   first();

   m_ftp.cmd_quit();
   m_ftp.close();

   m_eof = m_list.size() == 0;

   return !m_eof;
}
