/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CHttpConnect.cpp  -  description
                             -------------------
    begin                : July 19 2003
    copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/sptk.h>
#include <sptk3/CHttpConnect.h>
#include <sptk3/CHttpParams.h>

#include <iostream>

using namespace std;
using namespace sptk;

CHttpConnect::CHttpConnect() {
    m_port = 80;
    m_requestHeaders["Content-Type"] = "application/x-www-form-urlencoded";
    m_requestHeaders["Connection"] = "close";
}

CHttpConnect::~CHttpConnect() {
    close();
}

#define RSP_BLOCK_SIZE 16384

void CHttpConnect::getResponse() {
    m_readBuffer.reset();

    CStrings headers;
    CBuffer read_buffer(RSP_BLOCK_SIZE);

    int    bytes, contentLength = 0;
    string header;

    /// Reading HTTP headers
    for (;;) {
        readLine(header);
        if (header.empty())
            break;
        headers.push_back(header);
    }

    m_responseHeaders.clear();

    if (headers.empty())
        throw CException("Can't detect HTTP headers");

    /// Parsing HTTP headers
    if ( headers[0].find("HTTP/1.") != 0)
        throw CException("Broken HTTP version header");
    
    m_responseHeaders["HTTP version"] = headers[0];

    bool chunked = 0;
    for (unsigned i = 0; i < headers.size(); i++) {
        char* header = (char* ) headers[i].c_str();
        char* headerValue = strstr(header,": ");
        if (!headerValue)
            continue;
        *headerValue = 0;
        headerValue += 2;
        m_responseHeaders[header] = headerValue;
        if (strcmp(header,"Transfer-Encoding") == 0) {
            chunked = strcmp(headerValue,"chunked") == 0;
	    continue;
        }
        if (strcmp(header,"Content-Length") == 0) {
            contentLength = atoi(headerValue);
	    continue;
        }
    }

    if (!chunked) {
	int totalBytes = 0;
        for (;;) {
            bytes = (int) read(read_buffer,0);

            if (bytes <= 0) // No more data
                break;
            
            m_readBuffer.append(read_buffer);
            totalBytes += bytes;
	    
            if (contentLength && totalBytes >= contentLength)
		break;
        }
    } else {
        string chunkSizeStr;
        for (;;) {
            readLine(chunkSizeStr);
            if (chunkSizeStr.empty())
                readLine(chunkSizeStr);
            if (chunkSizeStr == "0")
                break;
            long chunkSize = strtol(chunkSizeStr.c_str(),0L,16);
            read_buffer.checkSize(chunkSize);
            bytes = (int) read(read_buffer.data(),chunkSize,0);
            if (bytes > 0) {
                read_buffer.data()[bytes] = 0;
                m_readBuffer.append(read_buffer);
            }
        }
    }
    close();
}

void CHttpConnect::sendCommand(string cmd) {
    cmd += "\r\n";
    if (!active())
        throw CException("Socket isn't open");
    write(cmd.c_str(),(uint32_t)cmd.length());
}

void CHttpConnect::cmd_get(string pageName,CHttpParams& postData) {
    m_readBuffer.checkSize(1024);

    CBuffer buffer;
    postData.encode(buffer);

    string parameters(buffer.data());

    string command = "GET http://" + m_host + "/" + pageName;
    if (parameters.length())
        command += "?" + parameters;
    command += " HTTP/1.0\r\n";
    //command += "Host: " + serverName + ":"+ string(m_port) + "\n";
    sendCommand(command);

    getResponse();
}

void CHttpConnect::cmd_post(string pageName,CHttpParams& postData) {
    CStrings headers;

    headers.push_back("POST " + pageName + " HTTP/1.1");
    headers.push_back("HOST: " + m_host);

    map<string,string>::iterator itor = m_requestHeaders.begin();
    map<string,string>::iterator iend = m_requestHeaders.end();
    for (; itor != iend; itor++)
        headers.push_back(itor->first + ": " + itor->second);

    CBuffer buffer;
    postData.encode(buffer);

    headers.push_back("Content-Length: " + int2string((uint32_t)buffer.bytes()));

    string command = headers.asString( "\r\n") + "\r\n\r\n";

    command += buffer.data();
    command += "\r\n";
    sendCommand(command);

    getResponse();
}
