/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CMemoryDS.cpp  -  description
                              -------------------
        begin                : Wed Feb 12 2003
        copyright            : (C) 2003-2007 by Alexey Parshin. All rights reserved.
        email                : alexeyp@gmail.com
***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#include <sptk3/CMemoryDS.h>
#include <stdlib.h>

using namespace std;
using namespace sptk;

#define checkDSopen(ds) if (!ds) throw CException("Dataset isn't open") 

      // access to the field by name
const CField& CMemoryDS::operator [] (const char *field_name) const {
   checkDSopen(m_current);
   return (*m_current)[field_name];
}

CField& CMemoryDS::operator [] (const char *field_name) {
   checkDSopen(m_current);
   return (*m_current)[field_name];
}

uint32_t CMemoryDS::recordCount() const {
   checkDSopen(m_current);
   return (uint32_t) m_list.size();
}

// how many fields do we have in the current record?
uint32_t CMemoryDS::fieldCount() const {
   if (!m_current) return 0;
   checkDSopen(m_current);
   return m_current->size();
}

// access to the field by number, 0..field.size()-1
const CField& CMemoryDS::operator [] (uint32_t index) const {
   checkDSopen(m_current);
   return (*m_current)[index];
}

CField& CMemoryDS::operator [] (uint32_t index) {
   checkDSopen(m_current);
   return (*m_current)[index];
}

// read this field data into external value
bool CMemoryDS::readField(const char *fname,CVariant& fvalue) {
   try {
      fvalue = (*this)[fname];
   }
   catch(CException &) { return false; }
   return true;
}

// write this field data from external value
bool CMemoryDS::writeField(const char *fname, const CVariant& fvalue) {
   try {
      (*this)[fname] = fvalue;      
   }
   catch(CException &) { return false; }
   return true;
}


bool CMemoryDS::close() {
   clear();
   return true;
}

bool CMemoryDS::first() {
   if (m_list.size()) {
      m_currentIndex = 0;
      m_current = (CFieldList *)m_list[m_currentIndex];
      m_eof = false;
      return true;
   }
   m_eof = true;
   return false;
}

bool CMemoryDS::last() {
   uint32_t cnt = (uint32_t) m_list.size();
   if (cnt) {
      m_currentIndex = cnt - 1;
      m_current = (CFieldList *)m_list[m_currentIndex];
      m_eof = false;
      return true;
   }
   m_eof = true;
   return false;
}

bool CMemoryDS::next() {
   uint32_t cnt = (uint32_t) m_list.size();
   if (m_currentIndex + 1 < cnt) {
      m_currentIndex++;
      m_current = (CFieldList *)m_list[m_currentIndex];
      m_eof = false;
      return true;
   }
   m_eof = true;
   return false;
}

bool CMemoryDS::prior() {
   if (m_currentIndex > 0) {
      m_currentIndex--;
      m_current = (CFieldList *)m_list[m_currentIndex];
      m_eof = false;
      return true;
   }
   m_eof = true;
   return false;
}

bool CMemoryDS::find(CVariant position) {
   uint32_t   cnt = (uint32_t) m_list.size();
   string     name;
   uint32_t   i;
   switch (position.dataType()) {
      case VAR_INT:  
         if (position.asInteger() < (int)cnt) {
            m_currentIndex = position;
            m_current = (CFieldList *)m_list[m_currentIndex];
            return true;
         }
         break;
      case VAR_STRING:
         name = position.asString();
         for (i = 0; i < cnt; i++) {
            CFieldList& entry = *(CFieldList *)m_list[i];
            if (entry["Name"].asString() == name) {
               m_currentIndex = i;
               m_current = (CFieldList *)m_list[m_currentIndex];
               return true;
            }
         }
         break;
      default:       
         break;
   }
   return false;
}

void CMemoryDS::clear() {
   uint32_t cnt = (uint32_t) m_list.size();
   for (uint32_t i = 0; i < cnt; i++) {
      CFieldList *df = (CFieldList *) m_list[i];
      delete df;
   }
   m_list.clear();
   m_current = 0L;
   m_currentIndex = 0;
   m_eof = true;
}
