/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CStrings.cpp  -  description
                             -------------------
    begin                : Thu August 11 2005
    copyright            : (C) 2005-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <fstream>
#include <sptk3/CStrings.h>
#include <sptk3/string_ext.h>

using namespace std;
using namespace sptk;

void CStrings::parseString(const char *src,const char *delimitter) {
    if (!src || !src[0])
        return;
    string buffer(src);
    clear();
    uint32_t dlen = (uint32_t) strlen(delimitter);
    if (!dlen)
        return;
    char *p = (char *)buffer.c_str();
    for (;;) {
        char *end = strstr(p,delimitter);
        if (end) {
            char sc = *end;
            *end = 0;
            push_back(p);
            *end = sc;
            p = end + dlen;
        } else {
            push_back(p);
            break;
        }
    }
}

string CStrings::asString(const char *separator) const {
    string result;
    const_iterator itor = begin();
    if (itor != end()) {
        result = *itor;
        itor++;
    }
    while (itor != end()) {
        result += separator + *itor;
        itor++;
    }
    return result;
}

int CStrings::indexOf(string s) const {
    const_iterator itor = find(begin(),end(),s.c_str());
    if (itor == end())
        return -1;
#ifdef __SUNPRO_CC
    int d;
    distance(begin(),itor,d);
    return d;
#else
    return (int) std::distance(begin(),itor);
#endif
}

void CStrings::saveToFile(string fileName) const throw(CException) {
#ifdef WIN32
    /// I'm using FILE IO here, since the ofstream implementation
    /// in Visual C++ works about 20 times slower
    FILE *f = fopen(fileName.c_str(),"w+t");
    if (!f)
        throw CException("Can't open file "+fileName+" for writing");
    char *buffer = new char[16384];
    setvbuf(f,buffer,_IONBF,16384);

    const_iterator itor = begin();
    for (; itor != end(); itor++) {
        const idstring& str = *itor;
        if (fputs(str.c_str(),f) == EOF) {
            delete [] buffer;
            throw CException("Can't write to file "+fileName);
        }
        if (fputs("\n",f) == EOF) {
            delete [] buffer;
            throw CException("Can't write to file "+fileName);
        }
    }
    fclose(f);
    delete [] buffer;
#else
    ofstream file;
    file.open(fileName.c_str());
    if (!file.is_open())
        throw CException("Can't open file "+fileName+" for reading");
    const_iterator itor = begin();
    for (; itor != end(); itor++) {
        const idstring& str = *itor;
        file << str << endl;
        if (!file.good())
            throw CException("Can't write to file "+fileName);
    }
    file.close();
#endif
}

void CStrings::loadFromFile(string fileName) throw(CException)  {
    string line;
    clear();
    ifstream infile;
    infile.open(fileName.c_str());
    if (!infile.is_open())
        throw CException("Can't open file "+fileName+" for reading");
    while (getline(infile,line,'\n'))
        push_back (line);
    infile.close();
}
