/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CUniqueInstance.cpp  -  description
                             -------------------
    begin                : Feb 6 2002
    copyright            : (C) 2002-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <sptk3/CException.h>
#include <sptk3/CUniqueInstance.h>

#ifndef _WIN32
#ifndef __USE_XOPEN_EXTENDED
#define __USE_XOPEN_EXTENDED
#endif
#include <unistd.h>
#endif

using namespace sptk;

// Constructor
CUniqueInstance::CUniqueInstance(std::string instanceName) {
    m_lockCreated = false;
    m_instanceName = instanceName;
#ifndef _WIN32

    m_fileName = "/tmp/" + m_instanceName + ".lock";
    if (read_pid() == 0)
        write_pid();
#else

    m_mutex = CreateMutex(NULL,true,m_instanceName.c_str());
    if (GetLastError()==0)
        m_lockCreated = true;
#endif
}

// Destructor
CUniqueInstance::~CUniqueInstance() {
    // Cleanup
    if (m_lockCreated) {
#ifndef _WIN32
        unlink(m_fileName.c_str());
#else

        CloseHandle(m_mutex);
#endif

    }
}

#ifndef _WIN32
// Get the existing process id (if any) from the file
int CUniqueInstance::read_pid() {
    char   buffer[32];

    // Try to read process id from the file
    FILE  *f = fopen(m_fileName.c_str(),"r+b");
    if (!f)
        return 0;
    fgets(buffer,32,f);
    fclose(f);

    // Is it a number?
    buffer[31] = 0;
    char *p = strchr(buffer,'\n');
    if (p)
        *p = 0;
    int pid = atoi(buffer);
    if (!pid)
        return 0;

    // Does the process with this id exist?
    int sid = getsid(pid);

    if (sid < 0) // No such process - stale lock file.
        return 0;

    return pid;
}

int CUniqueInstance::write_pid() {
    FILE  *f = fopen(m_fileName.c_str(),"w+b");
    if (!f)
        return 0;
    int pid = getpid();
    fprintf(f,"%i\n",pid);
    fclose(f);

    m_lockCreated = true;
    return pid;
}
#endif

bool CUniqueInstance::isUnique() {
    return m_lockCreated;
}
