/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CVariant.cpp  -  description
                             -------------------
    begin                : Tue Dec 14 1999
    copyright            : (C) 1999-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CVariant.h>
#include <sptk3/CField.h>
#include <sptk3/CException.h>
#include <sptk3/string_ext.h>

#include <map>
#include <math.h>

using namespace std;
using namespace sptk;

void CVariant::releaseBuffers() {
    if (dataType() & (VAR_STRING|VAR_BUFFER|VAR_TEXT)) {
        if (m_data.buffer.data) {
            free(m_data.buffer.data);
            m_data.buffer.data = NULL;
            m_data.buffer.size = 0;
        }
    }
}
//---------------------------------------------------------------------------
void CVariant::setBool(bool value) {
    releaseBuffers();
    dataType(VAR_BOOL);
    dataSize(sizeof(value));
    m_data.boolData = value;
    m_null = false;
}
//---------------------------------------------------------------------------
void CVariant::setInteger(int32_t value) {
    releaseBuffers();
    dataType(VAR_INT);
    dataSize(sizeof(value));
    m_data.intData = value;
    m_null = false;
}
//---------------------------------------------------------------------------
void CVariant::setInt64(int64_t value) {
    releaseBuffers();
    dataType(VAR_INT64);
    dataSize(sizeof(value));
    m_data.int64Data = value;
    m_null = false;
}
//---------------------------------------------------------------------------
void CVariant::setFloat(double value) {
    releaseBuffers();
    dataType(VAR_FLOAT);
    dataSize(sizeof(value));
    m_data.floatData = value;
    m_null = false;
}
//---------------------------------------------------------------------------
void CVariant::setMoney(double value) {
    releaseBuffers();
    dataType(VAR_MONEY);
    dataSize(sizeof(value));
    m_data.floatData = value;
    m_null = false;
}
//---------------------------------------------------------------------------
void CVariant::setString(const char * value,uint32_t maxlen) {
    m_null = false;
    if (dataType() == VAR_STRING && maxlen && m_data.buffer.size == maxlen+1) {
        if (value) {
            strncpy(m_data.buffer.data,value,maxlen);
            m_data.buffer.data[maxlen] = 0;
        } else {
            m_data.buffer.data[0] = 0;
            m_null = true;
        }
    } else {
        releaseBuffers();
        if (value) {
            if (maxlen) {
                dataSize(maxlen);
                m_data.buffer.size = maxlen + 1;
                m_data.buffer.data = (char *)malloc(m_data.buffer.size);
                strncpy(m_data.buffer.data,value,maxlen);
                m_data.buffer.data[maxlen] = 0;
            } else {
                dataSize((uint32_t)strlen(value));
                m_data.buffer.size = dataSize() + 1;
                m_data.buffer.data = strdup(value);
                m_null = false;
            }
        } else {
            m_data.buffer.data = NULL;
            dataSize(0);
            m_data.buffer.size = 0;
            m_null = true;
        }
    }
    dataType(VAR_STRING);
}
//---------------------------------------------------------------------------
void CVariant::setText(const char * value) {
    releaseBuffers();
    dataType(VAR_TEXT);
    if (value) {
        m_null = false;
        dataSize((uint32_t)strlen(value));
        m_data.buffer.size = dataSize() + 1;
        m_data.buffer.data = strdup(value);
    } else {
        m_null = true;
        m_data.buffer.data = NULL;
        m_data.buffer.size = 0;
        dataSize(0);
    }
}
//---------------------------------------------------------------------------
void CVariant::setText(const string& value) {
    releaseBuffers();
    dataType(VAR_TEXT);
    uint32_t vlen = (uint32_t)value.length();
    if (vlen) {
        m_null = false;
        dataSize(vlen);
        m_data.buffer.size = vlen + 1;
        m_data.buffer.data = strdup(value.c_str());
    } else {
        m_null = true;
        m_data.buffer.data = NULL;
        m_data.buffer.size = 0;
        dataSize(0);
    }
}
//---------------------------------------------------------------------------
void CVariant::setBuffer(const void * value,uint32_t sz) {
    releaseBuffers();
    dataType(VAR_BUFFER);
    if (value) {
        m_null = false;
        m_data.buffer.size = sz;
        dataSize(sz);
        m_data.buffer.data = (char *)malloc(sz);
        memcpy(m_data.buffer.data,value,sz);
    } else
        setNull();
}
//---------------------------------------------------------------------------
void CVariant::setBuffer(const string& value) {
    releaseBuffers();
    dataType(VAR_BUFFER);
    uint32_t vlen = (uint32_t) value.length();
    if (vlen) {
        uint32_t sz = vlen + 1;
        m_null = false;
        m_data.buffer.size = sz;
        dataSize(sz);
        m_data.buffer.data = (char *)malloc(sz);
        memcpy(m_data.buffer.data,value.c_str(),sz);
    } else
        setNull();
}
//---------------------------------------------------------------------------
void CVariant::setDateTime(CDateTime value) {
    m_null = false;
    releaseBuffers();
    dataType(VAR_DATE_TIME);
    dataSize(sizeof(value));
    m_data.floatData = value;
}
//---------------------------------------------------------------------------
void CVariant::setDate(CDateTime value) {
    m_null = false;
    releaseBuffers();
    dataType(VAR_DATE);
    dataSize(sizeof(value));
    m_data.floatData = (int)value;
}
//---------------------------------------------------------------------------
void CVariant::setImagePtr(const void *value) {
    m_null = false;
    releaseBuffers();
    dataType(VAR_IMAGE_PTR);
    m_data.buffer.size = 0;
    dataSize(sizeof(value));
    m_data.imagePtr = (void *) value;
}
//---------------------------------------------------------------------------
void CVariant::setImageNdx(uint32_t value) {
    m_null = false;
    releaseBuffers();
    dataType(VAR_IMAGE_NDX);
    m_data.buffer.size = 0;
    dataSize(sizeof(value));
    m_data.imageNdx = value;
}
//---------------------------------------------------------------------------
void CVariant::setData(const CVariant &C) {
    m_null = C.m_null;
    switch (C.dataType()) {
    case VAR_BOOL:
        setInteger(C.getBool());
        break;
    case VAR_INT:
        setInteger(C.getInteger());
        break;
    case VAR_INT64:
        setInt64(C.getInt64());
        break;
    case VAR_FLOAT:
        setFloat(C.getFloat());
        break;
    case VAR_MONEY:
        setFloat(C.getFloat());
        break;
    case VAR_STRING:
        setString(C.getString());
        break;
    case VAR_TEXT:
        setText(C.getText());
        break;
    case VAR_BUFFER:
        setBuffer(C.getBuffer(),C.dataSize());
        break;
    case VAR_DATE:
        setDate(C.getDateTime());
        break;
    case VAR_DATE_TIME:
        setDateTime(C.getDateTime());
        break;
    case VAR_IMAGE_PTR:
        setImagePtr(C.getImagePtr());
        break;
    case VAR_IMAGE_NDX:
        setImageNdx(C.getImageNdx());
        break;
    case VAR_NONE:
        break;
    }
}
//---------------------------------------------------------------------------
// convertors
int32_t CVariant::asInteger() const throw(CException) {
    if (m_null)
        return 0;
    switch (m_dataType) {
    case VAR_BOOL:
        return m_data.boolData;
    case VAR_INT:
        return m_data.intData;
    case VAR_INT64:
        return (int32_t) m_data.int64Data;
    case VAR_MONEY:
    case VAR_FLOAT:
        return (int32_t) m_data.floatData;
    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        return atol(m_data.buffer.data);
    case VAR_DATE:
    case VAR_DATE_TIME:
        return (long) m_data.floatData;
    default:
        throw CException("Can't convert field for that type");
    }
}

int64_t CVariant::asInt64() const throw(CException) {
    if (m_null)
        return 0;
    switch (m_dataType) {
        case VAR_BOOL:
            return m_data.boolData;
        case VAR_INT:
            return m_data.intData;
        case VAR_INT64:
            return m_data.int64Data;
        case VAR_MONEY:
        case VAR_FLOAT:
            return (long)m_data.floatData;
        case VAR_STRING:
        case VAR_TEXT:
        case VAR_BUFFER:
#ifdef _MSC_VER
            return _atoi64(m_data.buffer.data);
#else
            return atol(m_data.buffer.data);
#endif
        case VAR_DATE:
        case VAR_DATE_TIME:
            return int64_t(m_data.floatData);
	case VAR_IMAGE_PTR:
            return int64_t(m_data.imagePtr);
	case VAR_IMAGE_NDX:
            return int64_t(m_data.imageNdx);
        default:
            throw CException("Can't convert field for that type");
    }
}

bool CVariant::asBool() const throw(CException) {
    char ch;
    if (m_null)
        return true;
    switch (dataType()) {
    case VAR_BOOL:
        return m_data.boolData;
    case VAR_INT:
        return (m_data.intData>0);
    case VAR_INT64:
        return (m_data.int64Data>0);
    case VAR_MONEY:
    case VAR_FLOAT:
        return (m_data.floatData>0);
    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        ch = m_data.buffer.data[0];
        return (strchr("YyTt1",ch)!=0);
    case VAR_DATE:
    case VAR_DATE_TIME:
        return bool(m_data.floatData!=0);
    case VAR_IMAGE_PTR:
        return bool(m_data.imagePtr!=0);
    case VAR_IMAGE_NDX:
        return bool(m_data.imageNdx!=0);
    default:
        throw CException("Can't convert field for that type");
    }
}

double CVariant::asFloat() const throw(CException) {
    if (m_null)
        return 0;
    switch (dataType()) {
    case VAR_BOOL:
        return m_data.boolData;
    case VAR_INT:
        return m_data.intData;
    case VAR_INT64:
        return (double) m_data.int64Data;
    case VAR_MONEY:
    case VAR_FLOAT:
        return m_data.floatData;
    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        return strtod(m_data.buffer.data, 0);
    case VAR_DATE:
    case VAR_DATE_TIME:
        return m_data.floatData;
    default:
        throw CException("Can't convert field for that type");
    }
}

string CVariant::asString() const throw(CException) {
    if (m_null)
        return "";
    char print_buffer[64];
    switch (dataType()) {
    case VAR_NONE:
        return "";
    case VAR_BOOL:
        if (m_data.boolData)
            return "t";
        return "f";
    case VAR_INT:
        sprintf(print_buffer,"%i",m_data.intData);
        return string(print_buffer);
    case VAR_INT64:
#if SIZEOF_LONG == 8
        sprintf(print_buffer,"%li",m_data.int64Data);
#else
        sprintf(print_buffer,"%lli",m_data.int64Data);
#endif
        return string(print_buffer);
    case VAR_MONEY:
    case VAR_FLOAT: {
            const char *formatString = "%0.4f";
            if (floor(m_data.floatData) == m_data.floatData)
                formatString = "%0.0f";
            if (fabs(m_data.floatData) > 1e16)
                formatString = "%0.4e";
            sprintf(print_buffer,formatString,m_data.floatData);
            return string(print_buffer);
        }
    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        return m_data.buffer.data;
    case VAR_DATE:
        return CDateTime(m_data.floatData).dateString();
    case VAR_DATE_TIME:   {
            CDateTime dt(m_data.floatData);
            return dt.dateString() + " " + dt.timeString();
        }
    case VAR_IMAGE_PTR:
        sprintf(print_buffer,"%p",(char *)m_data.imagePtr);
        return string(print_buffer);
    case VAR_IMAGE_NDX:
        sprintf(print_buffer,"%i",m_data.imageNdx);
        return string(print_buffer);
    default:
        throw CException("Can't convert field for that type");
    }
}

CDateTime CVariant::asDate() const throw(CException) {
    if (m_null)
        return 0.0;
    switch (dataType()) {
    case VAR_BOOL:
        return double(0);
    case VAR_INT:
        return m_data.intData;
    case VAR_INT64:
        return (double) m_data.int64Data;
    case VAR_MONEY:
    case VAR_FLOAT:
        return m_data.floatData;

    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        return m_data.buffer.data;

    case VAR_DATE:
    case VAR_DATE_TIME:
        return int(m_data.floatData);

    default:
        throw CException("Can't convert field for that type");
    }
}

CDateTime CVariant::asDateTime() const throw(CException) {
    if (m_null)
        return 0.0;
    switch (dataType()) {
    case VAR_BOOL:
        return double(0);
    case VAR_INT:
        return m_data.intData;
    case VAR_INT64:
        return (double) m_data.int64Data;
    case VAR_MONEY:
    case VAR_FLOAT:
        return m_data.floatData;

    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        return m_data.buffer.data;

    case VAR_DATE:
    case VAR_DATE_TIME:
        return m_data.floatData;

    default:
        throw CException("Can't convert field for that type");
    }
}

void *CVariant::asImagePtr() const throw(CException) {
    if (m_null)
        return 0;
    switch (dataType()) {
    case VAR_IMAGE_PTR:
        return m_data.imagePtr;
    default:
        throw CException("Can't convert field for that type");
    }
}

void CVariant::setNull() {
    switch (dataType()) {
    default:
        m_data.int64Data = 0;
        break;

    case VAR_STRING:
    case VAR_TEXT:
    case VAR_BUFFER:
        if (m_data.buffer.data)
            m_data.buffer.data[0] = 0;
        break;
    }
    m_dataSize = 0;
    m_null = true;
}

string CVariant::typeName(CVariantType type) {
    switch (type) {
    default:
        return "undefined";
    case VAR_BOOL:
        return "bool";
    case VAR_INT:
        return "int";
    case VAR_INT64:
        return "int64";
    case VAR_FLOAT:
        return "double";
    case VAR_MONEY:
        return "money";
    case VAR_STRING:
        return "string";
    case VAR_TEXT:
        return "text";
    case VAR_BUFFER:
        return "blob";
    case VAR_DATE:
        return "date";
    case VAR_DATE_TIME:
        return "datetime";
    case VAR_IMAGE_PTR:
        return "imageptr";
    case VAR_IMAGE_NDX:
        return "imagendx";
    }
}

CVariantType CVariant::nameType(const char* name) {
    static std::map<string,CVariantType> nameToTypeMap;
    if (nameToTypeMap.empty()) {
        for (int type = VAR_INT; type <= VAR_BOOL; type*=2) {
            CVariantType vtype = (CVariantType) type;
            nameToTypeMap[typeName(vtype)] = vtype;
        }
    }
    if (!name || name[0] == 0)
        name = "string";
    std::map<string,CVariantType>::iterator itor = nameToTypeMap.find(lowerCase(name));
    if (itor == nameToTypeMap.end())
        throw CException("Type name "+string(name)+" isn't recognized",__FILE__,__LINE__);
    return itor->second;
}

void CVariant::load(const CXmlNode& node) {
    const string& ntype = node.getAttribute("type");
    unsigned type = nameType(ntype.c_str());
    switch (type) {
    case VAR_BOOL:
    case VAR_INT:
    case VAR_INT64:
    case VAR_FLOAT:
    case VAR_MONEY:
    case VAR_STRING:
    case VAR_DATE:
    case VAR_DATE_TIME:
    case VAR_IMAGE_NDX:
        *this = node.text();
        break;
    case VAR_TEXT:
    case VAR_BUFFER:
        *this = node.text();
        break;
    default:
    case VAR_IMAGE_PTR:
        break;
    }
}

void CVariant::save(CXmlNode& node) const {
    string stringValue(asString());
    node.setAttribute("type",typeName(dataType()));

    if (!stringValue.empty()) {
        switch (dataType()) {
        case VAR_BOOL:
        case VAR_INT:
        case VAR_INT64:
        case VAR_FLOAT:
        case VAR_MONEY:
        case VAR_STRING:
        case VAR_DATE:
        case VAR_DATE_TIME:
        case VAR_IMAGE_NDX:
            new CXmlText(node,stringValue);
            break;
        case VAR_TEXT:
        case VAR_BUFFER:
            new CXmlCDataSection(node,asString());
            break;
        default:
        case VAR_IMAGE_PTR:
            break;
        }
    }
}
