/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CWaiter.cpp  -  description
                             -------------------
    begin                : Fri Jul 20 2001
    copyright            : (C) 2001-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <sptk3/CWaiter.h>
#include <sptk3/CException.h>
#include <errno.h>
#include <string.h>

using namespace sptk;

CWaiter::CWaiter() {
#ifndef _WIN32
    pthread_mutex_init(&m_waiter,NULL);
    pthread_cond_init(&m_condition,NULL);
#else
    m_waiter = CreateEvent(NULL, FALSE, TRUE, NULL);
#endif
}

CWaiter::~CWaiter() {
#ifndef _WIN32
    pthread_mutex_destroy(&m_waiter);
    pthread_cond_destroy(&m_condition);
#else
    CloseHandle(m_waiter);
#endif
}

void CWaiter::lock() {
#ifndef _WIN32
    pthread_mutex_lock(&m_waiter);
#else
    WaitForSingleObject(m_waiter,INFINITE);
#endif
}

bool CWaiter::try_lock() {
#ifndef _WIN32
    #define WAIT_TIMEOUT EBUSY
    int res = pthread_mutex_trylock(&m_waiter);
#else
    DWORD res = WaitForSingleObject(m_waiter,0);
#endif
    switch (res) {
    case 0:
	    return true; // Got lock
    case WAIT_TIMEOUT:
	    return false; // Locked by someone else
    default:
	    throw CException("Unable to aquire lock: ");
    }
    return false;
}

void CWaiter::unlock() {
#ifndef _WIN32
    pthread_mutex_unlock(&m_waiter);
#else
    SetEvent(m_waiter);
#endif
}

int CWaiter::waitForSignal(int timeout) {
#ifndef _WIN32
    if (timeout > 0) {
        int   secs  = timeout / 1000;
        int   msecs = timeout % 1000;
        struct timespec   abstime = { time(NULL) + secs, msecs * 1000L };
        int rc = pthread_cond_timedwait(&m_condition,&m_waiter,&abstime);
	if(rc != 0)
	    return -1;
    } else 
        pthread_cond_wait(&m_condition,&m_waiter);
    
#else
    WaitForSingleObject(m_waiter,timeout);
#endif

    return 0;
}

void CWaiter::sendSignalNoLock() {
#ifndef _WIN32
    pthread_cond_signal(&m_condition);
#else
    SetEvent(m_waiter);
#endif
}

void CWaiter::sendSignal() {
    lock();
    sendSignalNoLock();
    unlock();
}

void CWaiter::broadcastSignalNoLock() {
#ifndef _WIN32
    pthread_cond_broadcast(&m_condition);
#endif
}

void CWaiter::broadcastSignal() {
    lock();
    broadcastSignalNoLock();
    unlock();
}
