/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          string_ext.cpp  -  description
                             -------------------
    begin                : Thu July 14 2005
    copyright            : (C) 2005-2007 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <fstream>
#include <iostream>
#include <errno.h>
#include <sptk3/sptk.h>
#include <sptk3/string_ext.h>

using namespace std;

namespace sptk {

string upperCase(const string& str) {
    uint32_t len = (uint32_t) str.length();
    string result;
    result.resize(len);
    for (uint32_t i = 0; i < len; i++) 
        result[i] = (char) toupper(str[i]);
    return result;
}

string lowerCase(const string& str) {
    uint32_t len = (uint32_t) str.length();
    string result;
    result.resize(len);
    for (uint32_t i = 0; i < len; i++) 
        result[i] = (char) tolower(str[i]);
    return result;
}

string trim(const string& str) {
    uint32_t len = (uint32_t) str.length();
    if (!len)
        return "";
    const unsigned char *s = (const unsigned char *)str.c_str();
    int i, startpos = 0, endpos = int(len - 1);
    bool found = false;
    for (i = endpos; i >= 0; i--) {
        if (s[i] > 32) {
            endpos = i;
            found = true;
            break;
        }
    }
    if (!found)
        return "";
    for (i = 0; i <= endpos; i++) {
        if (s[i] > 32) {
            startpos = i;
            break;
        }
    }
    return str.substr(startpos,endpos-startpos+1);
}

void join(string& dest, const vector<string> src, string separator) {
    dest = "";
    vector<string>::const_iterator itor = src.begin();
    for (; itor != src.end(); itor++) {
        dest += *itor + separator;
    }
}

void split(vector<string> dest, const string& src, string delimitter) {
    dest.clear();
    char *buffer = (char *)src.c_str();
    if (!strlen(buffer))
        return;
    uint32_t dlen = (uint32_t) delimitter.length();
    if (!dlen)
        return;
    char *p = buffer;
    for (;;) {
        char *end = strstr(p,delimitter.c_str());
        if (end) {
            //int len = end - p;
            char sc = *end;
            *end = 0;
            dest.push_back(p);
            *end = sc;
            p = end + dlen;
        } else {
            dest.push_back(p);
            break;
        }
    }
}

string int2string(int32_t value) {
    char buff[32];
    sprintf(buff,"%i",value);
    return buff;
}

string int2string(uint32_t value) {
    char buff[64];
    sprintf(buff,"%u",value);
    return buff;
}

string int2string(int64_t value) {
    char buff[128];
#if SIZEOF_LONG == 8
    sprintf(buff,"%li",value);
#else
    sprintf(buff,"%lli",value);
#endif
    return buff;
}

string int2string(uint64_t value) {
    char buff[128];
#if SIZEOF_LONG == 8
    sprintf(buff,"%lu",value);
#else
    sprintf(buff,"%llu",value);
#endif
    return buff;
}

int string2int(const string& str,int defaultValue) {
    char *endptr;
    errno = 0;
    int result = strtol(str.c_str(),&endptr,10);
    if (errno)
        return defaultValue;
    return result;
}

string capitalizeWords(const std::string& str) {
    string s(str);
    char *current = (char *)s.c_str();
    char *wordStart = NULL;
    if (*current) {
        for (;;) {
            if (isalnum(*current)) {
                if (!wordStart)
                    wordStart = current;
            } else {
                if (current - wordStart > 3) {
                    if (wordStart)
                        *wordStart = (char) toupper(*wordStart);
                    else
                        wordStart = current;
                    for (char *ptr = wordStart + 1; ptr < current; ptr++)
                        *ptr = (char) tolower(*ptr);
                }
                wordStart = NULL;
            }
            if (!*current)
                break;
            current++;
        }
    }
    return s;
}

string replaceAll(const string& src,const string& pattern,const string& replacement) {
    string str(src);

    if (pattern.empty())
        return src;
	
    size_t patternLength = pattern.length();
    size_t replacementLength = replacement.length();

    size_t i = str.find(pattern);

    while (i != string::npos) { // While not at the end of the string
        str.replace(i,patternLength,replacement);
        i = str.find(pattern, i + replacementLength);
    }
    return str;
}

void stringToStringVector(const string& src,vector<string> dest,string delimitter) {
    string buffer(src);
    dest.clear();
    if (!buffer[0])
        return;
    uint32_t dlen = (uint32_t) delimitter.length();
    if (!dlen)
        return;
    char *p = (char *)buffer.c_str();
    for (;;) {
        char *end = strstr(p,delimitter.c_str());
        if (end) {
            char sc = *end;
            *end = 0;
            dest.push_back(p);
            *end = sc;
            p = end + dlen;
        } else {
            dest.push_back(p);
            break;
        }
    }
}

}
