/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlDocType.cpp  -  description
                             -------------------
    begin                : Tue June 27 2006
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2006-2007 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk3/cxml>

using namespace std;
using namespace sptk;

CXmlDocType::CXmlDocType(const char *name, const char *public_id, const char *system_id) {
    m_name = name;
    if(public_id)
        m_public_id = public_id;
    if(system_id)
        m_system_id = system_id;
}

struct entity {
    const char *name;
    int replacement_len;
    const char *replacement;
};

typedef map<std::string,struct entity *> CEntityMap;

struct entity builtin_ent_xml[] = {
                                      { "amp", 1, "&"
                                      },
                                      { "lt",  1, "<" },
                                      { "gt",  1, ">" },
                                      { "apos",  1, "'" },
                                      { "quot",  1, "\"" },
                                      { NULL,  1, "\"" }
                                  };
const char xml_shortcut[] = "&<>'\"";

class CEntityCache {
    CEntityMap              m_hash;
    map<int,CEntityMap>     m_replacementMaps;
public:

    CEntityCache(struct entity entities[]) {
        struct entity *ent = entities;
        for(; ent->name; ent++) {
            m_hash[ent->name] = ent;
            m_replacementMaps[ent->replacement_len][ent->replacement] = ent;
        }
    }

    const struct entity *find(const std::string &ent) const {
            CEntityMap::const_iterator itor = m_hash.find(ent);
            if (itor != m_hash.end())
                return itor->second;
            return 0L;
        }

    const struct entity *encode(const char* str) const;
};

const struct entity *CEntityCache::encode(const char* str) const {
    map<int,CEntityMap>::const_iterator maps = m_replacementMaps.begin();
    for (; maps != m_replacementMaps.end(); maps++) {
        int len = maps->first;
        string fragment(str,len);
        const CEntityMap& replacements = maps->second;
        CEntityMap::const_iterator itor = replacements.find(fragment);
        if (itor != replacements.end())
            return itor->second;
    }
    return 0;
}

static const CEntityCache xml_entities(builtin_ent_xml);

void CXmlDocType::decodeEntities(const char* str, uint32_t sz, CBuffer& ret) {
    ret.bytes(0);

    const char* start = str;
    const char* ptr = str;
    while (*ptr) {
        const char* ent_start = strchr(ptr,'&');
        if (ent_start) {
            char* ent_end = (char*)strchr(ent_start+1,';');
            if (ent_end) {
                char ch = *ent_end;
                *ent_end = 0;
                uint32_t replacementLength = 0;
                const char* rep = getReplacement(ent_start+1,replacementLength);
                *ent_end = ch;
                if (rep) {
					uint32_t len = uint32_t(ent_start-start);
					if (len)
						ret.append(start,len);
                    ptr = ent_end+1;
                    start = ptr;
                    ret.append(rep,replacementLength);
                } else
                    ptr++;
            } else {
                ptr++;
            }
		} else {
			sz = (uint32_t) strlen(start);
            break;
		}
    }
    ret.append(start,sz);
}

bool CXmlDocType::encodeEntities(const char *str, CBuffer& ret) {
    entity* table = builtin_ent_xml;

    bool replaced = false;

    const char* ptr = str;
    bool found = false;
	CBuffer* src = &m_encodeBuffers[0];
	CBuffer* dst = &m_encodeBuffers[1];
	dst->bytes(0);
    while (*ptr) {
        const char* pos = strpbrk(ptr,xml_shortcut);
        if (pos) {
            uint32_t index = uint32_t(strchr(xml_shortcut,*pos) - xml_shortcut);
            entity* ent = table + index;
			dst->append(ptr,uint32_t(pos-ptr));
            dst->append('&');
            dst->append(ent->name);
            dst->append(';');
            found = replaced = true;
			ptr = pos + 1;
		} else {
			if (ptr != str) {
				dst->append(ptr);
				ptr = dst->data();
				CBuffer* tmp = dst; dst = src; src = tmp;
				dst->bytes(0);
			}
			break;
		}
	}

    if (!m_entities.empty()) {
        CXmlEntities::iterator it = m_entities.begin();
        for (; it != m_entities.end(); it++) {
            std::string& val = it->second;
            uint32_t len = (uint32_t) val.length();
            bool found = false;
            const char* pos = strstr(ptr,val.c_str());
            while (pos) {
				dst->append(ptr,uint32_t(pos-ptr));
                dst->append('&');
                dst->append(it->first);
                dst->append(';');
                found = replaced = true;
                ptr = pos + len;
				pos = strstr(ptr,val.c_str());
				if (!pos) {
					dst->append(ptr);
					ptr = dst->data();
					CBuffer* tmp = dst; dst = src; src = tmp;
					dst->bytes(0);
				}
            }
		}
    }

	if (replaced)
		ret.append(src->data(),src->bytes());
	else
		ret.append(ptr);

    return replaced;
}

const char* CXmlDocType::getReplacement(const char *name,uint32_t& replacementLength) {
    // &#123; style entity..
    if (name[0]=='#') {
        if (isdigit(name[1])) {
            m_replacementBuffer[0] = (char)strtol(name+1, NULL, 10);
            m_replacementBuffer[1] = '\0';
            replacementLength = 1;
            return m_replacementBuffer;
        } else if (name[1]=='x' || name[1]=='X') {
            m_replacementBuffer[0] = (char)strtol(name+2, NULL, 16);
            m_replacementBuffer[1] = '\0';
            replacementLength = 1;
            return m_replacementBuffer;
        }
    }

    // Find in built-ins, see entities.h
    const struct entity *entity = xml_entities.find(name);
    if (entity) {
        replacementLength = entity->replacement_len;
        return entity->replacement;
    }

    // Find in custom attributes
    CXmlEntities::const_iterator itor = m_entities.find(name);
    if (itor != m_entities.end()) {
        const string& rep = itor->second;
        replacementLength = (uint32_t)rep.length();
        return rep.c_str();
    }

    return 0;
}

bool CXmlDocType::hasEntity(const char *name) {
    uint32_t len;
    const char* tmp = getReplacement(name,len);
    return tmp != NULL;
}
