/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CXmlNode.cpp  -  description
                             -------------------
    begin                : Sun May 22 2003
    based on the code    : Mikko Lahteenmaki <Laza@Flashmail.com>
    copyright            : (C) 2003-2007 by Alexey S.Parshin
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
   This library is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at
   your option) any later version.

   This library is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
   General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.

   Please report all bugs and problems to "alexeyp@gmail.com"
 ***************************************************************************/

#include <sptk3/CException.h>
#include <sptk3/cxml>
#include <algorithm>

using namespace std;
using namespace sptk;

const string CXmlNode::emptyString;
const string CXmlText::nodeNameString("#text");
const string CXmlComment::nodeNameString("#comment");
const string CXmlCDataSection::nodeNameString("#cdata-section");

CXmlNodeList CXmlNode::emptyNodes;

void CXmlNode::parent(CXmlNode *p) {
    if (m_parent == p) return;
    if (m_parent)
        m_parent->unlink(p);

    m_parent = p;

    if (m_parent)
        m_parent->push_back(this);
}

static void parsePathElement(CXmlDoc* document,const string& pathElementStr,CXPathElement& pathElement) {
    pathElement.elementName = NULL;
    pathElement.attributeName = NULL;
    pathElement.axis = XPA_CHILD;
    size_t backBracketPosition = pathElementStr.rfind("]");
    string pathElementName;
    if (backBracketPosition == string::npos) {
        pathElementName = pathElementStr;
        pathElement.criteria.clear();
    } else {
        size_t bracketPosition = pathElementStr.find("[");
        if (bracketPosition == string::npos || backBracketPosition < bracketPosition) {
            pathElementName = pathElementStr;
            pathElement.criteria.clear();
        } else {
            pathElementName = pathElementStr.substr(0,bracketPosition);
            pathElement.criteria = pathElementStr.substr(bracketPosition+1,backBracketPosition-bracketPosition-1);
        }
    }

    size_t pos = pathElementName.find("::");
    if (pos != string::npos) {
        if (pos == 10 && pathElementName.compare(0,12,"descendant::") == 0)
            pathElement.axis = XPA_DESCENDANT;
        else
            if (pos == 6 && pathElementName.compare(0,8,"parent::") == 0)
                pathElement.axis = XPA_DESCENDANT;
        pathElementName.erase(0,pos+2);
    }

    if (pathElementName[0] == '@') 
        pathElement.attributeName = &document->shareString(pathElementName.c_str()+1);
    else
        pathElement.elementName = &document->shareString(pathElementName.c_str());

    string& criteria  = pathElement.criteria;
    int& nodePosition = pathElement.nodePosition;

    if (!criteria.empty()) {
        pathElement.nodePosition = string2int(pathElement.criteria);
        if (!nodePosition && criteria == "last()")
            nodePosition = -1;

        if (nodePosition == 0) {
            if (criteria[0] == '@') {
                size_t pos = criteria.find("=");
                if (pos == string::npos)
                    pathElement.attributeName = &document->shareString(criteria.c_str() + 1);
                else {
                    pathElement.attributeName = &document->shareString(criteria.substr(1,pos-1));
                    if (criteria[pos+1] == '\'' || criteria[pos+1] == '"')
                        pathElement.attributeValue = criteria.substr(pos+2,criteria.length()-(pos+3));
                    else
                        pathElement.attributeValue = criteria.substr(pos+1,criteria.length()-(pos+1));
                    pathElement.attributeValueDefined = true;
                }
            }
        }
    }
}

bool CXmlNode::matchPathElement(const CXPathElement& pathElement,int nodePosition,const string* starPointer,bool& nameMatches,bool& positionMatches) {
    if (pathElement.elementName && pathElement.elementName != starPointer && !nameIs(pathElement.elementName))
        return false;

    // Node criteria is position
    if (pathElement.nodePosition) {
        positionMatches = pathElement.nodePosition == nodePosition;
        return positionMatches;
    }
    // Node criteria is attribute
    else if (pathElement.attributeName && type() == DOM_ELEMENT) {
        CXmlAttributes& attributes = this->attributes();
        bool attributeMatch = false;
        if (pathElement.attributeValueDefined) {
            if (pathElement.attributeName == starPointer) {
                for (CXmlAttributes::const_iterator attr = attributes.begin(); attr != attributes.end(); attr++) {
                    CXmlNode* a = *attr;
                    if (a->name() == pathElement.attributeValue) {
                        attributeMatch = true;
                        break;
                    }
                }
            } else 
                attributeMatch = attributes.getAttribute(*pathElement.attributeName) == pathElement.attributeValue;
        } else {
            if (pathElement.attributeName == starPointer) 
                attributeMatch = hasAttributes();
            else
                attributeMatch = hasAttribute(pathElement.attributeName->c_str());
        }
        return attributeMatch;
    } else
        return true;
    return false;
}

void CXmlNode::scanDescendents(CXmlNodeVector& nodes,const std::vector<CXPathElement>& pathElements,int pathPosition,const std::string* starPointer) {
    const CXPathElement& pathElement = pathElements[pathPosition];
    CXmlNode* lastNode = 0;
    int currentPosition = 1;
    for (iterator itor = begin(); itor != end(); itor++) {
        CXmlNode* node = *itor;
        bool nameMatches;
        bool positionMatches;
        if (node->matchPathElement(pathElement,currentPosition,starPointer,nameMatches,positionMatches)) {
            currentPosition++;
            node->matchNode(nodes,pathElements,pathPosition,starPointer);
        } else {
            if (nameMatches)
                currentPosition++;
            if (pathElement.nodePosition < 0 && nameMatches)
                lastNode = node;
        }
        if (node->type() & (DOM_DOCUMENT|DOM_ELEMENT))
            node->scanDescendents(nodes,pathElements,pathPosition,starPointer);
    }
    if (lastNode)
        lastNode->matchNode(nodes,pathElements,pathPosition,starPointer);
}

void CXmlNode::matchNode(CXmlNodeVector& nodes,const vector<CXPathElement>& pathElements,int pathPosition,const std::string* starPointer) {
    pathPosition++;
    if (pathPosition == (int) pathElements.size()) {
        const CXPathElement& pathElement = pathElements[pathPosition - 1];
        if (pathElement.elementName)
            nodes.insert(nodes.end(),this);
        else if (pathElement.attributeName) {
            CXmlAttribute* attributeNode = attributes().getAttributeNode(*pathElement.attributeName);
            if (attributeNode)
                nodes.insert(nodes.end(),attributeNode);
        }
        return;
    }

    const CXPathElement& pathElement = pathElements[pathPosition];

    CXmlNode* lastNode = 0;
    int currentPosition = 1;
    for (iterator itor = begin(); itor != end(); itor++) {
        CXmlNode* node = *itor;
        bool nameMatches;
        bool positionMatches;
        string nodeName = node->name();
        if (node->matchPathElement(pathElement,currentPosition,starPointer,nameMatches,positionMatches)) {
            currentPosition++;
            node->matchNode(nodes,pathElements,pathPosition,starPointer);
        } else if (pathElement.nodePosition < 0 && nameMatches)
            lastNode = node;
        if (pathElement.axis == XPA_DESCENDANT)
            node->scanDescendents(nodes,pathElements,pathPosition,starPointer);
    }
    if (lastNode)
        lastNode->matchNode(nodes,pathElements,pathPosition,starPointer);
}

void CXmlNode::select(CXmlNodeVector& nodes,string xpath) {
    const char* ptr;
    CXmlNode*   baseNode;
    nodes.clear();

    xpath = replaceAll(xpath,"//","/descendant::");

    if (xpath[0] == '/') {
        baseNode = document();
        ptr = xpath.c_str() + 1;
    } else {
        baseNode = parent();
        ptr = xpath.c_str();
    }

    CStrings pathElementStrs(ptr,"/");
    vector<CXPathElement>   pathElements(pathElementStrs.size());
    for (unsigned i = 0; i < pathElements.size(); i++)
        parsePathElement(document(),pathElementStrs[i],pathElements[i]);

    const string* starPointer = &document()->shareString("*");
    matchNode(nodes,pathElements,-1,starPointer);
}

void CXmlNode::copy(const CXmlNode& node) {
    name(node.name());
    value(node.value());
    if (isElement() && node.isElement()) {
        attributes() = node.attributes();
    }

    const_iterator itor = node.begin();
    const_iterator iend = node.end();
    for (; itor != iend; itor++) {
        const CXmlNode* node = *itor;
        switch (node->type()) {
        case DOM_ELEMENT: {
            CXmlNode* element = new CXmlElement(this,"");
            element->copy(*node);
        }
        break;
        case DOM_PI:
            new CXmlPI(*this,node->name(),node->value());
            break;
        case DOM_TEXT:
            new CXmlText(*this,node->value());
            break;
        case DOM_CDATA_SECTION:
            new CXmlCDataSection(*this,node->value());
            break;
        case DOM_COMMENT:
            new CXmlComment(*this,node->value());
            break;
        default:
            break;
        }
    }
}

std::string CXmlNode::text() const {
    std::string ret;

    if (type() & (DOM_TEXT|DOM_CDATA_SECTION))
        ret += value();
    else {
        const_iterator itor = begin();
        const_iterator iend = end();
        for (; itor != iend; itor++ ) {
            CXmlNode *np = *itor;
            if (np->type() & (DOM_TEXT|DOM_CDATA_SECTION))
                ret += np->value();
        }
    }

    return ret;
}

void CXmlNode::text(std::string txt) {
    clear();
    new CXmlText(*this,txt);
}

void CXmlNode::save(CBuffer &buffer, int indent) const {
    // output indendation spaces
    if (indent > 0)
        buffer.append(CXmlDoc::indentsString.c_str(),indent);

    if (type() == DOM_ELEMENT) {
        // Output tag name
        buffer.append('<');
        buffer.append(name());
        const CXmlAttributes& attributes = this->attributes();
        if (attributes.size()) {
            // Output attributes
            CBuffer real_id, real_val;
            for (CXmlAttributes::const_iterator it = attributes.begin(); 
                it != attributes.end(); it++) 
            {
                CXmlNode* attributeNode = *it;
                real_id.bytes(0);
                real_val.bytes(0);
                if (!document()->docType().encodeEntities(attributeNode->name().c_str(), real_id))
                    real_id = attributeNode->name();
                if (!document()->docType().encodeEntities(attributeNode->value().c_str(), real_val))
                    real_val = attributeNode->value();

                buffer.append(' ');
                buffer.append(real_id);
                buffer.append("=\"");
                buffer.append(real_val);
                buffer.append("\"");
            }
        }
    }

    // depending on the nodetype, do output
    switch (type()) {
    case DOM_PI:
        buffer.append("<?" + std::string(name()) + " " + value() + "?>\n");
        break;

    case DOM_TEXT:
        document()->docType().encodeEntities(value().c_str(), buffer);
        break;

    case DOM_CDATA_SECTION:
        // output all subnodes
        buffer.append("<![CDATA[" + value() + "]]>\n");
        break;

    case DOM_COMMENT:
        // output all subnodes
        buffer.append("<!-- " + value() + " -->\n");
        break;

    case DOM_ELEMENT: {
        if (size()) {
            bool only_cdata;
            const_iterator itor = begin();
            const_iterator iend = end();
            CXmlNode *nd = *itor;
            if (size()==1 && nd->type() == DOM_TEXT) {
                only_cdata = true;
                buffer.append('>');
            } else {
                only_cdata = false;
                buffer.append(">\n",2);
            }

            // output all subnodes
            for (; itor != iend; itor++) {
                CXmlNode *np = *itor;
                if (only_cdata)
                    np->save(buffer, -1);
                else {
                    np->save(buffer, indent + CXmlDoc::indentSpaces());
                    if (buffer.data()[buffer.bytes()-1] != '\n')
                        buffer.append('\n');
                }
            }
            // output indendation spaces
            if (!only_cdata && indent > 0)
                buffer.append(CXmlDoc::indentsString.c_str(),indent);

            // output closing tag
            buffer.append("</",2);
            buffer.append(name());
            buffer.append(">\n",2);

        } else {
            //LEAF
            buffer.append("/>\n",3);
        }
    }
    break;

    default: // unknown nodetype
        break;
    }
}

CXmlNode *CXmlNode::findFirst(std::string aname,bool recursively) const {
    for (const_iterator itor = begin(); itor != end(); itor++) {
        CXmlNode *node = *itor;
        if (node->name() == aname)
            return node;
        if (recursively && node->size()) {
            CXmlNode *cnode = node->findFirst(aname,true);
            if (cnode) return cnode;
        }
    }
    return 0L;
}

CXmlNode *CXmlNode::findOrCreate(std::string aname,bool recursively) {
    CXmlNode *node = findFirst(aname,recursively);
    if (node) return node;
    return new CXmlElement(*this,aname);
}

void CXmlNamedItem::name(const std::string& name) {
    m_name = &document()->shareString(name.c_str());
}

void CXmlNamedItem::name(const char *name) {
    m_name = &document()->shareString(name);
}

void CXmlElement::insert(iterator itor,CXmlNode* node) {
    m_nodes.insert(itor,node);
    node->m_parent = this;
}

void CXmlElement::push_back(CXmlNode* node) {
    m_nodes.insert(m_nodes.end(),node);
    node->m_parent = this;
}

void CXmlElement::unlink(CXmlNode* node) {
    iterator itor = find(begin(),end(),node);
    if (itor == end()) return;
    m_nodes.erase(itor);
}

void CXmlElement::remove(CXmlNode* node) {
    iterator itor = find(begin(),end(),node);
    if (itor == end()) return;
    delete *itor;
    m_nodes.erase(itor);
}

void CXmlElement::clear() {
    CXmlNode::clear();
    m_nodes.clear();
    m_attributes.clear();
}

void CXmlPI::name(const std::string& name) {
    m_name = &document()->shareString(name.c_str());
}

void CXmlPI::name(const char *name) {
    m_name = &document()->shareString(name);
}

