/***************************************************************************
                          SIMPLY POWERFUL TOOLKIT (SPTK)
                          CHttpParams.h  -  description
                             -------------------
    begin                : July 22 2003
    copyright            : (C) 2003 by Alexey Parshin. All rights reserved.
    email                : alexeyp@gmail.com
 ***************************************************************************/

/***************************************************************************
  Redistribution and use in source and binary forms, with or without 
  modification, are permitted provided that the following conditions are met:

  * Redistributions of source code must retain the above copyright notice, 
    this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright notice, 
    this list of conditions and the following disclaimer in the documentation 
    and/or other materials provided with the distribution.
  * Neither the name of the <ORGANIZATION> nor the names of its contributors 
    may be used to endorse or promote products derived from this software 
    without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
  OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef __CHTTPPARAMS_H__
#define __CHTTPPARAMS_H__

#include <sptk3/sptk.h>
#include <sptk3/CBuffer.h>
#include <sptk3/istring.h>

#include <string>
#include <map>

namespace sptk {

/// @addtogroup utility Utility Classes
/// @{

/// HTTP fields are implemented as case-insensitive map
typedef std::map<istring,std::string> CStringHttpFieldMap;

/// @brief HTTP params map
///
/// Designed to hold HTTP parametrs in
/// CHttpConnect and CCgiApplication. It is, basically, a string-to-string
/// map with an addition of encode and decode functions for HTTP Mime.
class CHttpParams : public CStringHttpFieldMap {
    /// Encodes a string into HTML parameters
    static std::string encodeString(std::string& str);

    /// Decodes a string from HTML parameters
    static std::string decodeString(std::string& str);
public:
    /// Default constructor.
    CHttpParams() : CStringHttpFieldMap() {}

    /// Encodes HTTP parameters for sending to the server.
    /// @param result CBuffer&, output - encoded parameters string (if any) as the buffer.
    void encode(CBuffer& result);

    /// Decodes HTTP parameters that came from the server as a string
    /// into parameters map.
    /// @param paramString CBuffer, parameters string from HTTP server
    /// @param lowerCaseNames bool, true if you want to lower-case the parameter names
    void decode(const CBuffer& paramString,bool lowerCaseNames=false);
};
/// @}
}
#endif
